c ACDEPTH.FOR
c  Werner deconvolution method by Walt Anderson and Lin Cordell, USGS
c--mint2a-- magnetic (profile) interpretation/version 2 --  11/02/73.
c revised by r.godson,usgs,denver,colorado 4/78
c  *** uses 'werner deconvolution' on airborne profiles where
c  input--10--optional standard airborne format which may contain channels
c     for total field,horz.derivative, filtered, etc --- see ichn parm.
c  note: the indep. var 'time' must be previously interpolated to equal
c     'dist'--however, the notation below uses time for dist, because of
c     its position in the data record (e.g., deltx used for delx, etc)
c
c    input may also be a standard grid file or file containing
c    user formatted data
c    output--plot-- depth solution plots--the number of
c                      plots depends on time selected segments,etc.
c
c &parms: (via namelist/parms/ on file--file09)
c
c     ichn  = selected channel for processing (default 1; max. 20).
c
c  for each analysis block:
c
c     n     = degree of interference polynomial (n.le.5; default n=2).
c             usually n=1 or 2 will suffice.
c       note: n+2 is the highest degree term in system of n+5 eqs.
c                 no interference (i.e. n=0) is a special case with 4eqs
c     eps   = accuracy or solution tolerance (default 1.0e-2)
c     iskip = initial no. pts to skip (default 1)
c     incr  = increment to skip pts for successive passes (default 1)
c     nskip = final no. pts to skip (default 1)
c     zmin  = min. z to accept as a depth solution (default 0.0)
c     zmax  = max. z to accept as a depth solution (default 10.0)
c              note: in plot z-axis ranges from zmin(top) to zmax(botm.)
c                    zmin & zmax are given in same units as x-axis.
c                    e.g: if x-axis in kilofeet, so is z-axis,etc.
c                    however the axis size may vary for z (see parm
c                    vexag); the x-axis size should be in scaled map
c                    inches (see parm xscale)
c     ipr  = 1 to print all solutions accepted.
c           = 0 (default) to suppress all solution printing.
c             (the plot will always contain all accepted solutions).
c      xscale = horizontal scale in data units per inch,
c      e.g.,if data units are inches xscale = 62500,250000,etc,
c      if data units are kilometers, xscale = 1.587,6.35,etc.
c     vexag = exageration of z axis(default 1.)
c     idplot= '40-char' id  for plot label (default 40 blanks)
c     iplotr= plotter number for evenden's plotting system(default=6)
c             or for a printer plot(9)
c     filnam= input segment name of data(default=blanks)
c     name= type of data -airborne or gridded(default=blanks)
c     if name ne airborne or gridded user formatted input
c      is assumed
c      ifmt= format of user input data e.g.'(10f8.0)'
c      npts= number of user formatted input data values
c      deltx= distance between user input data values
c     iprof= array containing row numbers of gridded data
c
c &
c
c
c--selected time segment (tm1,tm2,revx) cards follow &parms card(s)
c     revx = y to reverse axis(i.e. right to left)
c  all time cards in deck must be in ascending time order.. and segments
c  should be without time breaks.
c
c
          common/mintcom1/idplot,revx,sym,crt
          common/mintcom/zbuf,xdata,npts,iplotr,deltx,
     & xscale,vexag,delx,dely
          character*40 idplot
          character*80 xxx,ifmt
          character*50 fname,filnam,outfile
          character*8 date1,name
          dimension zbuf(3000),z(100,20),nscale(5)
          double precision id(2),rngxy(4),xx(100),yy(100),fil(10)
          dimension card(16),iprof(50),title(14),pgm(2),xdata(3000),
     & aa(3000),bb(3000),cc(3000),pp(3000),ss(3000)
          character*1 sym,star,revx,crt(27000)
          namelist/parms/ichn,n,eps,iskip,incr,nskip,zmin,zmax,
     & ipr,xscale,vexag,idplot,iplotr,filnam,ifmt,
     & name,iprof,npts,outfile,deltx
          double precision x,f,a(100),b(10),x1,xnn,xc,xtest,x2,xf,x0,z0
          data dval/1.0e38/,size/.08/,dumm/0./,it/0/,dd/0./,
     & icard/9/,iprint/6/,jprint/8/,ldim1/10001/
          in=10
          write(iprint,1000)
1000      format('$ enter command segment name: ')
          read(5,500) fname
          open(icard,file=fname,status='old')
2         ichn=1
          jsw=0
          n=2
          eps=1.0e-2
          iskip=1
          incr=1
          nskip=30
          zmin=0.0
          zmax=10.0
          xscale=6.35
          vexag=1.0
          npts=0
          iprof(1)=0
          deltx=0
          ipr=1
          iplotr=9
          sym='x'
          idplot=' '
          filnam=' '
          name=' '
          outfile=' '
          ifmt=' '
          revx='n'
          nsvl=10
          nshl=6
c
c--read &parms
c
          read(icard,parms,end=99)
          open(jprint,file=outfile,status='new')
          if(name.eq.'GRIDDED') name='gridded'
          if(name.eq.'AIRBORNE') name='airborne'
          if(xscale.le.0.) xscale=6.35
          if(vexag.le.0.) vexag=1.
          if(n.lt.0.or.n.gt.5) go to 9991
          if(n.eq.0) nn=4
          if(n.gt.0) nn=n+5
          nn1=nn-1
          nn2=nn-2
          if(name.eq.'gridded'.or.name.eq.'airborne')
     & open(in,file=filnam,status='old',form='unformatted')
          if(name.eq.'airborne') go to 11
          if(name.eq.'gridded') go to 3
c
c         user formatted input
c
          if(npts.le.0) go to 9996
          if(deltx.le.0) go to 9997
          if(deltx.gt.5000.) deltx=deltx/39370.1
          open(in,file=filnam,status='old')
          read(in,ifmt,end=9998) (zbuf(i),i=1,npts)
          xdata(1)=0.
          do 111 i=2,npts
          xdata(i)=xdata(i-1)+deltx
111       continue
          tm1=0.
          tmin=0.
          revx='n'
c
c         plot user formatted input data
c
          if(iplotr.ge.9) write(jprint,1050) filnam
          if(iplotr.lt.9.and.ipr.ne.0) write(jprint,1050) filnam
1050      format('1',11x,'input segment - ',a50,/)
112       call mintplot(1,zmin,zmax,ksw)
          call mintplot(2,zmin,zmax,ksw)
          go to 101
c
c         airborne data
c
11        read(in) id,maxobs,nchn,date1,deltx
          write(iprint,60) filnam,id,maxobs,nchn,date1,deltx
60        format('1input segment:',a50,//' id=',2a8/' maxobs=',i10/,
     &  ' nchn=',i10/ ' file date=',a8/' deltx=',e16.8)
          if(iplotr.ge.9) write(jprint,60) filnam,id,maxobs,
     & nchn,date1,deltx
          if(iplotr.lt.9.and.ipr.ne.0) write(jprint,60) filnam,id,
     & maxobs,nchn,date1,deltx
          if(ichn.lt.1.or.ichn.gt.nchn) go to 9993
          if(deltx.gt.5000.) deltx=deltx/39370.1
          read(icard,*,end=9992) tm1,tm2,irevx
          if(irevx.eq.0) revx='y'
500       format(a)
          write(iprint,550) tm1,tm2,revx
          if(iplotr.lt.9.and.ipr.ne.0)
     &  write(jprint,550) tm1,tm2,revx
550       format(/' selected time segment tm1,tm2=',2e16.8,5x,
     & 'revx=',a1)
          if(iplotr.ge.9) write(jprint,600) tm1,tm2,revx
600       format('1',11x,'selected time segment tm1,tm2=',2e16.8,5x,
     & 'revx=',a1,/)
          if(tm2.le.tm1) go to 9992
          tmin=tm1
          tmax=tm2
3         ieof=0
          isw=0
          k=0
8         il=1
          k=k+1
          if(isw.ne.0.and.name.eq.'airborne') go to 5
          if(name.eq.'airborne') go to 4
          if(isw.ne.0) go to 92
c
c         gridded data
c
          if(iprof(1).eq.0) go to 9995
          read(in,end=99) title,pgm,ncol,nrow,nz,xstart,deltx,yo,dy
          write(iprint,1450) filnam,title,pgm,nrow,ncol,
     & xstart,deltx,yo,dy
          if(iplotr.ge.9) write(jprint,1450) filnam,title,pgm,nrow,ncol,
     & xstart,deltx,yo,dy
          if(iplotr.lt.9.and.ipr.ne.0) write(jprint,1450) filnam,title,
     & pgm,nrow,ncol,xstart,deltx,yo,dy
1450      format('1input segment: ',a50,/,
     & ' title = ',14a4,/,' creation program id = ',2a4,/,
     & ' no. of rows = ',i3,3x,' no. of columns = ',i3,/,
     & ' first column coordinate = ',e15.7,/,
     & ' spacing of columns = ',e15.7,/,
     & ' first row coordinate = ',e15.7,/,
     & ' spacing of rows = ',e15.7,/)
          if(deltx.gt.5000.) xstart=xstart/39370.1
          if(deltx.gt.5000.) deltx=deltx/39370.1
          do 90 i=1,50
          if(iprof(i).eq.0) go to 91
90        continue
          i=51
91        nprof=i-1
          lk=0
92        read(in,end=99) dlt1,(zbuf(j),j=1,ncol)
          lk=lk+1
          if(lk.eq.iprof(k)) go to 93
          go to 92
93        j=0
          tm1=xstart
          do 994 i=1,ncol
          if(zbuf(i).ge.dval) go to 993
          j=j+1
          zbuf(j)=zbuf(i)
          xdata(j)=xstart+(i-1)*deltx
          go to 994
993       if(j.gt.0) go to 994
          tm1=xstart+(i*deltx)
994       continue
          tmin=tm1
          npts=j
          write(iprint,995) iprof(k)
          if(iplotr.lt.9.and.ipr.ne.0) write(jprint,995) iprof(k)
995       format(/,' processing row ',i5)
          if(iplotr.ge.9) write(jprint,996) iprof(k)
996       format('1',11x,'processing row ',i5,/)
c
c         plot input row of gridded data
c
997       call mintplot(1,zmin,zmax,ksw)
          call mintplot(2,zmin,zmax,ksw)
          go to 101
c
c         airborne data
c
4         ii=1
          read(in,end=100) line1,its,nobs,isid,rngxy,
     & (xx(i),yy(i),(z(i,j),j=1,nchn),i=1,nobs)
          if(isid.lt.0.and.il.gt.1) go to 9994
5         do 6 i=ii,nobs
          ilast=i
          t=(its+i-1)*deltx
          if(t.lt.tm1) go to 6
          if(t.gt.tm2) go to 20
          tlast=t
          zbuf(il)=z(i,ichn)
          xdata(il)=tm1+(il-1)*deltx
          il=il+1
          if(il.eq.ldim1.or.t.eq.tm2) go to 20
6         continue
          go to 4
100       ieof=1
          go to 9999
20        ii=ilast
          npts=il-1
          il2=il-2
          tmax=tmin+il2*deltx
c
c         plot airborne input data
c
201       call mintplot(1,zmin,zmax,ksw)
          call mintplot(2,zmin,zmax,ksw)
c
c--perform werner deconvolution for current zbuf block--
c
101       kskip=iskip
21        kat=1
          xtest=kskip*deltx
          if(ipr.eq.0) go to 22
          if(kskip.eq.iskip) go to 211
          if(ipr.ne.0) write(jprint,621) kskip,xtest
621       format(56x,'iskip=',i2,' xtest=',f9.3)
          go to 22
211       if(ipr.ne.0) write(jprint,62) kskip,xtest
62        format('1',10x,'x',12x,'xc',10x,'dist',9x,
     & 'depth',3x,'iskip=',i2,
     & ' xtest=',f9.3//)
22        j=kat
          x1=(kat-1)*deltx
          xnn=(kat-1+kskip*nn1)*deltx
          xc=0.5d0*(x1+xnn)
          i=1
23        x=(j-1)*deltx-xc
          if(zbuf(j).ge.dval) go to 50
          f=zbuf(j)
          a(i)=1.0d0
          a(i+nn)=x
          if(nn.eq.4) go to 400
          x2=x*x
          do 40 l=3,nn2
          a(i+(l-1)*nn)=x2
40        x2=x2*x
400       a(i+nn2*nn)=f
          xf=x*f
          a(i+nn1*nn)=xf
          b(i)=x*xf
41        i=i+1
          j=j+kskip
          if(i.le.nn) go to 23
          call dgelg(b,a,nn,1,eps,ks)
          if(ks.eq.-1) go to 50
          x0=0.5d0*b(nn)+xc
          if(x0.lt.0.0d0) go to 50
          x=x0-xc
          if(dabs(x).gt.xtest) go to 50
          z0=-b(nn1)-x**2
          if(z0.lt.0.0d0) go to 50
          z0=dsqrt(z0)
          if(z0.lt.zmin.or.z0.gt.zmax) go to 50
c--(x0,z0) accepted as a solution
          x0=tmin+x0
          if(ipr.eq.0) go to 42
          xc=tmin+xc
          x1=tmin+x1
          if(ipr.ne.0) write(jprint,64) x1,xc,x0,z0
64        format(1x,4(2x,f11.3))
42        xp=x0
          yp=z0
c
c         plot individual solution positions
c
          if(ksw.eq.1) call pplot(3,nscale,nhl,nshl,nvl,nsvl,
     & crt,xmaxp,xminp,ymaxp,yminp,sym,yp,xp,1,1,' ',0)
          if(ksw.eq.0) call pplot(3,nscale,nhl,nshl,nvl,nsvl,
     & crt,xmaxp,xminp,ymaxp,yminp,sym,xp,yp,1,1,' ',0)
          if(iplotr.lt.9) call vchar(xp, yp,sym,1,0,size,0.,0.,0.)
50        kat=kat+kskip
          if(kat.le.npts-nn*kskip+1) go to 22
51        kskip=kskip+incr
          if(kskip.le.nskip) go to 21
c
c--end of plot for this zbuf block--
c
          if(jsw.eq.1) go to 70
c
c         compute horizontal derivative and compute solutions
c
          call mintplot(3,zmin,zmax,ksw)
          call splin1(npts,deltx,dumm,zbuf,aa,bb,cc,it,dd,pp,ss)
          jsw=1
          tm1=tm1+deltx
          tmin=tm1
          npts=npts-1
          do 511 i=1,npts
          zbuf(i)=(3.*cc(i)*deltx+2.*bb(i))*deltx+aa(i)
          xdata(i)=xdata(i+1)
511       continue
          if(iplotr.ge.9) write(jprint,2000)
2000      format('1following plots and data result from horizontal',
     & ' derivative data')
          if(iplotr.lt.9.and.ipr.ne.0) write(jprint,2000)
          if(name.eq.'gridded') go to 997
          if(name.eq.'airborne') go to 201
          go to 112
70        jsw=0
          if(name.eq.'gridded') go to 43
          if(name.ne.'airborne') go to 98
          call mintplot(3,zmin,zmax,ksw)
          if(ieof.eq.1) go to 99
          if(t.lt.tm2) go to 71
c
c--next segment,if any of airborne data?
c
          revx='n'
          read(icard,*,end=99) t1,t2,irevx
          if(irevx.eq.0) revx='y'
          write(iprint,600) t1,t2,revx
          if(t2.le.t1.or.t1.lt.tm2) go to 9992
          tm1=t1
          tm2=t2
          tmin=tm1
          isw=0
          go to 8
71        tmin=tmax
          tm1=tmin
          isw=1
          go to 8
c
c         next row of gridded data,if any
c
43        call mintplot(3,zmin,zmax,ksw)
          if(k.eq.nprof) go to 99
          isw=1
          go to 8
98        call mintplot(3,zmin,zmax,ksw)
99        write(iprint,1100) outfile
1100      format('0output is in file ',a50)
998       close(jprint)
          close(icard)
          close(in)
          stop
c
c         messages
c
9991      write(iprint,69991)
69991     format('0error--n.lt.0.or.n.gt.5')
          go to 998
9992      write(iprint,69992)
69992     format('0error--tm2.le.tm1 .or. time segment cards not',
     & ' in ascending order')
          go to 998
9993      write(iprint,69993)
69993     format('0error--ichn.lt.1.or.ichn.gt.nchn')
          go to 998
9994      t=its*deltx
          write(iprint,69994) t,tlast
69994     format('0warning--time break at t=',e16.8,
     & '--segment end set to tm2=',e16.8/)
9999      tm2=tlast
          t=tm2+deltx
          go to 20
9995      write(iprint,1600)
1600      format(' no rows selected for gridded data')
          go to 998
9996      write(iprint,1700)
1700      format(' npts le 0 for user formatted data')
          go to 998
9997      write(iprint,1800)
1800      format(' deltx le 0 for user formatted data')
          go to 998
9998      write(iprint,1900)
1900      format(' end of file while reading user formatted data',/,
     & ' npts is probably incorrect')
          go to 998
          end
          subroutine mintplot(ic,ymin,ymax,ksw)
          common/mintcom1/idplot,revx,sym,crt
          common/mintcom/zbuf,xdata,npts,iplotr,deltx,
     & xscale,vexag,delx,dely
          character*1 revx,sym,crt(27000)
          character*10 ind
          character*40 idplot
          dimension xd(2),yd(2),xs(4),ys(4),jdim(1),
     & xdata(3000),zbuf(3000),ifmt1(2),nscale(5),
     & iddplot(10),indd(3)
          equivalence(iddplot(1),idplot),(indd(1),ind)
 
          data jdim(1)/0/,nscale/1,0,2,0,2/,ifmt1/'(f10','.1) '/,
     & ind/'input data'/,jprint/8/,nshl/6/,nsvl/10/,
     & nx/2/,ny/2/,iprint/6/
c
c         ic = 1 = input profile
c              2 = depth profile
c              3 = close plot system
c
          zmin=ymin
          zmax=ymax
          go to (10,20,70),ic
10        call minmax(zbuf,npts,zmin,zmax)
c
c         denver plotting system
c
20        if(iplotr.ge.9) go to 30
          if(ic.eq.2) go to 78
          ksw=2
          if(iplotr.eq.1.or.iplotr.eq.4) go to 75
          nxl=(xdata(npts)-xdata(1))/xscale*2.+.5
          if(nxl.lt.2) nxl=2
          nyl=nxl
          go to 77
75        nxl=24
          nyl=17
77        call scale3(xdata(1),xdata(npts),nxl,xminp,xmaxp,xdel)
          call scale3(zmin,zmax,nyl,yminp,ymaxp,ydel)
          xd(1)=xminp
          xd(2)=xmaxp
          yd(1)=yminp
          yd(2)=ymaxp
          xs(1)=(xmaxp-xminp)/xscale
          ys(1)=xs(1)*.5
          if(ys(1).gt.18.) ys(1)=18.
          go to 80
78        nyl=(zmax-zmin)/xscale*vexag*4.+.5
          if(nyl.lt.2) nyl=2
135       call scale3(zmin,zmax,nyl,yminp,ymaxp,ydel)
          yd(1)=ymaxp
          yd(2)=yminp
          xs(1)=(xmaxp-xminp)/xscale
          ys(1)=(ymaxp-yminp)/xscale*vexag
80        call pltset(iplotr,xs(4),ys(4),jdim)
          yyss=ys(4)
          xs(2)=0.
          ys(2)=0.
          xs(3)=2.
          ys(3)=2.
          if((xs(1)+xs(3)+.2).gt.xs(4)) go to 85
          if((ys(1)+ys(3)+.2).gt.ys(4)) go to 85
          go to 87
85        xs(4)=amin1(xs(4),13.)
          if(ic.eq.1) go to 86
          ys(4)=(ymaxp-yminp)/(xmaxp-xminp)*xs(4)*vexag+1.2
          if(ys(4).gt.yyss) ys(4)=yyss
          go to 88
86        ys(4)=amin1(ys(4),10.)
88        xs(3)=1.0
          ys(3)=1.0
          xs(1)=xs(4)-xs(3)-.2
          ys(1)=ys(4)-ys(3)-.2
          go to 89
87        xs(4)=xs(1)+xs(3)+.2
          ys(4)=ys(1)+ys(3)+.2
89        if(revx.ne.'y') go to 90
          xd(1)=xmaxp
          xd(2)=xminp
90        call scale(xd,yd,xs,ys,4,icode)
          if(icode.lt.0) go to 99
          call neatl
          call xaxis(xd,yd,xs,xdel,2,.08,'(f10.2)',10)
          call yaxis(yd,xd,ys,ydel,2,.08,'(f10.2)',10)
          if(ic.eq.2) return
c
c         input profile
c
          if(iplotr.ne.1.and.iplotr.ne.4)
     & call vchar(2.,.3,indd,10,3,.15,0.,0.,0.)
          if(iplotr.ne.1.and.iplotr.ne.4)
     & call vchar(2.,.1,iddplot,40,3,.15,0.,0.,0.)
          call line(xdata,zbuf,npts,0,0)
          call endpt(jdim)
          return
c
c         setup for regular printer plot
c
30        nvl=(xdata(npts)-xdata(1))/xscale+.5
          if(nvl.gt.12) go to 50
          ksw=0
          if(nvl.lt.2) nvl=2
          if(ic.eq.1) go to 40
c
c         depth profile
c
          call scale2(xdata(1),xdata(npts),nvl,xminp,xmaxp,d1)
          nvl1=(xmaxp-xminp)/nvl+.5
          nvl2=(zmax-zmin)/nvl1+.5
          if(nvl2.lt.1) nvl2=1
          nhl=nvl2*ifix(vexag)
          if(nhl.lt.2) nhl=2
          call scale2(zmin,zmax,nhl,yminp,ymaxp,d1)
          call pplot(1,nscale,nhl,nshl,nvl,nsvl,crt,xmaxp,xminp,ymaxp,yminp,
     & sym,xdata,zbuf,npts,1,' ',0)
          call pplot(2,nscale,nhl,nshl,nvl,nsvl,crt,
     & xmaxp,xminp,yminp,ymaxp,sym,xdata,zbuf,npts,1,' ',0)
          return
c
c         input profile
c
40        nhl=nvl/2
          if(nhl.lt.2) nhl=2
          write(jprint,1000) idplot
1000      format(/,12x,'input profile - ',a40,//)
          call graph2(npts,xdata,zbuf,0.,0.,'*',1,
     & nhl,nshl,nvl,nsvl,2,' ',0)
          return
c
c         setup for long printer plot
c
50        nhl=nvl
          ksw=1
          if(nhl.lt.2) nhl=2
          if(ic.eq.1) go to 60
c
c         depth profile
c
          call scale2(xdata(1),xdata(npts),nhl,yminp,ymaxp,d1)
          nhl1=(ymaxp-yminp)/nhl+.5
          nhl2=(zmax-zmin)/nhl1+.5
          if(nhl2.lt.1) nhl2=1
          nvl=nhl2*ifix(vexag)
          if(nvl.lt.2) nvl=2
          if(nvl.gt.12) nvl=12
          call scale2(zmin,zmax,nvl,xminp,xmaxp,d1)
          call pplot(1,nscale,nhl,nshl,nvl,nsvl,crt,
     & xmaxp,xminp,ymaxp,yminp,
     & sym,xdata,zbuf,npts,1,' ',0)
          call pplot(2,nscale,nhl,nshl,nvl,nsvl,crt,
     & xminp,xmaxp,yminp,ymaxp,sym,xdata,zbuf,npts,1,' ',0)
          return
c
c         input profile
c
60        nvl=nhl/2
          if(nvl.lt.2) nvl=2
          if(nvl.gt.12) nvl=12
          write(jprint,1000) idplot
          call graph2(npts,zbuf,xdata,0.,0.,'*',5,
     & nhl,nshl,nvl,nsvl,2,' ',0)
          return
70        if(iplotr.lt.9) call endpt(jdim)
          if(iplotr.ge.9) write(jprint,1100) idplot
1100      format('1',11x,'depth profile - ',a40,//)
          if(iplotr.ge.9) call pplot(4,nscale,nhl,nshl,nvl,nsvl,
     & crt,xmaxp,xminp,ymaxp,yminp,sym,xdata,zbuf,npts,1,' ',0)
          return
99        write(iprint,1200)
1200      format(' unable to scale plotter')
          stop
          end
      subroutine dgelg(r,a,m,n,eps,ier)
      dimension a(1),r(1)
      double precision r,a,piv,tb,tol,pivi
      if(m)23,23,1
c     search for greatest element in matrix a
1     ier=0
      piv=0.d0
      mm=m*m
      nm=n*m
      do 3 l=1,mm
      tb=dabs(a(l))
      if(tb-piv)3,3,2
2     piv=tb
      i=l
3     continue
      tol=eps*piv
c     a(i) is pivot element.piv contains the absolute value of a(i)
c
c     start elimination loop
      lst=1
      do 17 k=1,m
c     test on singularity
      if(piv)23,23,4
4     if(ier)7,5,7
5     if(piv-tol)6,6,7
6     ier=k-1
7     pivi=1.d0/a(i)
      j=(i-1)/m
      i=i-j*m-k
      j=j+1-k
c     i+k is row index,j+k is column index of pivot element
      do 8 l=k,nm,m
      ll=l+i
      tb=pivi*r(ll)
      r(ll)=r(l)
8     r(l)=tb
c     is elimination terminated
      if(k-m)9,18,18
c     column interchange in matrix a
9     lend=lst+m-k
      if(j)12,12,10
10    ii=j*m
      do 11 l=lst,lend
      tb=a(l)
      ll=l+ii
      a(l)=a(ll)
11    a(ll)=tb
c     row interchange and pivot row reduction in matrix a
12    do 13 l=lst,mm,m
      ll=l+i
      tb=pivi*a(ll)
      a(ll)=a(l)
13    a(l)=tb
c     save column interchangr information
      a(lst)=j
c     element reduction and next pivot search
      piv=0.d0
      lst=lst+1
      j=0
      do 16 ii=lst,lend
      pivi=-a(ii)
      ist=ii+m
      j=j+1
      do 15 l=ist,mm,m
      ll=l-j
      a(l)=a(l)+pivi*a(ll)
      tb=dabs(a(l))
      if(tb-piv)15,15,14
14    piv=tb
      i=l
15    continue
      do 16 l=k,nm,m
      ll=l+j
16    r(ll)=r(ll)+pivi*r(l)
17    lst=lst+m
c     end of elimination loop
c     back substitution and back interchange
18    if(m-1)23,22,19
19    ist=mm+m
      lst=m+1
      do 21 i=2,m
      ii=lst-i
      ist=ist-lst
      l=ist-m
      l=a(l)+.5d0
      do 21 j=ii,nm,m
      tb=r(j)
      ll=j
      do 20 k=ist,mm,m
      ll=ll+1
20    tb=tb-a(k)*r(ll)
      k=j+l
      r(j)=r(k)
21    r(k)=tb
22    return
c     error return
23    ier=-1
      return
      end
          subroutine splin1(m,h,x,y,a,b,c,t,d,p,s)                              
c--one dimensional cubic spline interpolation                                   
c                                                                               
c        by  w.l.anderson, u.s. geological survey,                              
c             denver, colorado                                                  
c                                                                               
c  parms--- m= number of data points .gt. 2                                     
c           h= equal interval option when h.gt.0.                               
c              (use dummy x here),                                              
c              unequal intervals if h=0. (x required storage)                   
c           x= indep.var when h=0. (dim .ge. m).                                
c           y= dependent variable  (dim .ge. m).                                
c           a,b,c=coeff.arrays (each dim .ge. m)                                
c                 results are returned in 1st(m-1) elements of                  
c                 a,b,&c.                                                       
c                 also used as work arrays during execution.                    
c           t= type of boundary condition supplied in d array.                  
c              use t=1 if 1st derivatives given at end points, or               
c              t=0 if 2nd derivatives given at end points.                      
c           d= boundary array (dim 2) at point 1 and m                          
c              respectively.                                                    
c           p,s= work arrays (each dim=m).                                      
c--error return with m=-(abs(m)) if any parm out of range.                      
c  the resulting cubic spline is of the form:                                   
c     y=y(i)+a(i)*(x-x(i))+b(i)*(x-x(i))**2+c(i)*(x-x(i))**3                    
c       for i=1,2,...,m-1                                                       
c                                                                               
          real   x(1),y(2),a(2),b(100),c(100),d(2),                             
     & p(100),s(100),mul                                                   
          integer  t                                                            
          if(t.lt.0.or.t.gt.1.or.m.lt.3) go to 999                              
          n=m-1                                                                 
          if(t.eq.0) go to 20                                                   
c--1st derivative boundaries given                                              
          ne=n-1                                                                
          if(h) 1,11,1                                                          
c--equal spacing h .gt. 0. and t=1                                              
1         hh=3.0/h                                                              
          do 2 i=1,ne                                                           
          b(i)=4.0                                                              
          c(i)=1.0                                                              
          a(i)=1.0                                                              
2         p(i)=hh*(y(i+2)-y(i))                                                 
          p(1)=p(1)-d(1)                                                        
          p(ne)=p(ne)-d(2)                                                      
c--solution of tridiagonal matrix eq. of order ne                               
3         fa=1./b(1)                                                            
          c(1)=c(1)*fa                                                          
          p(1)=p(1)*fa                                                          
          do 4 i=2,ne                                                           
          mul=1.0/(b(i)-a(i)*c(i-1))                                            
          c(i)=mul*c(i)                                                         
4         p(i)=mul*(p(i)-a(i)*p(i-1))                                           
c--obtain spline coefficients                                                   
          a(ne+t)=p(ne)                                                         
          i=ne-1                                                                
5         a(i+t)=p(i)-c(i)*a(i+t+1)                                             
          i=i-1                                                                 
           if(i.ge.1) go to 5                                                   
          if(t.eq.0) go to 6                                                    
          a(1)=d(1)                                                             
          a(m)=d(2)                                                             
6         if(h.eq.0.) go to 14                                                  
          hh=1.0/h                                                              
          do 7 i=1,n                                                            
          mul=hh*(y(i+1)-y(i))                                                  
          b(i)=hh*(3.0*mul-(a(i+1)+2.0*a(i)))                                   
7         c(i)=hh*hh*(-2.0*mul+a(i+1)+a(i))                                     
          return                                                                
c--unequal spacing h=0.. and t=1                                                
11        do 12 i=1,n                                                           
12        s(i+1)=x(i+1)-x(i)                                                    
          do 13 i=1,ne                                                          
          b(i)=2.0*(s(i+1)+s(i+2))                                              
          c(i)=s(i+1)                                                           
          a(i)=s(i+2)                                                           
13        p(i)=3.0*(s(i+1)**2*(y(i+2)-y(i+1))+s(i+2)**2
     & *(y(i+1)-y(i)))/(s(i+1)*s(i+2))         
          p(1)=p(1)-s(3)*d(1)                                                   
          p(ne)=p(ne)-s(n)*d(2)                                                 
          go to 3                                                               
14        do 15 i=1,n                                                           
          hh=1.0/s(i+1)                                                         
          mul=(y(i+1)-y(i))*hh**2                                               
          b(i)=3.0*mul-(a(i+1)+2.0*a(i))*hh                                     
15        c(i)=-2.0*mul*hh+(a(i+1)+a(i))*hh**2                                  
          return                                                                
c--2nd derivative boundaries given                                              
20        ne=n+1                                                                
          if(h) 21,31,21                                                        
c--equal spacing h .gt. 0 and t=0                                               
21        hh=3.0/h                                                              
          do 22 i=2,n                                                           
          b(i)=4.0                                                              
          c(i)=1.0                                                              
          a(i)=1.0                                                              
22        p(i)=hh*(y(i+1)-y(i-1))                                               
          b(1)=2.0                                                              
          b(ne)=2.0                                                             
          c(1)=1.0                                                              
          c(ne)=1.0                                                             
          a(ne)=1.0                                                             
          p(1)=hh*(y(2)-y(1))-0.5*h*d(1)                                        
          p(ne)=hh*(y(m)-y(n))+0.5*h*d(2)                                       
          go to 3                                                               
c--unequal spacing h=0 and t=0                                                  
31        do 32 i=1,n                                                           
32        s(i+1)=x(i+1)-x(i)                                                    
          n1=n-1                                                                
          do 33 i=1,n1                                                          
          b(i+1)=2.0*(s(i+1)+s(i+2))                                            
          c(i+1)=s(i+1)                                                         
          a(i+1)=s(i+2)                                                         
33        p(i+1)=3.0*(s(i+1)**2*(y(i+2)-y(i+1))+s(i+2)**2
     & *(y(i+1)-y(i)))/(s(i+1)*s(i+2))         
          b(1)=2.0                                                              
          b(ne)=2.0                                                             
          c(1)=1.0                                                              
          c(ne)=1.0                                                             
          a(ne)=1.0                                                             
          p(1)=3.0*(y(2)-y(1))/s(2)-0.5*s(2)*d(1)                               
          p(ne)=3.0*(y(m)-y(n))/s(m)+0.5*s(m)*d(2)                              
          go to 3                                                               
999       m=-iabs(m)                                                            
          return                                                                
          end                                                                   
      subroutine graph2(m,x,y,xoff,yoff,sym,iplott,nhl,nshl,nvl,nsvl,
     & ndec,ytitle,nchy)
          save
 
c--general printer plots (with log options) on unit#8.
c  by w.l.anderson, u.s. geological survey, denver, colorado.
c  modified by r.godson,usgs,denver,colorado to use iplott=5,6 and 7
c--parameters:
c     m     = no.points given in x,y arrays..may be splined before call.
c     x     = abscissa array(dim.m). must be supplied and should be in
c             ascending order. neg.values ignored in log options only.
c     y     = ordinate array(dim.m) corresponding to x array. neg.values
c               ignored in logy options only.
c     xoff  = x-translation option. use 0.0 if x array not to be transl.
c     yoff  = y-translation option. use 0.0 if y array not to be transl.
c              note: xoff or yoff are 'added' at plot time only and
c                    does not distroy the x,y arrays.
c                  xoff,yoff are in 'linear' or 'powers of 10' units
c                  depending on iplot parameter--see below
c                    when iplott>0, xoff=yoff=0.0 should be used.
c     sym   = 'char.' to use in plotting curve. normally use a
c             symmetrical char., e.g: '*','x','o', etc.
c     iplott=  1 to initilize image area & plot (x,y) curve.
c           = -1 to plot (x,y) curve over previous image area.
c                i.e., after a previous iplott=1 has been used.
c           =  2 to initilize image area & plot (logx,y) curve.
c           = -2 to plot (logx,y) curve over previous image area.
c                i.e., after a previous iplott=2 has been used.
c           =  3 to initilize image area & plot (logx,logy) curve.
c             -3 to plot (logx,logy) curve over previous image area.
c                i.e., after a previous iplott=3 has been used.
c           =  +&- 5,6,7 are equal to +&- 1,2,3 except that
c                the axes are reversed to allow a long x axis down
c                the pages.
c                when iplott=+&-5,6 or 7 the x,y,xoff and yoff arguments
c                are interchanged on input;i.e. call graph2(m,y,x,yoff,xoff,...
c                      note:when iplott is positive,the(x,y) data & other plot
c                      parms are used to initilize the image area first
c                      when iplott is negative, the (x,y) data is simply
c                      plotted over last image & scale established--
c                      thus points off-scale will be ignore.
c     nhl   = no.horiz.line intervals..for y or logy axis labels & scale
c             nhl is fixed for iplott=1,2 cases (nhl.ge.1;max.10)
c            *nhl is max.no.log cycle intervals(powers of 10) desired
c             when iplott=3 (actual no.cycles should be .le.nhl)
c     nshl  = no.spaces between horiz.line intervals. e.g., nshl=6 for
c             about 1 inch /interval on line printer. no limit on
c             line printer but nshl*nhl should not exceed 3 pages per
c             graph (else array crt may need to be larger).
c             see note(6) below.
c     nvl   = no.vert.line intervals..for x or logx axis labels & scale
c             nvl is fixed for iplott=1 case (nvl.ge.1--see note(1)).
c            *nvl is max.no.log cycle intervals(powers of 10) desired
c             when iplott=2,3 (actual no.cycles should be .le.nvl)
c     nsvl  = no.spaces between vert.line intervals. e.g., nsvl=20 for
c             about 2 inches /interval on line printer.
c             see note(6) below.
c     ndec  = no.decimal places to use in axis labeling.
c             ndec.lt.7--normally ndec=2 is sufficient.
c     ytitle= 'max. of 40 char's for ytitle'
c              (or a dummy name if nchy=0)
c     nchy  = no.char's in ytitle (0<=nchy<=40)-- note: use
c              nchy=0 (default) to suppress ytitle
c            note:  xtitle my be obtained with a fortran write
c                  (see e.g. below under operation note)
c
c--subroutines called: minmax,scale2,& pplot (pseudo entry calls: plot1,
c                      plot2,plot3,plot4--see note(5) below).
c--note(1): xsize=(nsvl*nvl)/10 .le. 12.0 inches is a'prplot'restriction
c      (2): the no. log cycles is adjusted to the data range but should
c           not exceed nhl or nvl--else data is not plotted.
c      (3): when iplott=3, neg. y data and any 0.0 data is ignored.
c      (4): when iplott=2 or 3, neg. or x=0.0 data is ignored.
c      (5): ref.--see write-up on 'prplot' for more details.
c      (6): no checks are made on parms--so 'user beware' if subr graph
c           is called with 'bum' parms--$$--
c--operation note: line printer page & title control should be supplied
c                  by the user before and/or after call to subr graph.
c                  ===============================================
c              e.g:      write(8,61)
c                     61 format('1',10x,'graph title here'//)
c                        call graph2(m,x,y,x0,y0,'*',2,5,6,5,20,2,'y',1)
c                        write(8,62)
c                     62 format('0',10x,'graph xtitle here')
c                  ===============================================
c
c
      dimension x(1),y(1),crt(27000),n(5),p(100)
      character*1 sym,crt,ytitle(40)
      data n/1,0,2,0,2/,iprint/8/
      iplot=mod(iplott,4)
      iplt=iabs(iplot)
      if(iplot.lt.0) go to 1111
c--set up new image area (iplot.gt.0)
      mhl=nhl
      mvl=nvl
      n(3)=ndec
      n(5)=ndec
      call minmax(x,m,xmin,xmax)
      call minmax(y,m,ymin,ymax)
      if(iplot.gt.2) go to 10
      call scale2(ymin,ymax,nhl,yminp,ymaxp,d)
      if(iplot.gt.1) go to 20
      call scale2(xmin,xmax,nvl,xminp,xmaxp,d)
      call pplot(1,n,nhl,nshl,nvl,nsvl,crt,xmaxp,xminp,ymaxp,yminp,
     & sym,x,y,m,1,' ',0)
      if(iplott.eq.1) call pplot(2,n,nhl,nshl,nvl,nsvl,crt,
     & xmaxp,xminp,ymaxp,yminp,sym,x,y,m,1,' ',0)
      if(iplott.eq.5) call pplot(2,n,nhl,nshl,nvl,nsvl,crt,
     & xmaxp,xminp,yminp,ymaxp,sym,x,y,m,1,' ',0)
      go to 1111
   10 if(ymin.le.0.0) go to 12
   11 i=alog10(ymin)
      if(ymin.lt.10.**i) i=i-1
      ymin=10.**i
      yminp=i
      j=alog10(ymax)
      if(ymax.gt.10.**j) j=j+1
      if(j-i.ge.nhl) j=i+nhl
      mhl=j-i
      ymax=10.**j
      ymaxp=j
      go to 20
12    if(ymax.le.0.0) go to 9999
      y1=y(1)
      do 13 i=2,m
      y2=y(i)
      if(sign(1.0,y1).ne.sign(1.0,y2)) go to 14
13    y1=y2
      go to 9999
14    if(sign(1.0,y1).eq.1.0) ymin=y1
      if(sign(1.0,y2).eq.1.0) ymin=y2
      go to 11
   20 if(xmin.le.0.0) go to 22
   21 i=alog10(xmin)
      if(xmin.lt.10.**i) i=i-1
      xmin=10.**i
      emin=i
      j=alog10(xmax)
      if(xmax.gt.10.**j) j=j+1
      if(j-i.ge.nvl) j=i+nvl
      mvl=j-i
      xmax=10.**j
      emax=j
      go to 30
22    if(xmax.le.0.0) go to 9999
      do 23 i=2,m
      xmin=x(i)
      if(xmin.gt.0.0) go to 21
   23 continue
      go to 9999
   30 call pplot(1,n,mhl,nshl,mvl,nsvl,crt,emax,emin,ymaxp,yminp,
     & sym,x,y,m,1,' ',0)
      if(iplott.le.3)call pplot(2,n,mhl,nshl,mvl,nsvl,crt,
     & emax,emin,ymaxp,yminp,sym,x,y,m,1,' ',0)
      if(iplott.gt.3)call pplot(2,n,mhl,nshl,mvl,nsvl,crt,
     & emax,emin,yminp,ymaxp,sym,x,y,m,1,' ',0)
c--tick horiz. log lines
      xl=xmin
      xc=10.0*xl
      xd=xmin
      np=0
  105 xl=xl+xd
      if(xl.gt.xc) go to 106
      np=np+1
      if(np.gt.100) go to 9999
      p(np)=alog10(xl)
      go to 105
  106 xl=xc
      xc=10.0*xc
      if(xc.gt.xmax) go to 107
      xd=10.0*xd
      go to 105
  107 if(iplot.lt.3.) go to 110
      yp=yminp
  108 do 109 i=1,np
  109 call pplot(3,n,mhl,nshl,mvl,nsvl,crt,emax,emin,ymaxp,yminp,
     & '+',p(i),yp,1,1,' ',0)
      yp=yp+1.0
      if(yp.le.ymaxp) go to 108
      go to 113
  110 dy=(ymaxp-yminp)/float(nhl)
      yp=yminp
      j=0
 1101 do 112 i=1,np
  112 call pplot(3,n,mhl,nshl,mvl,nsvl,crt,emax,emin,ymaxp,yminp,
     & '+',p(i),yp,1,1,' ',0)
      j=j+1
      yp=yminp+j*dy
      if(yp.le.ymaxp) go to 1101
  113 if(iplot.lt.3) go to 1111
c--tick vert. log lines (iplot.ge.3)
      yl=ymin
      yc=10.0*yl
      yd=ymin
      np=0
  114 yl=yl+yd
      if(yl.gt.yc) go to 115
      np=np+1
      if(np.gt.100) go to 9999
      p(np)=alog10(yl)
      go to 114
  115 yl=yc
      yc=10.0*yc
      if(yc.gt.ymax) go to 116
      yd=10.0*yd
      go to 114
  116 xp=emin
  117 do 118 i=1,np
  118 call pplot(3,n,mhl,nshl,mvl,nsvl,crt,emax,emin,ymaxp,yminp,
     & '+',xp,p(i),1,1,' ',0)
      xp=xp+1.0
      if(xp.le.emax) go to 117
c--plot requested curve
 1111 do 2222 i=1,m
      xp=x(i)
      yp=y(i)
      if(iplt.eq.1) go to 1112
      if(xp.le.0.0) go to 2222
      xp=alog10(xp)
      if(iplt.eq.2) go to 1112
      if(yp.le.0.0) go to 2222
      yp=alog10(yp)
 1112 xp=xp+xoff
      yp=yp+yoff
      call pplot(3,n,mhl,nshl,mvl,nsvl,crt,emax,emin,ymaxp,yminp,
     & sym,xp,yp,1,1,' ',0)
 2222 continue
      if(iplt.eq.3) go to 2224
      call pplot(4,n,mhl,nshl,mvl,nsvl,crt,emax,emin,ymaxp,yminp,
     & sym,x,y,m,nchy,ytitle,0)
      if(iplt.gt.1) write(iprint,60)
   60 format(/11x,'powers of 10'/)
      go to 9999
 2224 call pplot(4,n,mhl,nshl,mvl,nsvl,crt,emax,emin,ymaxp,yminp,
     & sym,x,y,m,nchy,ytitle,0)
      write(iprint,60)
 9999 return
      end
      subroutine minmax(a,n,amin,amax)
      dimension a(1)
      amin=a(1)
      amax=amin
      do 1 i=2,n
      amin=amin1(amin,a(i))
      amax=amax1(amax,a(i))
    1 continue
      return
      end
      subroutine scale2(xmin,xmax,n,xminp,xmaxp,dist)
c--see comm. acm, vol.16, no.10, oct 1973, pp.639-640.
      dimension vint(5)
      data vint(1),vint(2),vint(3),vint(4),vint(5)/1.,2.,5.,10.,20./
      if(xmin.lt.xmax.and.n.gt.1) go to 10
      write(6,99999)
99999 format(34h improper input supplied to scale2)
      return
   10 del=.00002
      fn=n
      a=(xmax-xmin)/fn
      al=alog10(a)
      nal=al
      if(a.lt.1.) nal=nal-1
      b=a/10.**nal
      do 20 i=1,3
      if(b.lt.(vint(i)+del)) go to 30
   20 continue
      i=4
   30 dist=vint(i)*10.**nal
      fm1=xmin/dist
      m1=fm1
      if(fm1.lt.0.) m1=m1-1
      if(abs(float(m1)+1.-fm1).lt.del) m1=m1+1
      xminp=dist*float(m1)
      fm2=xmax/dist
      m2=fm2+1.
      if(fm2.lt.(-1.)) m2=m2-1
      if(abs(fm2+1.-float(m2)).lt.del) m2=m2-1
      xmaxp=dist*float(m2)
      np=m2-m1
      if(np.le.n) go to 40
      i=i+1
      go to 30
   40 nx=(n-np)/2
      xminp=xminp-float(nx)*dist
      xmaxp=xminp+float(n)*dist
      if(xminp.gt.xmin) xminp=xmin
      if(xmaxp.lt.xmax) xmaxp=xmax
      return
      end
      subroutine scale3(xmin,xmax,n,xminp,xmaxp,dist)
c--see comm. acm, vol.16, no.10, oct 1973, pp.639-640.
      dimension vint(5)
      data vint(1),vint(2),vint(3),vint(4),vint(5)/1.,2.,5.,10.,20./
      if(xmin.lt.xmax.and.n.gt.1) go to 10
      write(6,99999)
99999 format(34h improper input supplied to scale3)
      return
   10 del=.00002
      fn=n
      a=(xmax-xmin)/fn
      al=alog10(a)
      nal=al
      if(a.lt.1.) nal=nal-1
      b=a/10.**nal
      do 20 i=1,3
      if(b.lt.(vint(i)+del)) go to 30
   20 continue
      i=4
   30 dist=vint(i)*10.**nal
      fm1=xmin/dist
      m1=fm1
      if(fm1.lt.0.) m1=m1-1
      if(abs(float(m1)+1.-fm1).lt.del) m1=m1+1
      xminp=dist*float(m1)
      fm2=xmax/dist
      m2=fm2+1.
      if(fm2.lt.(-1.)) m2=m2-1
      if(abs(fm2+1.-float(m2)).lt.del) m2=m2-1
      xmaxp=dist*float(m2)
      np=m2-m1
      if(np.le.n) go to 40
      i=i+1
      go to 30
c
c         next 3 statements comented out by r.godson,usgs,denver,co.
c
c   40 nx=(n-np)/2
c      xminp=xminp-float(nx)*dist
c      xmaxp=xminp+float(n)*dist
40      if(xminp.gt.xmin) xminp=xmin
      if(xmaxp.lt.xmax) xmaxp=xmax
      return
      end
      subroutine pplot(it,nscale,nhl,nsbh,nvl,nsbv,image,xmax,
     & xmin,ymax,ymin,ch,x,y,n3,nl,label,lsw)
      save
c--original version called 'prplot' by
c  pete smidinger   summer 1966  math & comp br  gsfc  nasa
c--modified on 12/3/76 by w.l.anderson, usgs, denver
c  uses pseudo entries, etc...
c  and to make it work on the 'honeywell multics system'--
      dimension  nscale(5),abnos(26),x(1),y(1)
      character*1 nos(10),image(27000),ch,label(40),vc,hc,nc,bl,hf,hf1,
     & hf2,wl, for1(21),for3(20)
      logical kplot,kplot1,kplot2,knhor,kord,kabsc,kbotgl
      data nos/'0','1','2','3','4','5','6','7','8','9'/
      data hc,nc,bl,hf,hf1,hf2,vc/'-','+',' ','f','.',',','|'/
      data for1/'(','1','x',',','a','1',',','f','9','.',' ',',','1','x',
     & ',', '1','2','1','a','1',')'/
      data for3/'(','1','h','0',',','f',2*' ','.',' ',',',2*' ','f',
     & 3*' ', '.',' ',')'/
      data kplot1 /.false./, kplot2/.false./
      data  kabsc,kord,kbotgl /3*.false./,iprint/8/,imax/27000/
      go to (1,2,3,4,5),it
c
c--pseudo      entry plot1(nscale,nhl,nsbh,nvl,nsbv)
1     continue
      kplot1=.true.
      kplot2=.false.
125   nh=iabs(nhl)
      nsh=iabs(nsbh)
      nv=iabs(nvl)
      nsv=iabs(nsbv)
      nscl=nscale(1)
      if(nh*nsh*nv*nsv.ne.0) go to 128
      write (6,14)
14    format(t5,'some plot1 arg. illegally 0')
      kplot=.false.
      return
128   kplot=.true.
      if(nv.le.25) go to 126
      write (6,12)
      kplot=.false.
12    format(t5,'no. of vertical lines >25')
      return
126   continue
      nvm=nv-1
      nvp=nv+1
      ndh=nh*nsh
      ndhp=ndh+1
      ndv=nv*nsv
      ndvp=ndv+1
      nimg=(ndhp*ndvp)
      if(nimg.gt.imax) go to 60
      if(ndv.le.120) go to 130
      kplot=.false.
      write (6,11)
11    format(t5,'width of graph >121')
      return
60     kplot=.false.
      write(6,61)
61    format(' plot too large')
      return
130   continue
      if(nscl.eq.0) go to 70
      fsy=10.**nscale(2)
      fsx=10.**nscale(4)
      iy=min0(iabs(nscale(3)),7)+1
      ix=min0(iabs(nscale(5)),9)+1
      go to 75
70    fsy=1.
      fsx=1.
      iy=4
      ix=4
75    for1(11)=nos(iy)
      na=min0(ix,nsv)-1
      ns=na-min0(na,120-ndv)
      nb=12-ns+na
      i1=nb/10
      i2=nb-i1*10
      for3(7)=nos(i1+1)
      for3(8)=nos(i2+1)
      for3(10)=nos(na+1)
      if(nv.gt.0) go to 90
      do 80 j=11,19
80    for3(j)=bl
      go to 100
90    i1=nv/10
      i2=nv-i1*10
      for3(12)=nos(i1+1)
      for3(13)=nos(i2+1)
      i1=nsv/100
      i3=nsv-i1*100
      i2=i3/10
      i3=i3-i2*10
      for3(15)=nos(i1+1)
      for3(16)=nos(i2+1)
      for3(17)=nos(i3+1)
      for3(19)=for3(10)
100   if(kplot1) return
      kplot1=.true.
c
c--pseudo      entry plot2(image,xmax,xmin,ymax,ymin)
2     continue
      kplot2=.true.
      if(kplot1) go to 210
      nscl=0
      nh=5
      nsh=10
      nv=10
      nsv=10
      go to 128
210   continue
      if(.not.kplot)return
      ymx=ymax
      dh=(ymax-ymin)/float (ndh)
      dv=(xmax-xmin)/float(ndv)
      do 220 i=1,nvp
220   abnos(i)=(xmin+float((i-1)*nsv)*dv)*fsx
      do 225 i=1,nimg
225   image(i)=bl
      do 240 i=1,ndhp
      i2=i*ndvp
      i1=i2-ndv
      knhor=mod(i-1,nsh).ne.0
      if(knhor) go to 230
      do 228 j=i1,i2
228   image(j)=hc
230   continue
      do 240 j=i1,i2,nsv
      if(knhor) go to 235
      image(j)=nc
      go to 240
235   image(j)=vc
240   continue
      xmin1=xmin-dv/2.
      ymin1=ymin-dh/2.
      return
c
c--pseudo      entry plot3(ch,x,y,n3)
3     continue
300   if(kplot2) go to 312
301   write (6,13)
13    format(t5,'plot2 must be called')
      return
312   continue
      if(.not.kplot) return
      if(n3.gt.0) go to 314
      kplot=.false.
      write (6,15)
15    format(t5,'plot3, arg2 < 0')
      return
314   do 320 i=1,n3
      dum1=(x(i)-xmin1)/dv
      dum2=(y(i)-ymin1)/dh
      if(dum1.lt.0..or.dum2.lt.0.) go to 320
      if(dum1.ge.ndvp.or.dum2.ge.ndhp) go to 320
      nx=1+int(dum1)
      ny=1+int(dum2)
315   j=(ndhp-ny)*ndvp+nx
      image(j)=ch
320   continue
      return
c
c--pseudo      entry plot4(nl,label)
c--pseudo      entry fplot4(nl,label)
4     continue
      if(.not.kplot) return
      if(.not.kplot2) go to 301
      do 420 i=1,ndhp
      if(i.eq.ndhp.and.kbotgl) go to 420
      wl=bl
      if(i.le.nl) wl=label(i)
      i2=i*ndvp
      i1=i2-ndv
      if(mod(i-1,nsh).eq.0.and..not.kord) go to 410
      write (iprint,222) wl,(image(j),j=i1,i2)
222   format(1x,a1,10x,121a1)
      go to 420
410   continue
      ordno=(ymx-float(i-1)*dh)*fsy
      write (iprint,for1) wl,ordno,(image(j),j=i1,i2)
420   continue
      if(kabsc) go to 430
      write (iprint,for3) (abnos(j),j=1,nvp)
430   return
c
c--pseudo      entry omit(lsw)
5     continue
      kabsc=mod(lsw,2).eq.1
      kord=mod(lsw,4).ge.2
      kbotgl=lsw.ge.4
      return
      end
