c
c  --  PROGRAM BOUNDARY
c
c     A package of programs written by Rick Blakely, Bob Simpson, and
c  others (with inspiration from Lin Cordell, Tien Grauch, and Tom 
c  Hildenbrand) for locating edges of apparent magnetic or gravity 
c  sources.  See file BOUNDARY.HLP for further information.
c
      write(6,100)
  100 format(/,' BOUNDARY - VERSION 1',//,
     &         ' See file BOUNDARY.HLP for information',//)
    1 write(6,101)
  101 format(/,' Operation [0-4]:',//,
     &         '     0.  Information',/,
     &         '     1.  Transform grid to pseudogravity',/,
     &         '     2.  Calculate maximum horizontal gradient',/,
     &         '     3.  Locate maxima in grid',/,
     &         '     4.  Stop',/)
      ifunc=int_ask('  = ? ')
      if(ifunc.lt.0.or.ifunc.gt.4)call scold(*1)
      if(ifunc.eq.0)call info
      if(ifunc.eq.1)call pseudo
      if(ifunc.eq.2)call hgrad
      if(ifunc.eq.3)call maxima
      if(ifunc.eq.4)stop
      go to 1
      end
c-----------------------------------------------------------------------
      subroutine info
      character yn_ask*1,answer*1,line*77,flag*3
c
c  --  !! Change following directory to location of BOUNDARY.INF !!
c
      open(10,file='SHARE$DISK:[RESLO]BOUNDARY.INF',status='old',
     &      form='formatted',readonly)
    2 read(10,100,end=3)flag,line
  100 format(a3,a77)
      if(flag.eq.'999')then
         answer=yn_ask('More info [y/n] ? ')
         if(answer.eq.'n')go to 3
         else
            write(6,101)flag,line
  101       format(1x,a3,a77)
            end if
      go to 2
    3 close(10)
      return
      end
c------------------------------------------------------------------------
      subroutine maxima
c
c  --  subroutine MAXIMA scans a standard grid along columns and rows.
c      Maxima are found, unprojected, and, if they meet certain criteria,
c      are sent to a binary xyz file.  Programmed by Rick Blakely from
c      an earlier algorithm by Bob Simpson.
c
      common /label/ a(5000),b(5000),c(5000),x(50000),
     &               y(50000),zmax(50000)
      real*8 sum,sum2,avg,signum,sigden,sig
      character id*56,pgm*8,yn_ask*1,answer*1,char_ask*1,downshift*1
      data dval/'ffff7fff'x/,nmax/50000/
c
c  --  Open the input file (standard grid) and read the header record
c
      write(6,102)
  102 format(/,' LOCATION OF MAXIMA WITHIN A GRID',//)
      call getfile(10,'Filename of input grid = ? ','in',
     &             'unformatted')
      call rdheader(10,id,pgm,ncol,nrow,nz,col1,dcol,row1,drow,iproj,
     &              cmer,blat)
c
c  --  Fill arrays b and c with first two rows of grid
c
      call getrow(10,ncol,b)
      call getrow(10,ncol,c)
c
c  --  Initialize the projection routines
c
    8 write(6,105)
  105 format(
     & /,' Output is a binary xyz file.',/)
      answer=char_ask(
     &  ' x and y in cartesian or geodetic coordinates [g/c] ? ')
      answer=downshift(answer)
      if(answer.ne.'g'.and.answer.ne.'c')call scold(*8)
      if(answer.eq.'g')then
         if(iproj.eq.0)call getproj(iproj,cmer,blat)
         call projset(iproj,cmer,blat)
      else
         iproj=0
         end if
c
c  --  Determine level of significance
c
    6 write(6,100)
  100 format(
     & /, ' 1.  Significance levels = 1,2,3, and 4',
     & /, ' 2.       "         "    = 1',
     & /, ' 3.       "         "    = 2,3, and 4',
     & /, ' 4.       "         "    = 2',
     & /, ' 5.       "         "    = 3 and 4',
     & /, ' 6.       "         "    = 3',
     & /, ' 7.       "         "    = 4',/)
      leveltest=int_ask('Pick one (1 through 7) ? ')
      if(leveltest.lt.1.or.leveltest.gt.7)call scold(*6)
c
c  --  Scan one row at a time
c
      ntotal=0
      do 1 j=2,nrow-1
         do 2 i=1,ncol
            a(i)=b(i)
    2       b(i)=c(i)
         call getrow(10,ncol,c)
c
c  --  Call subroutine maximum at each grid intersection
c
         do 3 i=2,ncol-1
            if(b(i).gt.1.e38)go to 3
            call maximum(a(i-1),a(i),a(i+1),b(i-1),b(i),b(i+1),
     &                  c(i-1),c(i),c(i+1),xoff,yoff,ztest,level)
c
c  --  If significance level is satisfied, unproject the maxima and file away
c      for future use
c
            if(ipass(level,leveltest).eq.1)then
               ntotal=ntotal+1
               if(ntotal.gt.nmax)then
                  write(6,149)ntotal
  149             format(/,' ***Number of maxima = ',i5,
     &                     '...exceeds program limits')
                  stop
                  end if
               xmax=(i-1+xoff)*dcol+col1
               ymax=(j-1+yoff)*drow+row1
               zmax(ntotal)=ztest
               if(iproj.eq.0)then
                  x(ntotal)=xmax
                  y(ntotal)=ymax
               else
                  call projinv(xmax,ymax,x(ntotal),y(ntotal))
                  end if
               end if
    3       continue
    1    continue
c
c  --  Calculate and report some statistics
c
      sum=0.d0
      sum2=0.d0
      zmaxmax=-1.e38
      zmaxmin= 1.e38
      do 4 i=1,ntotal
         zmaxmax=amax1(zmaxmax,zmax(i))
         zmaxmin=amin1(zmaxmin,zmax(i))
         sum=sum+zmax(i)
    4    sum2=sum2+zmax(i)**2
      avg=sum/ntotal
      signum=float(ntotal)*sum2-sum**2
      sigden=float(ntotal)*float(ntotal-1)
      sig=sqrt(signum/sigden)
    7 write(6,101)ntotal,avg,sig,zmaxmax,zmaxmin
  101 format(
     & /,' The total set of maxima from this grid have the following',
     & /,' statistics:',//,
     &         ' Number = ',i7,/,
     &         ' Average = ',g11.4,/,
     &         ' Standard deviation = ',g11.4,/,
     &         ' Maximum = ',g11.4,/,
     &         ' Minimum = ',g11.4,/)
c
c  -- Determine the upper and lower thresholds
c
      write(6,104)
  104 format(/,' Specify range of maxima to be output...',/)
      threshlow=float_ask(' Lower limit = ? ')
      threshup=float_ask(' Upper limit = ? ')
      if(threshup.le.threshlow)call scold(*7)
      call getfile(11,'Filename for output list = ? ','out',
     &             'unformatted')
c
c  --  Send all maxima that lie between the thresholds to the ouput file
c
      nout=0
      do 5 i=1,ntotal
         if(zmax(i).ge.threshlow.and.zmax(i).
     &       lt.threshup)then
            nout=nout+1
            write(11)x(i),y(i),zmax(i)
            end if
    5    continue
      write(6,103)nout
  103 format(/,i10,' records written',/)
      close(11)
c
c  --  Go back for new thresholds
c
      answer=yn_ask(
     &   'Another output file with a new range (y/n) ? ')
      if(answer.eq.'y')go to 7
c
c  --  Terminate
c
      close(10)
      return
      end
c---------------------------------------------------------------------------
      subroutine maximum(a1,a2,a3,b1,b2,b3,c1,c2,c3,xoff,yoff,zmax,
     &                   level)
c
c     Subroutine maximum determines if point b2 is a maximum with respect
c  to its neighbors, a1,a2,a3,b1,b3,c1,c2, and c3.
c
      xoff=0.
      yoff=0.
      zmax=-1.e38
      level=0
      if(b2.gt.b1.and.b2.gt.b3)then
         level=level+1
         call interp(b1,b2,b3,dtest,ztest,ierr)
         if(ztest.gt.zmax)then
            zmax=ztest
            xoff=dtest
            yoff=0.
            end if
         end if
      if(b2.gt.a2.and.b2.gt.c2)then
         level=level+1
         call interp(a2,b2,c2,dtest,ztest,ierr)
         if(ztest.gt.zmax)then
            zmax=ztest
            xoff=0.
            yoff=dtest
            end if
         end if
      if(b2.gt.a1.and.b2.gt.c3)then
         level=level+1
         call interp(a1,b2,c3,dtest,ztest,ierr)
         if(ztest.gt.zmax)then
            zmax=ztest
            xoff=dtest*.707107
            yoff=dtest*.707107
            end if
         end if
      if(b2.gt.a3.and.b2.gt.c1)then
         level=level+1
         call interp(a3,b2,c1,dtest,ztest,ierr)
         if(ztest.gt.zmax)then
            zmax=ztest
            xoff=-dtest*.707107
            yoff=dtest*.707107
            end if
         end if
      return
      end
c---------------------------------------------------------------------------
      subroutine interp(y1,y2,y3,xmax,ymax,ierr)
c
c     Subroutine interp fits a parabola through ordinants y1,y2 and y3, assumed
c  to be evenly spaced, and reports the x and y coordinate of the parabola's
c  maximum.  Note...y1 must be .lt. y2 and y3 must be .lt. y2.
c
      ierr=0
      a=0.5*(y1-2.*y2+y3)
      b=0.5*(y3-y1)
      c=y2
      if(a.eq.0.)then
         ierr=1
         return
         end if
      xmax=-0.5*b/a
      ymax=a*xmax**2+b*xmax+c
      return
      end
c-------------------------------------------------------------------------
      subroutine getfile(unit,question,inout,form)
c      Opens files...filenames not recognized by VAX will generate
c      error message and request a second try.
c      Sample call:
c           call getfile(21,'*Give input file:','in','formatted')
      integer unit
      character filename*50, status*8
      character*(*) question, inout, form
c
      print *, ' '
 10   write(*,101) question
 101  format('$',a,' ')
      read '(a50)', filename
c
      if(inout.eq.'in') then
        status='old'
      else
        status='unknown'
      endif
      if(form.eq.'formatted'.and.inout.eq.'out') then
        open(unit,file=filename,form=form,status=status,
     &   carriagecontrol='list',err=900)
        return
        end if
      if(inout.eq.'in')then
        open(unit,file=filename,form=form,status=status,readonly,
     &    err=900)
        else
        open(unit,file=filename,form=form,status=status,err=900)
        endif
        return
c
 900  print *,'ERROR IN OPENING ',filename
      print *,'   try again......'
      close(unit)
      go to 10
c
      end
c----------------------------------------------------------------------
      subroutine rdheader(unit,id,pgm,ncol,nrow,nz,
     & xo,dx,yo,dy,iproj,cm,bl)
      character id*56, pgm*8
      integer unit
      read(unit,err=20) id,pgm,ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl
c      print '(1h0,a,i2,2(a,f10.4))',
c     &    '  Proj=',iproj,'  cm=',cm,'  bl=',bl
      return
 20   rewind unit
      read(unit) id,pgm,ncol,nrow,nz,xo,dx,yo,dy
c      print '(1h0,a)', '  Grid has no projection specs in header...'
      iproj=0
      cm=0.0
      bl=0.0
      return
      end
c-------------------------------------------------------------------------
      subroutine getrow(iounit,ncol,row)
      real row(ncol)
      read(iounit) dummy,row
      return
      end
c------------------------------------------------------------------------
      function yn_ask(request)
      character*1 yn_ask                      
      character*(*) request
    2 write(6,100)request
  100 format('$',a,' ')
      read(5,102,err=1)yn_ask
  102 format(a1)
      if(yn_ask.eq.'Y')yn_ask='y'
      if(yn_ask.eq.'N')yn_ask='n'
      if(yn_ask.ne.'y'.and.yn_ask.ne.'n')call scold(*2)
      return
    1 call scold(*2)
      end
c-------------------------------------------------------------------------
      subroutine bell
      character*1 ding
      ding=char(007)
      print *, ding
      return
      end
c-----------------------------------------------------------------------
      subroutine scold(*)
      call bell
      write(6,100)
  100 format(/,' *** Unacceptable response...Try again')
      return 1
      end
c----------------------------------------------------------------------
      function ipass(level,leveltest)
      ipass=0
      if(level.eq.0)return
      go to (1,2,3,4,5,6,7)leveltest
    1 if(level.ge.1)ipass=1
      return
    2 if(level.eq.1)ipass=1
      return
    3 if(level.ge.2)ipass=1
      return
    4 if(level.eq.2)ipass=1
      return
    5 if(level.ge.3)ipass=1
      return
    6 if(level.eq.3)ipass=1
      return
    7 if(level.ge.4)ipass=1
      return
      end
c----------------------------------------------------------------------------
      subroutine pseudo
c
c     Calculates the pseudogravity transformation.  Modified by Rick Blakely
c  from Bob Simpson's GFILTER.
c
      real finc,fdec,minc,mdec,az
      character id*56,pgm*8
      common/gridid/ id,pgm
      common/gridspecs/ ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl
      common/origspecs/ ncol0,nrow0
      complex grid(500000), fftgrid(500000)
      common/grid/ grid
      common/fftgrid/ fftgrid
      real kx(5000),ky(5000),kxsq(5000),kysq(5000)
      common/wavek/ kx,ky,kxsq,kysq
      data inunit/21/
c
      write(6,100)
  100 format(
     & /,' PSEUDOGRAVITY TRANSFORMATION',
     &//,' NOTE:  Automatic augmented dimensions are about 15% bigger.',
     & /,'        Shallow inclinations (<10?) may cause errors.',/)
c
c  --  Get grid to be filtered and read header
c
      call getfile(inunit,'Filename of input grid = ? ','in',
     &             'unformatted')
      call read_sfheader(inunit)
      if(ncol*nrow.gt.500000) then
        print *, ' GRID IS TOO BIG...ncol*nrow =',ncol*nrow
        print *, ' NCOL*NROW MUST BE .LE. 500000...'
        return
      endif
      ncol0=ncol
      nrow0=nrow
c
c  --  Read in grid, augment, and take fft...
      call get_newfft(inunit)
c
c  --  Calculate and store wavenumbers for fftfiltering...
c
 90   call wavenums(ncol,nrow,dx,dy)
c
c  --  Do the transformation
c
      finc=float_ask('Field inclination (degrees) = ? ')
      fdec=float_ask('Field declination = ? ')
      minc=float_ask('Magnetization inclination = ? ')
      mdec=float_ask('Magnetization declination = ? ')
      az=  float_ask
     &     ('Azimuth of grid relative to north (usually 0) = ? ')
         call get_units(ufactor)
         call psgrav(finc,fdec,minc,mdec,az,ufactor)
c
c  --  Inverse transform and output
c
         call output
c
c  --  Reset 'grid' to 'fftgrid' for further operations...
c
         do 185 ind=1,ncol*nrow
 185     grid(ind)=fftgrid(ind)
 900  return
      end
c-----------------------------------------------------------------------------
      subroutine get_newfft(inunit)
c
c  --  reads in grid, augments, and calculates the fftgrid
c
      dimension row(5000)
      complex crow(5000)
      dimension nn(2), work(5000)
      character ans*4, augans*1, yn_ask*1
      character id*56,pgm*8
      common/gridid/ id,pgm
      common/gridspecs/ ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl
      common/origspecs/ ncol0,nrow0
      complex grid(500000), fftgrid(500000)
      common/grid/ grid
      common/fftgrid/ fftgrid
      integer inunit,outunit
      data outunit/22/
      ij(i,j,ncol)=ncol*(j-1)+i
c**** read in body of grid
      do 30 j=1,nrow
      call row_read(inunit,ncol,row)
      do 30 i=1,ncol
 30   grid(ij(i,j,ncol))=cmplx(row(i),0.)
      close(inunit)
c**** check grid for dvals...stop if found.
      if(ndval(2*ncol*nrow,grid).ne.0) then
       print *,'ERROR...GRID HAS DVALS...FILL THEM IN!'
       stop
      endif
c**** augment grid to speed fft
   40 augans=yn_ask('Do you want automatic augmentation [y/n] ? ')
      if(augans.eq.'y') then
        call piknewdims(ncol0,nrow0,ncol,nrow,15.)
      else if(augans.eq.'n') then
        call fftdims(ncol0,nrow0)
      write(6,101)
  101 format(/,' Give aumented ncol and nrow ? '$)
        read *, ncol,nrow
      else
        goto 40
      endif
      write(6,1001) ncol,nrow
 1001 format(/,10x,'ncolp=',i8,'   nrowp=',i8,/)
      if(ncol*nrow.gt.500000) then
        print *, 'AUGMTD GRID IS TOO BIG...ncol*nrow =',ncol*nrow
        print *, 'NCOL*NROW MUST BE .LE. 500000...'
        return
      endif
      call cmplxaug(ncol0,nrow0,ncol,nrow,+1)
c**** take fft of augmented grid.
      nn(1)=ncol
      nn(2)=nrow
      call fourt(grid,nn,2,-1,0,work)
c**** Transfer to fftgrid to save for future operations...
      do 50 ind=1,ncol*nrow
 50   fftgrid(ind)=grid(ind)
 90   return
      end
c----------------------------------------------------------------------
      function ndval(num,array)
c      function value is number of dvals in the array of length num.
      real array(num)
      parameter (dval='ffff7fff'x, dval2=1.0E+38)
c      Values .ge. 1.0E+38 flag pts for which no data is available.
      ndval=0
      do 20 i=1,num
      if(array(i).ge.dval2) ndval=ndval+1
 20   continue
      return
      end
c-------------------------------------------------------------------------
      subroutine output
c
c  --  Takes inverse fft and outputs filtered standard file.
c
      dimension row(5000)
      dimension nn(2), work(5000)
      character ans*4
      character id*56,pgm*8
      common/gridid/ id,pgm
      common/gridspecs/ ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl
      common/origspecs/ ncol0,nrow0
      complex grid(500000)
      common/grid/ grid
      integer outunit
      data outunit/22/
      ij(i,j,ncol)=ncol*(j-1)+i
      nn(1)=ncol
      nn(2)=nrow
      call fourt(grid,nn,2,+1,1,work)
      do 10 ind=1,ncol*nrow
 10   grid(ind)=grid(ind)/(ncol*nrow)
c**** deaugment
      call cmplxaug(ncol0,nrow0,ncol,nrow,-1)
c**** output grid
      call getfile(outunit,'Filename of output grid = ? ',
     &             'out','unformatted')
      write(6,101)
  101 format(/,' New ID (56 characters or less) = ? '$)
      read '(a56)', id
      pgm='filter  '
      call wrheader(outunit,id,pgm,ncol0,nrow0,nz,xo,dx,yo,dy,
     &   iproj,cm,bl)
      do 50 j=1,nrow0
      do 40 i=1,ncol0
 40   row(i)=real(grid(ij(i,j,ncol0)))
      call row_write(outunit,ncol0,row)
 50   continue
      close(outunit)
      print *,' '
      return
      end
c-----------------------------------------------------------------------
      complex function phase(x,y,z,kx,ky,k)
      real kx,ky,k
      phase=cmplx(+z,(x*kx+y*ky)/k)
      return
      end
c------------------------------------------------------------------------
      subroutine psgrav(finc,fdec,minc,mdec,az,ufactor)
c
c     Calculates pseudograv. f=field direction, m=mag direction
c  density contrast= 0.1 g/cc; mag= 0.01 emu.  y=north,x=east,z=dn,
c  or... az= angle of y from north
c
      real jay,kmtocm
      real finc,fdec,minc,mdec,fx,fy,fz,mx,my,mz,inc,dec
      real k,ksq
      real kx(5000),ky(5000),kxsq(5000),kysq(5000)
      common/wavek/ kx,ky,kxsq,kysq
      complex phase,filt
      complex grid(500000)
      common/grid/ grid
      common/gridspecs/ ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl
      data rho/0.1/,jay/.001/,gbig/6.670E-8/
      data kmtocm/1.0E5/,gamtogauss/1.0E-5/,galtomgal/1.0E+3/
      ij(i,j,ncol)=ncol*(j-1)+i
      call cmpnts(finc,fdec-az,fx,fy,fz)
      call cmpnts(minc,mdec-az,mx,my,mz)
      const=gbig*rho*kmtocm*gamtogauss*galtomgal/jay
      do 100 j=1,nrow
      do 100 i=1,ncol
      k=sqrt(kxsq(i)+kysq(j))
      filt=cmplx(0.,0.)
      if(i.eq.1.and.j.eq.1) go to 100
c      this avoids the case k=0.
      filt=1./(phase(fx,fy,fz,kx(i),ky(j),k)*
     1            phase(mx,my,mz,kx(i),ky(j),k))
      filt=const*filt/(k*ufactor)
c      [units specified]/ufactor = km
  100 grid(ij(i,j,ncol))=filt*grid(ij(i,j,ncol))
      return
      end
c-------------------------------------------------------------------------
      subroutine cmpnts(inc,dec,x,y,z)
      real inc,dec
      data pi/3.1415927/,rad/1.745329E-2/
      x=cos(inc*rad)*sin(dec*rad)
      y=cos(inc*rad)*cos(dec*rad)
      z=sin(inc*rad)
      return
      end
c----------------------------------------------------------------------
      subroutine get_units(ufactor)
      character units*8
    1 write(6,100)
  100 format(/,' Units of grid dx and dy (km,m,kft,ft,mi) = ? '$)
      read '(a8)', units
      if(units.eq.'km'.or.units.eq.'KM')then
         ufactor=1.0
         return
         end if
      if(units.eq.'m'.or.units.eq.'M')then
         ufactor=1000.
         return
         end if
      if(units.eq.'kft'.or.units.eq.'KFT')then
         ufactor=3.2802
         return
         end if
      if(units.eq.'ft'.or.units.eq.'FT')then
         ufactor=3.2802e3
         return
         end if
      if(units.eq.'mi'.or.units.eq.'MI')then
         ufactor=0.6214
         return
         end if
      call scold(*1)
      return
      end
c----------------------------------------------------------------------------
      subroutine wavenums(ncol,nrow,dx,dy)
c      calculate wavenumbers to be used in filters
      real kx(5000),ky(5000),kxsq(5000),kysq(5000)
      common/wavek/ kx,ky,kxsq,kysq
      data pi/3.1415927/
      xscale=2.*pi/(ncol*dx)
      yscale=2.*pi/(nrow*dy)
      do 100 j=1,nrow
      jk=j-1
      if(jk.gt.nrow/2) jk=jk-nrow
      ky(j)=yscale*jk
 100  kysq(j)=ky(j)**2
      do 200 i=1,ncol
      ik=i-1
      if(ik.gt.ncol/2) ik=ik-ncol
      kx(i)=xscale*ik
 200  kxsq(i)=kx(i)**2
      return
      end
c--------------------------------------------------------------------------
      subroutine cmplxaug(ncol,nrow,ncolp,nrowp,idir)
c
c     Modified from a program written by R. Blakely.  Augments a 2d complex
c  grid stored in a linear array.  ncol,nrow = original dimensions;
c  ncolp,nrowp = augmented dimensions.  idir=+1 to augment, -1 to go back
c  to original size.  Grid is stored in common block grid row by row,
c  starting with bottom row.
c
      complex grid,first,last,diff
      common/grid/ grid(500000)
c
      if(ncolp.lt.ncol.or.nrowp.lt.nrow) go to 900
      if(ncolp.eq.ncol.and.nrowp.eq.nrow) go to 910
c
c     Deaugmenting steps:
      if(idir.ne.-1) go to 200
      do 110 j=1,nrow
      do 110 i=1,ncol
 110  grid(ncol*(j-1)+i)=grid(ncolp*(j-1)+i)
      return
c
c     Augmenting steps:
 200  if(idir.ne.+1) go to 920
 210  do 220 j=nrow,1,-1
      do 220 i=ncol,1,-1
 220  grid(ncolp*(j-1)+i)=grid(ncol*(j-1)+i)
c     add to rows:
      if(ncolp.eq.ncol) go to 235
        do 230 j=1,nrow
        first=grid(ncolp*(j-1)+1)
        last=grid(ncolp*(j-1)+ncol)
        diff=(first-last)/float(ncolp-ncol+1)
        do 230 i=ncol+1,ncolp
 230    grid(ncolp*(j-1)+i)=last+diff*float(i-ncol)
c     add to cols:
 235  if(nrowp.eq.nrow) go to 250
        do 240 i=1,ncolp
        first=grid(i)
        last=grid(ncolp*(nrow-1)+i)
        diff=(first-last)/float(nrowp-nrow+1)
        do 240 j=nrow+1,nrowp
 240    grid(ncolp*(j-1)+i)=last+diff*float(j-nrow)
 250  return
c
 900  print *,'***AUGMENTED GRID IS SMALLER THAN ORIGINAL!!!'
      return
 910  print *,'***AUGMENTED GRID IS SAME SIZE AS ORIGINAL!!!'
      return
 920  print *,' idir in subroutine aug must be +1 or -1!!!'
      return
      end
c-------------------------------------------------------------------------
      subroutine piknewdims(ncol0,nrow0,ncol,nrow,pct)
c
c     Returns new dimensions approx 'pct' percent larger than originals.
c  New dimensions should be fast for fft.
c  first 2 cols from table III of Singleton's paper on fft....
c  (Singleton,R.C., 1969, An algotithm for computing the mixed radix fast
c  Fourier transform:  IEEE Trans., Audio and Electro-acoustics: v. AU-17,
c  p. 93-103.
c
      dimension nlist(104),ngood(100)
      data nlist/2,3,4,5,6,8,9,10,12,15,16,18,20,24,25,27,30,
     1   32,36,40,45,48,50,54,60,64,72,75,80,81,90,96,100,108,120,
     2   125,128,135,144,150,160,162,180,192,200,216,225,240,
     3   243,250,256,270,288,300,320,324,360,375,384,400,405,432,450,
     4   480,486,500,512,540,576,600,625,640,648,675,720,729,750,
     5   768,800,810,864,900,960,972,1000,1024,1080,1125,1152,1200,
     6   1215,1250,1280,1296,1350,1440,1458,1500,1536,1600,1620,1728,
     7   1800,1875/
c
      ncol=(1.0+.01*pct)*ncol0
      nrow=(1.0+.01*pct)*nrow0
      do 50 i=1,104
      if(nlist(i).ge.ncol) ncol=nlist(i)
      if(nlist(i).eq.ncol) go to 55
 50   continue
 55   do 60 i=1,104
      if(nlist(i).ge.nrow) nrow=nlist(i)
      if(nlist(i).eq.nrow) go to 90
 60   continue
 90   if(ncol0.eq.1) ncol=1
      if(nrow0.eq.1) nrow=1
      return
      end
c------------------------------------------------------------------------------
      subroutine fftdims(ncol,nrow)
c     Prints out numbers less than 1024 with no prime factor greater than 5.
c  The FFT algorithm works fastest for these numbers.  Subroutine prints
c  out good numbers between min of (ncol,nrow) and max of (ncol+100,nrow+100).
c  first 2 cols from table III of Singleton's paper on fft....
c  (Singleton,R.C., 1969, An algotithm for computing the mixed radix fast
c  Fourier transform:  IEEE Trans., Audio and Electro-acoustics: v. AU-17,
c  p. 93-103.
c
      dimension nlist(104),ngood(100)
      data nlist/2,3,4,5,6,8,9,10,12,15,16,18,20,24,25,27,30,
     1   32,36,40,45,48,50,54,60,64,72,75,80,81,90,96,100,108,120,
     2   125,128,135,144,150,160,162,180,192,200,216,225,240,
     3   243,250,256,270,288,300,320,324,360,375,384,400,405,432,450,
     4   480,486,500,512,540,576,600,625,640,648,675,720,729,750,
     5   768,800,810,864,900,960,972,1000,1024,1080,1125,1152,1200,
     6   1215,1250,1280,1296,1350,1440,1458,1500,1536,1600,1620,1728,
     7   1800,1875/
c
      min=min0(ncol,nrow)
      max=max0(ncol+100,nrow+100)
      if(min.eq.1) min=max0(ncol,nrow)
      n=0
      do 10 i=1,104
      if(nlist(i).lt.min) go to 10
      if(nlist(i).gt.max) go to 20
      n=n+1
      ngood(n)=nlist(i)
 10   continue
c
 20   print *,'    *Good fft dimensions:'
      write(6,101) (ngood(i),i=1,n)
 101  format('      ',14i5)
c
      return
      end
c-----------------------------------------------------------------------------
      subroutine read_sfheader(iunit)
      character id*56,pgm*8
      common/gridid/ id,pgm
      common/gridspecs/ ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl
      call rdheader(iunit,id,pgm,ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl)
      call prheader(6,id,pgm,ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl)
      return
      end
c------------------------------------------------------------------------------
      subroutine print_sfspecs
      character id*56,pgm*8
      common/gridid/id,pgm
      common/gridspecs/ ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl
      print *,'      id=',id
      print *,'      pgm=',pgm
      print *,'      nz=  ',nz
      print *,'      ncol=',ncol,'   xo=',xo,'   dx=',dx
      print *,'      nrow=',nrow,'   yo=',yo,'   dy=',dy
      print *,' '
      print *,'      iprj=',iproj,'   cm=',cm,'   bl=',bl
      return
      end
c---------------------------------------------------------------------------
      subroutine row_read(iunit,ncol,zrow)
      dimension zrow(ncol)
      read(iunit) dum,zrow
      return
      end
c----------------------------------------------------------------------------
      subroutine row_write(iunit,ncol,zrow)
      dimension zrow(ncol)
      dum=0.
      write(iunit) dum,zrow
      return
      end
c----------------------------------------------------------------------------
      subroutine crow_read(iunit,ncol,zrow)
      complex zrow(ncol)
      read(iunit) dum,zrow
      return
      end
c----------------------------------------------------------------------------
      subroutine crow_write(iunit,ncol,zrow)
      complex zrow(ncol)
      dum=0.
      write(iunit) dum,zrow
      return
      end
c--------------------------------------------------------------------------
      subroutine ask(ques)
      character*(*) ques
      character*40 string
      string=ques
c
c  --  This pads ques to left with blanks (or truncates).
c
      write(6,101) string
 101  format(5x,a40,$)
      return
      end        
c--------------------------------------------------------------------------
      subroutine hgrad
c
c     Calculates horizontal gradients...Modified slightly by Rick Blakely
c  from a program by Bob Simpson.
c
      character*50 infil,outfil
      character id*56,pgm1*8,pgm2*8
      dimension a(5000),b(5000),c(5000),grad(5000)
      data dcval/'ffff7fff'x/,dcvaltest/1.e37/,pgm2/'gradient'/
      rad=3.1415927/180.
c
c      print *,'*Mode: 1=magnitude of gradient'
c      print *,'       2=direction of gradient (0 to 360)'
c      print *,'       3=trend of gradient (0 to 180)'
c      print *,'*Give mode'
c      read *,mode
c
      write(6,101)
  101 format(/,' HORIZONTAL GRADIENT CALCULATION ',//)
      call getfile(10,'Filename of input grid = ? ','in',
     &             'unformatted')
      call getfile(11,'Filename of output grid = ? ','out',
     &             'unformatted')
c
      call rdheader(10,id,pgm1,ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl)
      print *,id
      print *,pgm1
      print *,'ncol=',ncol,'     nrow=',nrow
      print *,'xo=  ',xo,'     yo=  ',yo
      print *,'dx=  ',dx,'     dy=  ',dy
      write(6,102)
  102 format(/,' New ID (56 characters or less) = ? '$)
      read '(a56)', id
      call wrheader(11,id,pgm2,ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl)
c
      call getrow(10,ncol,b)
      call getrow(10,ncol,c)
c
      do 10 i=1,ncol
 10   grad(i)=dcval
      call putrow(11,ncol,grad)
c
      do 200 j=2,nrow-1
      do 120 i=1,ncol
      a(i)=b(i)
 120  b(i)=c(i)
      call getrow(10,ncol,c)
      do 40 i=1,ncol
 40   grad(i)=dcval
      do 100 i=2,ncol-1
      if(b(i-1).gt.dcvaltest.or.b(i+1).gt.dcvaltest) go to 100
      if(a(i).gt.dcvaltest.or.c(i).gt.dcvaltest) go to 100
      dzdx=(b(i+1)-b(i-1))/(2*dx)
      dzdy=(c(i)-a(i))/(2*dy)
      grad(i)=sqrt(dzdx**2+dzdy**2)
c      if(mode.eq.1) grad(i)=sqrt(dzdx**2+dzdy**2)
c      if(mode.eq.2) grad(i)=atan2(dzdy,dzdx)/rad
c      if(mode.eq.3) grad(i)=amod((atan2(dzdy,dzdx)+360.)
c     1  /rad,180.)
 100  continue
c
      call putrow(11,ncol,grad)
 200  continue
c
      do 510 i=1,ncol
 510  grad(i)=dcval
      call putrow(11,ncol,grad)
c
      close(10)
      close(11)
      return
      end
c------------------------------------------------------------------------------
      function char_ask(request)
      character*1 char_ask
      character*(*) request
    2 write(6,100)request
  100 format('$',a,' ')
      read(5,102,err=1)char_ask
  102 format(a1)
      return
    1 write(6,101)
  101 format(/,' **ERROR - try again')
      call bell
      go to 2
      end
c------------------------------------------------------------------------
      function downshift(lchar)
      character*1 downshift,lchar
      downshift=lchar
      i=ichar(lchar)
      if(i.ge.65.and.i.le.90)then
         downshift=char(i+32)
         end if
      return
      end
c------------------------------------------------------------------------------
      function float_ask(request)
      character*(*) request
    2 write(6,100)request
  100 format('$',a,' ')
      read(5,*,err=1)float_ask
      return
    1 write(6,101)
  101 format(/,' **ERROR - try again')
      call bell
      go to 2
      end
c----------------------------------------------------------------------------
      subroutine fourt(data,nn,ndim,isign,iform,work)
c
c     the cooley-tukey fast fourier transform in usasi basic fortran
c
c     transform(j1,j2,,,,) = sum(data(i1,i2,,,,)*w1**((i1-1)*(j1-1))
c                                 *w2**((i2-1)*(j2-1))*,,,),
c     where i1 and j1 run from 1 to nn(1) and w1=exp(isign*2*pi*
c     sqrt(-1)/nn(1)), etc.  there is no limit on the dimensionality
c     (number of subscripts) of the data array.  if an inverse
c     transform (isign=+1) is performed upon an array of transformed
c     (isign=-1) data, the original data will reappear.
c     multiplied by nn(1)*nn(2)*,,,  the array of input data must be
c     in complex format.  however, if all imaginary parts are zero (i.e.
c     the data are disguised real) running time is cut up to forty per-
c     cent.  (for fastest transform of real data, nn(1) should be even.)
c     the transform values are always complex and are returned in the
c     original array of data, replacing the input data.  the length
c     of each dimension of the data array may be any integer.  the
c     program runs faster on composite integers than on primes, and is
c     particularly fast on numbers rich in factors of two.
c
c     timing is in fact given by the following formula.  let ntot be the
c     total number of points (real or complex) in the data array, that
c     is, ntot=nn(1)*nn(2)*...  decompose ntot into its prime factors,
c     such as 2**k2 * 3**k3 * 5**k5 * ...  let sum2 be the sum of all
c     the factors of two in ntot, that is, sum2 = 2*k2.  let sumf be
c     the sum of all other factors of ntot, that is, sumf = 3*k3*5*k5*..
c     the time taken by a multidimensional transform on these ntot data
c     is t = t0 + ntot*(t1+t2*sum2+t3*sumf).  on the cdc 3300 (floating
c     point add time = six microseconds), t = 3000 + ntot*(600+40*sum2+
c     175*sumf) microseconds on complex data.
c
c     implementation of the definition by summation will run in a time
c     proportional to ntot*(nn(1)+nn(2)+...).  for highly composite ntot
c     the savings offered by this program can be dramatic.  a one-dimen-
c     sional array 4000 in length will be transformed in 4000*(600+
c     40*(2+2+2+2+2)+175*(5+5+5)) = 14.5 seconds versus about 4000*
c     4000*175 = 2800 seconds for the straightforward technique.
c
c     the fast fourier transform places three restrictions upon the
c     data.
c     1.  the number of input data and the number of transform values
c     must be the same.
c     2.  both the input data and the transform values must represent
c     equispaced points in their respective domains of time and
c     frequency.  calling these spacings deltat and deltaf, it must be
c     true that deltaf=2*pi/(nn(i)*deltat).  of course, deltat need not
c     be the same for every dimension.
c     3.  conceptually at least, the input data and the transform output
c     represent single cycles of periodic functions.
c
c     the calling sequence is--
c     call fourt(data,nn,ndim,isign,iform,work)
c
c     data is the array used to hold the real and imaginary parts
c     of the data on input and the transform values on output.  it
c     is a multidimensional floating point array, with the real and
c     imaginary parts of a datum stored immediately adjacent in storage
c     (such as fortran iv places them).  normal fortran ordering is
c     expected, the first subscript changing fastest.  the dimensions
c     are given in the integer array nn, of length ndim.  isign is -1
c     to indicate a forward transform (exponential sign is -) and +1
c     for an inverse transform (sign is +).  iform is +1 if the data are
c     complex, 0 if the data are real.  if it is 0, the imaginary
c     parts of the data must be set to zero.  as explained above, the
c     transform values are always complex and are stored in array data.
c     work is an array used for working storage.  it is floating point
c     real, one dimensional of length equal to twice the largest array
c     dimension nn(i) that is not a power of two.  if all nn(i) are
c     powers of two, it is not needed and may be replaced by zero in the
c     calling sequence.  thus, for a one-dimensional array, nn(1) odd,
c     work occupies as many storage locations as data.  if supplied,
c     work must not be the same array as data.  all subscripts of all
c     arrays begin at one.
c
c     example 1.  three-dimensional forward fourier transform of a
c     complex array dimensioned 32 by 25 by 13 in fortran iv.
c     dimension data(32,25,13),work(50),nn(3)
c     complex data
c     data nn/32,25,13/
c     do 1 i=1,32
c     do 1 j=1,25
c     do 1 k=1,13
c  1  data(i,j,k)=complex value
c     call fourt(data,nn,3,-1,1,work)
c     example 2.  one-dimensional forward transform of a real array of
c     length 64 in fortran ii,
c     dimension data(2,64)
c     do 2 i=1,64
c     data(1,i)=real part
c  2  data(2,i)=0.
c     call fourt(data,64,1,-1,0,0)
c
c     there are no error messages or error halts in this program.  the
c     program returns immediately if ndim or any nn(i) is less than one.
c
c     program by norman brenner from the basic program by charles
c     rader,  june 1967.  the idea for the digit reversal was
c     suggested by ralph alter.
c
c     this is the fastest and most versatile version of the fft known
c     to the author.  a program called four2 is available that also
c     performs the fast fourier transform and is written in usasi basic
c     fortran.  it is about one third as long and restricts the
c     dimensions of the input array (which must be complex) to be powers
c     of two.  another program, called four1, is one tenth as long and
c     runs two thirds as fast on a one-dimensional complex array whose
c     length is a power of two.
c
c     reference--
c     ieee audio transactions (june 1967), special issue on the fft.
c
      dimension data(*),nn(*),ifact(32),work(*)
      data twopi/6.2831853071796/,rthlf/0.70710678118655/
      if(ndim-1)920,1,1
1     ntot=2
      do 2 idim=1,ndim
      if(nn(idim))920,920,2
2     ntot=ntot*nn(idim)
c
c     main loop for each dimension
c
      np1=2
      do 910 idim=1,ndim
      n=nn(idim)
      np2=np1*n
      if(n-1)920,900,5
c
c     is n a power of two and if not, what are its factors
c
5     m=n
      ntwo=np1
      if=1
      idiv=2
10    iquot=m/idiv
      irem=m-idiv*iquot
      if(iquot-idiv)50,11,11
11    if(irem)20,12,20
12    ntwo=ntwo+ntwo
      ifact(if)=idiv
      if=if+1
      m=iquot
      go to 10
20    idiv=3
      inon2=if
30    iquot=m/idiv
      irem=m-idiv*iquot
      if(iquot-idiv)60,31,31
31    if(irem)40,32,40
32    ifact(if)=idiv
      if=if+1
      m=iquot
      go to 30
40    idiv=idiv+2
      go to 30
50    inon2=if
      if(irem)60,51,60
51    ntwo=ntwo+ntwo
      go to 70
60    ifact(if)=m
c
c     separate four cases--
c        1. complex transform or real transform for the 4th, 9th,etc.
c           dimensions.
c        2. real transform for the 2nd or 3rd dimension.  method--
c           transform half the data, supplying the other half by con-
c           jugate symmetry.
c        3. real transform for the 1st dimension, n odd.  method--
c           set the imaginary parts to zero.
c        4. real transform for the 1st dimension, n even.  method--
c           transform a complex array of length n/2 whose real parts
c           are the even numbered real values and whose imaginary parts
c           are the odd numbered real values.  separate and supply
c           the second half by conjugate symmetry.
c
70    icase=1
      ifmin=1
      i1rng=np1
      if(idim-4)71,100,100
71    if(iform)72,72,100
72    icase=2
      i1rng=np0*(1+nprev/2)
      if(idim-1)73,73,100
73    icase=3
      i1rng=np1
      if(ntwo-np1)100,100,74
74    icase=4
      ifmin=2
      ntwo=ntwo/2
      n=n/2
      np2=np2/2
      ntot=ntot/2
      i=1
      do 80 j=1,ntot
      data(j)=data(i)
80    i=i+2
c
c     shuffle data by bit reversal, since n=2**k.  as the shuffling
c     can be done by simple interchange, no working array is needed
c
100   if(ntwo-np2)200,110,110
110   np2hf=np2/2
      j=1
      do 150 i2=1,np2,np1
      if(j-i2)120,130,130
120   i1max=i2+np1-2
      do 125 i1=i2,i1max,2
      do 125 i3=i1,ntot,np2
      j3=j+i3-i2
      tempr=data(i3)
      tempi=data(i3+1)
      data(i3)=data(j3)
      data(i3+1)=data(j3+1)
      data(j3)=tempr
125   data(j3+1)=tempi
130   m=np2hf
140   if(j-m)150,150,145
145   j=j-m
      m=m/2
      if(m-np1)150,140,140
150   j=j+m
      go to 300
c
c     shuffle data by digit reversal for general n
c
200   nwork=2*n
      do 270 i1=1,np1,2
      do 270 i3=i1,ntot,np2
      j=i3
      do 260 i=1,nwork,2
      if(icase-3)210,220,210
210   work(i)=data(j)
      work(i+1)=data(j+1)
      go to 230
220   work(i)=data(j)
      work(i+1)=0.
230   ifp2=np2
      if=ifmin
240   ifp1=ifp2/ifact(if)
      j=j+ifp1
      if(j-i3-ifp2)260,250,250
250   j=j-ifp2
      ifp2=ifp1
      if=if+1
      if(ifp2-np1)260,260,240
260   continue
      i2max=i3+np2-np1
      i=1
      do 270 i2=i3,i2max,np1
      data(i2)=work(i)
      data(i2+1)=work(i+1)
270   i=i+2
c
c     main loop for factors of two.  perform fourier transforms of
c     length four, with one of length two if needed.  the twiddle factor
c     w=exp(isign*2*pi*sqrt(-1)*m/(4*mmax)).  check for w=isign*sqrt(-1)
c     and repeat for w=w*(1+isign*sqrt(-1))/sqrt(2).
c
300   if(ntwo-np1)600,600,305
305   np1tw=np1+np1
      ipar=ntwo/np1
310   if(ipar-2)350,330,320
320   ipar=ipar/4
      go to 310
330   do 340 i1=1,i1rng,2
      do 340 k1=i1,ntot,np1tw
      k2=k1+np1
      tempr=data(k2)
      tempi=data(k2+1)
      data(k2)=data(k1)-tempr
      data(k2+1)=data(k1+1)-tempi
      data(k1)=data(k1)+tempr
340   data(k1+1)=data(k1+1)+tempi
350   mmax=np1
360   if(mmax-ntwo/2)370,600,600
370   lmax=max0(np1tw,mmax/2)
      do 570 l=np1,lmax,np1tw
      m=l
      if(mmax-np1)420,420,380
380   theta=-twopi*float(l)/float(4*mmax)
      if(isign)400,390,390
390   theta=-theta
400   wr=cos(theta)
      wi=sin(theta)
410   w2r=wr*wr-wi*wi
      w2i=2.*wr*wi
      w3r=w2r*wr-w2i*wi
      w3i=w2r*wi+w2i*wr
420   do 530 i1=1,i1rng,2
      kmin=i1+ipar*m
      if(mmax-np1)430,430,440
430   kmin=i1
440   kdif=ipar*mmax
450   kstep=4*kdif
      if(kstep-ntwo)460,460,530
460   do 520 k1=kmin,ntot,kstep
      k2=k1+kdif
      k3=k2+kdif
      k4=k3+kdif
      if(mmax-np1)470,470,480
470   u1r=data(k1)+data(k2)
      u1i=data(k1+1)+data(k2+1)
      u2r=data(k3)+data(k4)
      u2i=data(k3+1)+data(k4+1)
      u3r=data(k1)-data(k2)
      u3i=data(k1+1)-data(k2+1)
      if(isign)471,472,472
471   u4r=data(k3+1)-data(k4+1)
      u4i=data(k4)-data(k3)
      go to 510
472   u4r=data(k4+1)-data(k3+1)
      u4i=data(k3)-data(k4)
      go to 510
480   t2r=w2r*data(k2)-w2i*data(k2+1)
      t2i=w2r*data(k2+1)+w2i*data(k2)
      t3r=wr*data(k3)-wi*data(k3+1)
      t3i=wr*data(k3+1)+wi*data(k3)
      t4r=w3r*data(k4)-w3i*data(k4+1)
      t4i=w3r*data(k4+1)+w3i*data(k4)
      u1r=data(k1)+t2r
      u1i=data(k1+1)+t2i
      u2r=t3r+t4r
      u2i=t3i+t4i
      u3r=data(k1)-t2r
      u3i=data(k1+1)-t2i
      if(isign)490,500,500
490   u4r=t3i-t4i
      u4i=t4r-t3r
      go to 510
500   u4r=t4i-t3i
      u4i=t3r-t4r
510   data(k1)=u1r+u2r
      data(k1+1)=u1i+u2i
      data(k2)=u3r+u4r
      data(k2+1)=u3i+u4i
      data(k3)=u1r-u2r
      data(k3+1)=u1i-u2i
      data(k4)=u3r-u4r
520   data(k4+1)=u3i-u4i
      kdif=kstep
      kmin=4*(kmin-i1)+i1
      go to 450
530   continue
      m=m+lmax
      if(m-mmax)540,540,570
540   if(isign)550,560,560
550   tempr=wr
      wr=(wr+wi)*rthlf
      wi=(wi-tempr)*rthlf
      go to 410
560   tempr=wr
      wr=(wr-wi)*rthlf
      wi=(tempr+wi)*rthlf
      go to 410
570   continue
      ipar=3-ipar
      mmax=mmax+mmax
      go to 360
c
c     main loop for factors not equal to two.  apply the twiddle factor
c     w=exp(isign*2*pi*sqrt(-1)*(j1-1)*(j2-j1)/(ifp1+ifp2)), then
c     perform a fourier transform of length ifact(if), making use of
c     conjugate symmetries.
c
600   if(ntwo-np2)605,700,700
605   ifp1=ntwo
      if=inon2
      np1hf=np1/2
610   ifp2=ifact(if)*ifp1
      j1min=np1+1
      if(j1min-ifp1)615,615,640
615   do 635 j1=j1min,ifp1,np1
      theta=-twopi*float(j1-1)/float(ifp2)
      if(isign)625,620,620
620   theta=-theta
625   wstpr=cos(theta)
      wstpi=sin(theta)
      wr=wstpr
      wi=wstpi
      j2min=j1+ifp1
      j2max=j1+ifp2-ifp1
      do 635 j2=j2min,j2max,ifp1
      i1max=j2+i1rng-2
      do 630 i1=j2,i1max,2
      do 630 j3=i1,ntot,ifp2
      tempr=data(j3)
      data(j3)=data(j3)*wr-data(j3+1)*wi
630   data(j3+1)=tempr*wi+data(j3+1)*wr
      tempr=wr
      wr=wr*wstpr-wi*wstpi
635   wi=tempr*wstpi+wi*wstpr
640   theta=-twopi/float(ifact(if))
      if(isign)650,645,645
645   theta=-theta
650   wstpr=cos(theta)
      wstpi=sin(theta)
      j2rng=ifp1*(1+ifact(if)/2)
      do 695 i1=1,i1rng,2
      do 695 i3=i1,ntot,np2
      j2max=i3+j2rng-ifp1
      do 690 j2=i3,j2max,ifp1
      j1max=j2+ifp1-np1
      do 680 j1=j2,j1max,np1
      j3max=j1+np2-ifp2
      do 680 j3=j1,j3max,ifp2
      jmin=j3-j2+i3
      jmax=jmin+ifp2-ifp1
      i=1+(j3-i3)/np1hf
      if(j2-i3)655,655,665
655   sumr=0.
      sumi=0.
      do 660 j=jmin,jmax,ifp1
 659  sumr=sumr+data(j)
660   sumi=sumi+data(j+1)
      work(i)=sumr
      work(i+1)=sumi
      go to 680
665   iconj=1+(ifp2-2*j2+i3+j3)/np1hf
      j=jmax
      sumr=data(j)
      sumi=data(j+1)
      oldsr=0.
      oldsi=0.
      j=j-ifp1
670   tempr=sumr
      tempi=sumi
      sumr=twowr*sumr-oldsr+data(j)
      sumi=twowr*sumi-oldsi+data(j+1)
      oldsr=tempr
      oldsi=tempi
      j=j-ifp1
      if(j-jmin)675,675,670
675   tempr=wr*sumr-oldsr+data(j)
      tempi=wi*sumi
      work(i)=tempr-tempi
      work(iconj)=tempr+tempi
      tempr=wr*sumi-oldsi+data(j+1)
      tempi=wi*sumr
      work(i+1)=tempr+tempi
      work(iconj+1)=tempr-tempi
680   continue
      if(j2-i3)685,685,686
685   wr=wstpr
      wi=wstpi
      go to 690
686   tempr=wr
      wr=wr*wstpr-wi*wstpi
      wi=tempr*wstpi+wi*wstpr
690   twowr=wr+wr
      i=1
      i2max=i3+np2-np1
      do 695 i2=i3,i2max,np1
      data(i2)=work(i)
      data(i2+1)=work(i+1)
695   i=i+2
      if=if+1
      ifp1=ifp2
      if(ifp1-np2)610,700,700
c
c     complete a real transform in the 1st dimension, n even, by con-
c     jugate symmetries.
c
700   go to (900,800,900,701),icase
701   nhalf=n
      n=n+n
      theta=-twopi/float(n)
      if(isign)703,702,702
702   theta=-theta
703   wstpr=cos(theta)
      wstpi=sin(theta)
      wr=wstpr
      wi=wstpi
      imin=3
      jmin=2*nhalf-1
      go to 725
710   j=jmin
      do 720 i=imin,ntot,np2
      sumr=(data(i)+data(j))/2.
      sumi=(data(i+1)+data(j+1))/2.
      difr=(data(i)-data(j))/2.
      difi=(data(i+1)-data(j+1))/2.
      tempr=wr*sumi+wi*difr
      tempi=wi*sumi-wr*difr
      data(i)=sumr+tempr
      data(i+1)=difi+tempi
      data(j)=sumr-tempr
      data(j+1)=-difi+tempi
720   j=j+np2
      imin=imin+2
      jmin=jmin-2
      tempr=wr
      wr=wr*wstpr-wi*wstpi
      wi=tempr*wstpi+wi*wstpr
725   if(imin-jmin)710,730,740
730   if(isign)731,740,740
731   do 735 i=imin,ntot,np2
735   data(i+1)=-data(i+1)
740   np2=np2+np2
      ntot=ntot+ntot
      j=ntot+1
      imax=ntot/2+1
745   imin=imax-2*nhalf
      i=imin
      go to 755
750   data(j)=data(i)
      data(j+1)=-data(i+1)
755   i=i+2
      j=j-2
      if(i-imax)750,760,760
760   data(j)=data(imin)-data(imin+1)
      data(j+1)=0.
      if(i-j)770,780,780
765   data(j)=data(i)
      data(j+1)=data(i+1)
770   i=i-2
      j=j-2
      if(i-imin)775,775,765
775   data(j)=data(imin)+data(imin+1)
      data(j+1)=0.
      imax=imin
      go to 745
780   data(1)=data(1)+data(2)
      data(2)=0.
      go to 900
c
c     complete a real transform for the 2nd or 3rd dimension by
c     conjugate symmetries.
c
800   if(i1rng-np1)805,900,900
805   do 860 i3=1,ntot,np2
      i2max=i3+np2-np1
      do 860 i2=i3,i2max,np1
      imin=i2+i1rng
      imax=i2+np1-2
      jmax=2*i3+np1-imin
      if(i2-i3)820,820,810
810   jmax=jmax+np2
820   if(idim-2)850,850,830
830   j=jmax+np0
      do 840 i=imin,imax,2
      data(i)=data(j)
      data(i+1)=-data(j+1)
840   j=j-2
850   j=jmax
      do 860 i=imin,imax,np0
      data(i)=data(j)
      data(i+1)=-data(j+1)
860   j=j-np0
c
c     end of loop on each dimension
c
900   np0=np1
      np1=np2
910   nprev=n
920   return
      end
c----------------------------------------------------------------------
      subroutine getproj(iprj,cm,bl)
c     Requests projection information including cental meridian and
c      base latitude.
      print '(/,(1x,a))',
     &  ' Available Projections...',
     &  ' ',
     &  ' 0=none, 1=poly, 2=transverse mercator="UTM"',
     &  '  3=merc, 4=lambert US, 14=general lambert',
     &  '   5=albers US, 6=albers Alaska, 7=albers Hawaii, 12=DNAG',
     &  '    99=proj not known'
      call ask(' Projection (0 to 14):  ')
      read *, iprj
      if(iprj.gt.0.and.iprj.lt.99) then
        call ask(' CM, BL (d m, d m) =       ')
        read *, cmd,cmm,bld,blm
        cm=-abs(decdeg(cmd,cmm))
c        this forces west longitude
        bl=decdeg(bld,blm)
      endif
      return
      end
c-----------------------------------------------------------------------------
      function int_ask(request)
      character*(*) request
    2 write(6,100)request
  100 format('$',a,' ')
      read(5,*,err=1)int_ask
      return
    1 write(6,101)
  101 format(/,' **ERROR - try again')
      call bell
      go to 2
      end
c--------------------------------------------------------------------------
      subroutine prheader(unit,id,pgm,ncol,nrow,nz,xo,dx,yo,dy,
     &    iproj,cm,bl)
      character id*56, pgm*8
      integer unit
      write(unit,1001) id,pgm
 1001 format(/,'    ID=',a56,'   pgm=',a8)
      write(unit,1002) 'nz  =',nz
      write(unit,1002) 'ncol=',ncol,'xo=',xo,'dx=',dx
      write(unit,1002) 'nrow=',nrow,'yo=',yo,'dy=',dy
 1002 format(1x,a8,i8,a8,g14.8,a8,g14.8)
      write(unit,'(1h0,a,i2,2(a,f9.4))')
     & '   Projection=',iproj,'     cm=',cm,'          bl=',bl
      write(unit,1003)
 1003 format(1h )
      return
      end
c------------------------------------------------------------------------
      subroutine putrow(iounit,ncol,row)
      real row(ncol)
      parameter (dummy = 0.0)
      write(iounit) dummy,row
      return
      end
c-----------------------------------------------------------------------
      subroutine wrheader(unit,id,pgm,ncol,nrow,nz,
     & xo,dx,yo,dy,iproj,cm,bl)
      character id*56, pgm*8
      integer unit
      write(unit) id,pgm,ncol,nrow,nz,xo,dx,yo,dy,iproj,cm,bl
      return
      end
c-----------------------------------------------------------------------
      subroutine proj
c      This subroutine is called thru its 3 entry points:
c           1. projset(iproj,cmdeg,bldeg) - sets up both fwd and inverse prjts.
c                iproj = projection (see list below)
c                cm = central meridian in degrees, negative for west lon.
c                bl = base latitude in degrees, positive for north lat.
c           2. projfwd(xdeg,ydeg,xkm,ykm) - does forward projection.
c                xdeg,ydeg = lon,lat in decimal degrees (- for west lon).
c                xkm,ykm = projected coordinates in km.
c           3. projinv(xkm,ykm,xdeg,ydeg) - does inverse projection.
c
c      Projections available are:
c            1 = polyconic
c            2 = ellipsoidal transverse mercator
c            3 = mercator
c            4 = Lambert (standard U.S. parallels at 33N and 45N.
c            5 = Albers US (parallels at 29.5 and 45.5)
c            6 = Albers Alaska (         55.0 and 65.0)
c            7 = Albers Hawaii (          8.0 and 18.0)
c           12 = DNAG = spherical transverse mercator
c           14 = Lambert with standard parallels requested.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      common/projconsts/ ipr,cm,bl,xcm,ybl
      data sca/0.001/
      decdeg(deg,xmin)= deg + sign(xmin,deg)/60.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry projset(iproj,cmdeg,bldeg)
      ipr=iproj
      cm=cmdeg
      bl=bldeg
c      if(ipr.eq.0) call getproj(ipr,cm,bl)
      if(ipr.eq.0) return
      goto (10,20,90,40,50,60,70,90,5,5,5,22,5,42), ipr
  5   print *, 'PROJECTION NUMBER GIVEN NOT SUPPORTED...'
      stop
c.....Polyconic
 10   call polyset(cm,bl)
       return
c.....Ellipsoidal transverse mercator
 20   call etmset(cm,bl)
       return
c.....DNAG = spherical transverse mercator
 22   call stmset(cm,bl,0.926)
c      scale factor set for DNAG base.
       return
c.....Lambert conformal conic (ellipsoidal) with standard US parallels
 40   call lambset(cm,bl,33.,45.)
c      parallels set to standard for US
       return
c.....Lambert conformal conic (ellipsoidal) with parallels requested
 42   print '(/,1h$,a)', '     *Give southern standard parallel (d m): '
      read *, p1d,p1m
      p1=decdeg(p1d,p1m)
      print '(/,1h$,a)', '     *Give northern standard parallel (d m): '
      read *, p2d,p2m
      p2=decdeg(p2d,p2m)
      call lambset(cm,bl,p1,p2)
       return
c.....Albers conical equal-area (ellipsoidal) - US standard parallels
 50   call albset(cm,bl,29.5,45.5)
       return
c.....Albers conical equal-area (ellipsoidal) - Alaska standard parallels
 60   call albset(cm,bl,55.0,65.0)
       return
c.....Albers conical equal-area (ellipsoidal) - Hawaii standard parallels
 70   call albset(cm,bl,8.0,18.0)
       return
c.....All others
 90   call prjctl(bl,cm,xcm,ybl,cm,sca,ipr)
       return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry projfwd(xdeg,ydeg,xkm,ykm)
      if(ipr.eq.0) then
        xkm=xdeg
        ykm=ydeg
        return
      endif
      goto (110,120,190,140,150,150,150,190,105,105,105,122,105,140),
     &   ipr
 105  print *, 'PROJECTION NUMBER GIVEN NOT SUPPORTED...'
      stop
 110  call polyfwd(xdeg,ydeg,xkm,ykm)
       return
 120  call etmfwd(xdeg,ydeg,xkm,ykm)
       return
 122  call stmfwd(xdeg,ydeg,xkm,ykm)
       return
 140  call lambfwd(xdeg,ydeg,xkm,ykm)
       return
 150  call albfwd(xdeg,ydeg,xkm,ykm)
       return
 190  call prjctl(ydeg,xdeg,xkm,y,cm,sca,ipr)
      ykm=y-ybl
       return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry projinv(xkm,ykm,xdeg,ydeg)
      if(ipr.eq.0) then
        xdeg=xkm
        ydeg=ykm
        return
      endif
      goto (210,220,205,240,250,250,250,205,205,205,205,222,205,240),
     &    ipr
 205  print *, 'THIS INVERSE PROJECTION NOT SUPPORTED'
      stop
 210  call polyinv(xkm,ykm,xdeg,ydeg)
       return
 220  call etminv(xkm,ykm,xdeg,ydeg)
       return
 222  call stminv(xkm,ykm,xdeg,ydeg)
       return
 240  call lambinv(xkm,ykm,xdeg,ydeg)
       return
 250  call albinv(xkm,ykm,xdeg,ydeg)
       return
      end
c--------------------------------------------------------------------------
      subroutine polyconic
c
c      Ellipsoidal form of the polyconic projection,
c       based on the Clarke 1866 ellipsoid.
c      Single precision used ....
c      See J.P. Snyder, 1982, Map projections used by the USGS:
c       USGS Bulletin no. 1532, p. 129-131, p. 256-258.
c      Programmed by R.Simpson 5/16/84
c
c      Used by calling 1st entry point to set constants, then 2nd or 3rd
c       entry points to do forward or inverse projection respectively.
c      For example:
c                         .
c                         .
c               call polyset(cm,bl)
c                         .
c                         .
c               call polyfwd(xdeg,ydeg,xkm,ykm)
c                         .
c                         .
c         (or)  call polyinv(xkm,ykm,xdeg,ydeg)
c                         .
c                         .
c
c      where: cm = central meridian in decimal degrees (-for west)
c             bl = base latidude in decimal degrees (+ for north)
c             xdeg = longitude in decimal degrees
c             ydeg = latitude in decimal degrees
c             xkm,ykm = projected coordinates in km.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      real n,mfcn,mpfcn,m,m0,mn,mpn,ma,lam,lam0
      parameter (maxiter=10, tol=1.0E-6)
      parameter (a=6378.2064)
      parameter (e=0.0822719, esq=0.676866E-2)
      parameter (deg2rad=0.17453293E-1, rad2deg=57.295778)
      common/polyconsts/ m0,c0,c2,c4,c6,lam0,phi0
c     functions...
c      (eqn 3-21, p. 20)....
      mfcn(phi) =
     &    a*(c0*phi-c2*sin(2.*phi)+c4*sin(4.*phi)-c6*sin(6.*phi))
c      (eqn 15-17, p. 130)....
      mpfcn(phi) =
     &    c0-2.0*c2*cos(2.0*phi)+4.0*c4*cos(4.0*phi)-6.0*c6*cos(6.0*phi)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry polyset(cm,bl)
c      cm= central meridian in degrees (- for w)
c      bl= base latitude in degrees
      lam0=cm*deg2rad
      phi0=bl*deg2rad
      c0=1.-e**2/4.-3.*e**4/64.-5.*e**6/256.
      c2=3.*e**2/8.+3.*e**4/32.+45.*e**6/1024.
      c4=15.*e**4/256.+45.*e**6/1024.
      c6=35.*e**6/3072.
      m0=mfcn(phi0)
      return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry polyfwd(xdeg,ydeg,xkm,ykm)
c      xdeg,ydeg = lon (- for w) and lat in decimal degrees.
c      xkm,ykm = projected coordinates in km.
      lam=xdeg*deg2rad
      phi=ydeg*deg2rad
      n=a/sqrt(1.-esq*sin(phi)**2)
      cotphi=1.0/tan(phi)
      m=mfcn(phi)
      if(phi.ne.0) then
        bige=(lam-lam0)*sin(phi)
        xkm=n*sin(bige)*cotphi
        ykm=m-m0+n*(1.0-cos(bige))*cotphi
      else
        xkm=a*(lam-lam0)
        ykm=-m0
      endif
      return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry polyinv(xkm,ykm,xdeg,ydeg)
c      xkm,ykm = projected coordinates in km.
c      xdeg,ydeg = lon (- for w) and lat in decimal degrees.
      if(y.ne.-m0) then
        biga=(m0+ykm)/a
        bigb=(xkm/a)**2 + biga**2
        phin=biga
        do 40 i=1,maxiter
         bigc=sqrt(1.-esq*sin(phin)**2)*tan(phin)
         mn=mfcn(phin)
         mpn=mpfcn(phin)
         ma=mn/a
         phinp1=phin - (biga*(bigc*ma+1.)-ma-0.5*(ma**2+bigb)*bigc)/
     &   (esq*sin(2.*phin)*(ma**2+bigb-2.*biga*ma)/(4.*bigc)
     &    +(biga-ma)*(bigc*mpn-2./sin(2.*phin))-mpn)
         if(abs(phinp1-phin).le.tol) go to 60
         phin=phinp1
 40     continue
c.....  Upon normal exit from DO loop print...
        print *, 'WARNING...no convergence to tolerance'
        print *, '  after maxiterations =', maxiter
        print *, '    for xkm,ykm =', xkm,ykm
 60     phi=phinp1
        lam=asin(xkm*bigc/a)/sin(phi) + lam0
      else
        phi=0.0
        lam=xkm/a + lam0
      endif
      xdeg=lam*rad2deg
      ydeg=phi*rad2deg
      return
      end
c--------------------------------------------------------------------------
      subroutine etm
c      Ellipsoidal form of transverse mercator,
c       based on the Clarke 1866 ellipsoid.
c      Single precision used ....
c      See J.P. Snyder, 1982, Map projections used by the USGS:
c       USGS Bulletin no. 1532, p. 67-69, p.232-235.
c      Formulas not accurate for longitudes greater than 4 degrees from
c       the chosen central meridian (Snyder, p. 67).
c      Programmed by R.Simpson 12/21/83.
c      Used by calling 1st entry point to set constants, then 2nd or 3rd
c       entry points to do forward or inverse projection respectively.
c      For example:
c                         .
c                         .
c               call etmset(cm,bl)
c                         .
c                         .
c               call etmfwd(xdeg,ydeg,xkm,ykm)
c                         .
c                         .
c         (or)  call etminv(xkm,ykm,xdeg,ydeg)
c                         .
c                         .
c
c      where: cm = central meridian in decimal degrees (-for west)
c             bl = base latidude in decimal degrees (+ for north)
c             xdeg = longitude in decimal degrees
c             ydeg = latitude in decimal degrees
c             xkm,ykm = projected coordinates in km.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      real k0,n,n1,mu,mfcn,m,m0,lam,lam0
      parameter (a=6378.2064)
      parameter (e=0.0822719, esq=0.676866E-2)
      parameter (k0=0.9996)
      parameter (deg2rad=0.17453293E-1, rad2deg=57.295778)
      common/etmconsts/ iwarn,epsq,m0,c0,c2,c4,c6,lam0,phi0,cmdeg
c     functions....
      mfcn(phi) =
     &    a*(c0*phi-c2*sin(2.*phi)+c4*sin(4.*phi)-c6*sin(6.*phi))
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry etmset(cm,bl)
c      cm= central meridian in degrees (- for w)
c      bl= base latitude in degrees
      iwarn=0
      cmdeg=cm
      lam0=cm*deg2rad
      phi0=bl*deg2rad
      c0=1.-e**2/4.-3.*e**4/64.-5.*e**6/256.
      c2=3.*e**2/8.+3.*e**4/32.+45.*e**6/1024.
      c4=15.*e**4/256.+45.*e**6/1024.
      c6=35.*e**6/3072.
      m0=mfcn(phi0)
      epsq=esq/(1.0-esq)
      return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry etmfwd(xdeg,ydeg,xkm,ykm)
c      xdeg,ydeg = lon (- for w) and lat in decimal degrees.
c      xkm,ykm = projected coordinates in km.
      if(abs(xdeg-cmdeg).gt.4.0.and.iwarn.eq.0) then
       print *, 'WARNING...accuracy drops for lons greater than'
       print *, '    4 degrees from the central meridian...'
       iwarn=iwarn+1
      endif
      lam=xdeg*deg2rad
      phi=ydeg*deg2rad
      n=a/sqrt(1.-esq*sin(phi)**2)
      tnphi=tan(phi)
      t=tnphi**2
      c=epsq*csphi**2
      biga=cos(phi)*(lam-lam0)
      m=mfcn(phi)
      xkm=k0*n*(biga+(1.-t+c)*biga**3/6. +
     &     (5.-18.*t+t**2+72.*c-58.*epsq)*biga**5/120.)
      ykm=k0*(m-m0+n*tnphi*(.5*biga**2 +
     &     (5.-t+9.*c+4.*c**2)*biga**4/24. +
     &     (61.-58.*t+t**2+600.*c-330.*epsq)*biga**6/720.))
      return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry etminv(xkm,ykm,xdeg,ydeg)
c      xkm,ykm = projected coordinates in km.
c      xdeg,ydeg = lon (- for w) and lat in decimal degrees.
      m=m0 + ykm/k0
      e1=(1.-sqrt(1.-esq))/(1.+sqrt(1.-esq))
      mu=m/(a*(1.-esq/4.-3.*esq**2/64.-5.*esq**3/256.))
      phi1=mu+(3.*e1/2.-27.*e1**3/32.)*sin(2.*mu) +
     &      (21.*e1**2/16.-55.*e1**4/32.)*sin(4.*mu) +
     &      (151.*e1**3/96.)*sin(6.*mu)
      c1=epsq*cos(phi1)**2
      t1=tan(phi1)**2
      n1=a/sqrt(1.-esq*sin(phi1)**2)
      r1=a*(1-esq)/((1-esq*sin(phi1)**2)*sqrt(1-esq*sin(phi1)**2))
      d=xkm/(n1*k0)
      phi=phi1-(n1*tan(phi1)/r1)*
     &     (d**2/2.-(5.+3.*t1+10.*c1-4.*c1**2-9.*epsq)*d**4/24.
     &     + (61.+90*t1+298*c1+45*t1**2-252.*epsq-3.*c1**2)*d**6/720.)
      lam=lam0+(d-(1.+2.*t1+c1)*d**3/6.
     &     +(5.-2.*c1+28.*t1-3.*c1**2+8.*epsq+24.*t1**2)*d**5/120.)
     &     /cos(phi1)
      xdeg=lam*rad2deg
      ydeg=phi*rad2deg
      return
      end
c--------------------------------------------------------------------------
      subroutine stm
c      Spherical form of transverse mercator.
c      Single precision used ....
c      See J.P. Snyder, 1982, Map projections used by the USGS:
c       USGS Bulletin no. 1532, p. 64-67, p.231-232.
c      Programmed by R.Simpson 1/10/84.
c      Used by calling 1st entry point to set constants, then 2nd or 3rd
c       entry points to do forward or inverse projection respectively.
c      For example:
c                         .
c                         .
c               call stmset(cm,bl,scalefac)
c                         .
c                         .
c               call stmfwd(xdeg,ydeg,xkm,ykm)
c                         .
c                         .
c         (or)  call stminv(xkm,ykm,xdeg,ydeg)
c                         .
c                         .
c
c      where: cm = central meridian in decimal degrees (-for west)
c             bl = base latidude in decimal degrees (+ for north)
c             scalefac = central scale factor
c             xdeg = longitude in decimal degrees
c             ydeg = latitude in decimal degrees
c             xkm,ykm = projected coordinates in km.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      real k0,k,phi,phi0,lam,lam0
      double precision d,e,dphi,dlam
      parameter (r=6371.204)
c      parameter (r=1.)
      parameter (deg2rad=0.17453293E-1, rad2deg=57.295778)
      common/stmconsts/ cmdeg,lam0,phi0,k0
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry stmset(cm,bl,scalefac)
c      scalefac = central scale factor
c      cm= central meridian in degrees (- for w)
c      bl= base latitude in degrees
      cmdeg=cm
      k0=scalefac
      lam0=cm*deg2rad
      phi0=bl*deg2rad
      return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry stmfwd(xdeg,ydeg,xkm,ykm)
c      xdeg,ydeg = lon (- for w) and lat in decimal degrees.
c      xkm,ykm = projected coordinates in km.
      lam=xdeg*deg2rad
      phi=ydeg*deg2rad
      b = cos(phi)*sin(lam-lam0)
      xkm=0.5*r*k0*log((1.+b)/(1.-b))
      ykm=r*k0*(atan(tan(phi)/cos(lam-lam0)) - phi0)
c      k=k0/sqrt(1.-b**2)
      return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry stminv(xkm,ykm,xdeg,ydeg)
c      xkm,ykm = projected coordinates in km.
c      xdeg,ydeg = lon (- for w) and lat in decimal degrees.
      d=dble(ykm)/dble(r*k0) + dble(phi0)
      e=dble(xkm)/dble(r*k0)
      dphi=asin(sin(d)/cosh(e))
      dlam=dble(lam0) + atan(sinh(e)/cos(d))
      xdeg=dlam*rad2deg
      ydeg=dphi*rad2deg
      return
      end
c--------------------------------------------------------------------------
      subroutine lambert
c
c      Lambert conformal conic projection
c            based on the Clarke 1866 ellipsoid.
c      Single precision used .... max error probably < 5 meters.
c      See J.P. Snyder, 1982, Map projections used by the USGS:
c            USGS Bulletin no. 1532, p. 101-109, p.250-252.
c      Normal standard parallels for US are p1=33N and p2=45N.
c       See Snyder Table 8, p. 60-62 for state plane coordinate systems.
c      Programmed by R.Simpson 8/4/1983.
c
c      Used by calling 1st entry point to set constants, then 2nd or 3rd
c       entry points to do forward or inverse projection respectively.
c      For example:
c                         .
c                         .
c               call lambset(-96., 30.,p1,p2)
c                         .
c                         .
c               call lambfwd(-123., 45.,xkm,ykm)
c                         .
c                         .
c         (or)  call lambinv(xkm,ykm,dlon,dlat)
c                         .
c                         .
c      This will give x,y in km... (or) call to inverse will give back
c       dlon=-123., dlat=45.  p1 and p2 are the chosen parallels.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      parameter (a=6378.2064)
      parameter (e=0.0822719, esq=0.676866E-2)
      parameter (deg2rad=0.17453293E-1, rad2deg=57.295778, pi=3.1415927)
      real m1,m2,mfcn,n,lam,lam0
      common/lambconsts/ n,bigf,lam0,phi0,rho0
c     functions....
c.....(eqn 12-15, p. 97)...
      mfcn(cphi,sphi) = cphi/sqrt(1.0-esq*sphi**2)
c.....(eqn 13-9, p. 107)...
      tfcn(phi)=
     &   tan(0.25*pi-0.5*phi)/((1.-e*sin(phi))/(1.+e*sin(phi)))**(0.5*e)
c.....(eqn 13-7, p 107)...
      rhofcn(t) = a*bigf*(t)**n
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry lambset(cm,bl,p1,p2)
c      cm= central meridian in degrees (- for w)
c      bl= base latitude in degrees
c      p1,p2= parallels chosen (33 and 45 for U.S. State Maps)
c              (p1=south parallel, p2=north parallel)
      lam0=cm*deg2rad
      phi0=bl*deg2rad
      phi1=p1*deg2rad
      phi2=p2*deg2rad
      csphi1=cos(phi1)
      snphi1=sin(phi1)
      csphi2=cos(phi2)
      snphi2=sin(phi2)
      m1=mfcn(csphi1,snphi1)
      m2=mfcn(csphi2,snphi2)
      t0=tfcn(phi0)
      t1=tfcn(phi1)
      t2=tfcn(phi2)
      n=(log(m1)-log(m2))/(log(t1)-log(t2))
      bigf=m1/(n*(t1)**n)
      rho0=rhofcn(t0)
      return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry lambfwd(xdeg,ydeg,xkm,ykm)
c      xdeg,ydeg = lon (- for w) and lat in decimal degrees.
c      xkm,ykm = projected coordinates in km.
      lam=xdeg*deg2rad
      phi=ydeg*deg2rad
      t=tfcn(phi)
      rho=rhofcn(t)
      th=n*(lam-lam0)
      xkm=rho*sin(th)
      ykm=rho0-rho*cos(th)
      return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry lambinv(xkm,ykm,xdeg,ydeg)
c      xkm,ykm = projected coordinates in km.
c      xdeg,ydeg = lon (- for w) and lat in decimal degrees.
      rho=sign(sqrt(xkm**2+(rho0-ykm)**2),n)
      th=atan2(xkm*sign(1.,n),(rho0-ykm)*sign(1.,n))
      t=(rho/(a*bigf))**(1./n)
      chi=0.5*pi - 2.0*atan(t)
      ydeg=(chi+(esq/2+5*esq**2/24+esq**3/12)*sin(2*chi)
     &    +(7*esq**2/48+29*esq**3/240)*sin(4*chi)
     &    +(7*esq**3/160)*sin(6*chi))*rad2deg
      xdeg=(th/n+lam0)*rad2deg
      return
      end
c--------------------------------------------------------------------------
      subroutine alber
c      Albers equal-area conic projection
c       based on the Clarke 1866 ellipsoid.
c      Single precision used .... max error probably < 5 meters.
c      See J.P. Snyder, 1982, Map projections used by the USGS:
c       USGS Bulletin no. 1532, p. 93-99, p.245-248.
c      Set up to take standard parallels for US, Alaska, or Hawaii.
c      Programmed by R.Simpson 8/4/1983.
c      Used by calling 1st entry point to set constants, then 2nd or 3rd
c       entry points to do forward or inverse projection respectively.
c      For example:
c                         .
c                         .
c               call albset(-96., 30., p1,p2)
c                         .
c                         .
c               call albfwd(-123., 45., xkm,ykm)
c                         .
c                         .
c         (or)  call albinv(xkm,ykm,dlon,dlat)
c                         .
c                         .
c      This will give x,y in km... call to inverse will give back
c       dlon=-123., dlat=45.  p1 and p2 are the chosen standard parallels.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      parameter (a=6378.2064)
      parameter (e=0.0822719, esq=0.676866E-2)
      parameter (deg2rad=0.17453293E-1, rad2deg=57.295778)
      real m1,m2,mfcn,n,lam,lam0
      common/albconsts/ c,n,lam0,phi0,rho0
c     functions....
      mfcn(cphi,sphi) = cphi/sqrt(1.0-esq*sphi**2)
      qfcn(sphi) = (1.-esq)*(sphi/(1.-esq*sphi**2) - (0.5/e)*
     &                 log((1.-e*sphi)/(1.+e*sphi)))
      rhofcn(q) = a*sqrt(c-n*q)/n
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry albset(cm,bl,p1,p2)
c      cm= central meridian in degrees (- for w)
c      bl= base latitude in degrees
c      p1= southern standard parallel
c      p2= northern standard parallel
c               p1,p1= 29.5, 45.5 for U.S.
c                    = 55.0, 65.0 for Alaska
c                    =  8.0, 18.0 for Hawaii
      lam0=cm*deg2rad
      phi0=bl*deg2rad
      phi1=p1*deg2rad
      phi2=p2*deg2rad
      snphi0=sin(phi0)
      csphi1=cos(phi1)
      snphi1=sin(phi1)
      csphi2=cos(phi2)
      snphi2=sin(phi2)
      m1=mfcn(csphi1,snphi1)
      m2=mfcn(csphi2,snphi2)
      q1=qfcn(snphi1)
      q2=qfcn(snphi2)
      q0=qfcn(snphi0)
      n=(m1**2-m2**2)/(q2-q1)
      c=m1**2+n*q1
      rho0=rhofcn(q0)
      return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry albfwd(xdeg,ydeg,xkm,ykm)
c      xdeg,ydeg = lon (- for w) and lat in decimal degrees.
c      xkm,ykm = projected coordinates in km.
      lam=xdeg*deg2rad
      phi=ydeg*deg2rad
      snphi=sin(phi)
      csphi=cos(phi)
      q=qfcn(snphi)
      rho=rhofcn(q)
      th=n*(lam-lam0)
      xkm=rho*sin(th)
      ykm=rho0-rho*cos(th)
      return
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      entry albinv(xkm,ykm,xdeg,ydeg)
c      xkm,ykm = projected coordinates in km.
c      xdeg,ydeg = lon (- for w) and lat in decimal degrees.
      rho=sqrt(xkm**2+(rho0-ykm)**2)
      th=atan2(xkm*sign(1.,n),(rho0-ykm)*sign(1.,n))
      q=(c-(rho*n/a)**2)/n
      beta=asin(q/(1-((1-esq)/(2*e))*log((1-e)/(1+e))))
      ydeg=(beta+(esq/3+31*esq**2/180+517*esq**3/5040)*sin(2*beta)
     &    +(23*esq**2/360+251*esq**3/3780)*sin(4*beta)
     &    +(761*esq**3/45360)*sin(6*beta))*rad2deg
      xdeg=(lam0+th/n)*rad2deg
      return
      end
c--------------------------------------------------------------------------
      subroutine prjctl(ylat,xlon,x,y,cm,sfac,iprojt)
c
c     Programmed by Dick Godson, USGS Denver...
c
      double precision yl,xl,x1,y1
      yl=dble(ylat)*.0174532925199433d0
      xl=dble(abs(cm)-abs(xlon))*.0174532925199433d0
      go to (10,20,30,40,50,50,50,60),iprojt
   10 call poly(yl,xl,x1,y1)
      go to 90
   20 call utmfwd(yl,xl,x1,y1)
      go to 90
   30 call merctr(yl,xl,x1,y1)
      go to 90
   40 call lambrt(yl,xl,x1,y1)
      go to 90
   50 call albers(yl,xl,x1,y1)
      go to 90
   60 call pstereo(yl,xl,x1,y1)
      go to 90
   90 x=sngl(x1)*sfac
      y=sngl(y1)*sfac
      return
      end
c--------------------------------------------------------------------------
      subroutine poly(phi,dlamb,x,y)
      double precision phi,dlamb,xc,x,y,a,b,t,q,phi2,c1,c2
      a(xc)=
     &   6.378206402718907d 06 +xc*(
     &  -3.167517353503576d 06 +xc*(
     &   2.478805037574243d 05 +xc*(
     &  -3.530710396439220d 03 +xc*(
     &  -6.565371848240127d 02 +xc*(
     &   6.822539551727124d 01 +xc*(
     &  -2.888860980506611d 00 ))))))
      b(xc)=
     &   3.189103200618349d 06 +xc*(
     &  -2.115275857345996d 06 +xc*(
     &   4.144758431728325d 05 +xc*(
     &  -3.625295427368928d 04 +xc*(
     &   1.322429746943889d 03 +xc*(
     &   4.427183005616853d 01 +xc*(
     &  -8.630738701658902d 00 +xc*(
     &   4.279183401413811d-01 )))))))
      q(xc)=
     &   6.335034386662446d 06 +xc*(
     &   2.144094496614083d 04 +xc*(
     &  -4.182226973512467d 03 +xc*(
     &   3.609995316780635d 02 +xc*(
     &  -1.346978283534684d 01 ))))
      t(xc)=
     &   9.999999957157490d-01+xc*(
     &  -1.666665796975878d-01+xc*(
     &   8.333050613721043d-03+xc*(
     &  -1.980904608528695d-04+xc*
     &   2.605165638554101d-06 )))
      phi2=phi**2
      c1=dlamb*phi
      c2=(c1*t(phi2))**2
      x=dlamb*a(phi2)*t(c2)
      y=dlamb*c1   *b(phi2)*t(.25d0*c2)**2 + phi*q(phi2)
      return
      end
c--------------------------------------------------------------------------
      subroutine utmfwd(phi,dlam,x,y)
c
c  SUBROUTINE UTMFWD
c
c  PURPOSE..
c    TO DETERMINE BASIC UTM COORDIATES FROM LATITUDE
c    AND LONGITUDE INPUT.  CLARK 1866 SPHEROID CONSTANTS.
c
c  USAGE..
c    CALL UTMFWD(PHI,DLAM,X,Y,IER)
c
c  PARAMETER DESCRIPTION..
c    PHI    - INPUT LATITUDE IN DOUBLE PRECISION RADIANS.
c             DABS(PHI).LT.1.3963 (80 DEGREES).
c    DLAM   - INPUT LONGITUDE IN DOUBLE PRECISION RADIANS.
c             DABS(DLAM).LT.0.06109 (3.5 DEGREES).
c             NOTE.. DLAM MEASURED FROM CENTRAL MERIDIAN.
c    X      - PROJECTED COORDINATE IN METERS FROM THE
c             CENTRAL MERIDIAN (DOUBLE PRECISION).
c    Y      - PROJECTED COORDINATE IN METERS FROM THE
c             EQUATOR (DOUBLE PRECISION).
c    IER    - ERROR RETURN CODE.
c             IER=0 SUCCESSFUL COMPLETION.
c             IER.NE.0 ERROR IN PHI OR DLAM INPUT.
c
c  REMARKS..
c    THIS ROUTINE IS ACCURATE TO 1 METER.  ABSOLUTE
c    ERROR LESS THAN .5 METERS.
c    NEAREST CENTRAL MERIDIAN MAY BE DETERMINED EXTERNALLY
c    BY..
c    CN=DSIGN(10800D0,LON)+AINT(SNGL(LON/21600D0))*21600D0
c      WHERE LON IS LONGITUDE IN DOUBLE PRECISION SECONDS.
c    FALSE EASTING (500000 METERS) AND NORTHING (0 FOR
c    NORTHERN LATITUDES, 10000000 METERS FOR SOUTHERN
c    LATITUDES) FOR TRUE UTM COORDINATES MUST BE COMPUTED
c    EXTERNALLY.
c
c  SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED..
c    NONE
c
c  DEVELOPED AND CODED BY..
c    GERALD IAN EVENDEN
c    U. S. GEOLOGICAL SURVEY
c
c
          double precision phi,dlam,x,y,dl2,p
c
c  CHECK INPUT PRAMETERS..
          p=phi*phi
          dl2=dlam*dlam
c         if (p .le.1.94965360d0.and.dl2.le.3.7319881d-3)
c             go to 10
c         ier=-1
c         return
c
10        ier=0
c
c  DETERMINE PROJECTED COORDINATES..
          y=phi*(
     &          6332500.47d0+p*(21431.67d0+p*(-4179.269d0+p*(
     &          359.981d0-p*13.267d0)))
     &      -dl2*(-3187827d0+p*(2114440d0+p*(-414363.5d0+p*(
     &          36344.6d0-p*1420.3d0)))
     &       -dl2*(1334935d0+p*(-2356027d0+p*(1371758d0-p*
     &          267852d0)))))
          x=dlam*(6375655.2d0+p*(-3166253.6d0+p*(247800.26d0+p*(
     &          -3569.65d0+p*(-617.35d0+p*50.89d0))))
     &       -dl2*(-1069479d0+p*(2661562d0+p*(-1785956d0+p*(
     &         485045d0-p*52022d0)))))
c
          return
c
          end
c-------------------------------------------------------------------------
          subroutine merctr(lat,long,x,y)
          double precision lat,long,x,y,a,halfpi,b2da2,z,phi
          data a/6378206.d0/,halfpi/1.57079632679489d0/,
     &      b2da2/9.932315290818186d-1/
          x=a*long
c
c         compute z/2
c
          z=0.5d0*(halfpi-datan(b2da2*dsin(lat)/dcos(lat)))
          y=a*dlog(dcos(z)/dsin(z))
          return
          end
c--------------------------------------------------------------------------
          subroutine lambrt(lat,long,x,y)
          double precision lat,long,x,y,z,k,l,r,b2da2,halfpi,theta
          data k/1.245265545827296d7/,l/6.304998344052458d-1/,
     &      b2da2/9.932315290818186d-1/,halfpi/1.57079632679489662d0/
c
c         compute z/2
c
          z=0.5d0*(halfpi-datan(b2da2*dsin(lat)/dcos(lat)))
c
c         compute r
c
          r=k*(dsin(z)/dcos(z))**l
c
c         compute coordinates
c
          theta=l*long
          x=r*dsin(theta)
          y=-r*dcos(theta)
          return
          end
c--------------------------------------------------------------------------
          subroutine albers(ylat,xlon,x,y)
          common/albcom/n,rho1sq,sinbt1,twoc2n
          double precision ylat,xlon,x,y,n,rho1sq,sinbt1,twoc2n,
     &      nus,nals,nhaw,rho295,rho55,rho8,tcnus,tcnals,tcnhaw,
     &      a1,b,c1,d,e1,f1,g,h,theta,rho,sinbet,sinphi,s2,
     &      sbt295,sbt55,sbt8
          data a1/9.954804334645587d-1/,b/4.492024607745888d-3/,
     &      c1/2.736435989866449d-5/,d/1.763992166249299d-7/,
     &      e1/1.160814577272288d-9/,f1/7.714265487727804d-12/,
     &      g/5.154557173568170d-14/,h/3.455700205911349d-16/,
     &      nus/6.02903493787094d-1/,nals/8.627447947235633d-1/,
     &      nhaw/2.241096394314637d-1/,rho295/8.49196923967458d13/,
     &      rho55/1.806308673895081d13/,rho8/7.943986660586285d14/,
     &      tcnus/1.346470921892769d14/,tcnals/9.409410848453636d13/,
     &      tcnhaw/3.622298555079059d14/,sbt295/4.907351753179611d-1/,
     &      sbt55/8.1792905450587868d-1/,sbt8/1.385562096187223d-1/
          sinphi=dsin(ylat)
          s2=sinphi*sinphi
          sinbet=sinphi*(a1+s2*(b+s2*(c1+s2*(d+s2*
     &      (e1+s2*(f1+s2*(g+s2*h)))))))
          rho=dsqrt(rho1sq+twoc2n*(sinbt1-sinbet))
          theta=n*xlon
          x=rho*dsin(theta)
          y=-rho*dcos(theta)
          return
c
c set up constants
          entry setalb(iproj)
          if(iproj-6)500,510,520
500       n=nus
          rho1sq=rho295
          sinbt1=sbt295
          twoc2n=tcnus
          return
510       n=nals
          rho1sq=rho55
          sinbt1=sbt55
          twoc2n=tcnals
          return
520       n=nhaw
          rho1sq=rho8
          sinbt1=sbt8
          twoc2n=tcnhaw
          return
          end
c
c----------------------------------------------------------
c
c  SUBROUTINE UTMINV
c
c  PURPOSE..
c    TO DETERMINE LATITUDE AND LONGITUDE FROM UTM
c    COORDINATES.  CLARK 1866 SPHEROID CONSTANTS.
c
c  USAGE..
c    CALL UTMINV(PHI,DLAM,X,Y,IER)
c
c  DESCRIPTION OF PARAMETERS..
c    PHI    - OUTPUT LATITUDE IN DOUBLE PRECISION RADIANS.
c    DLAM   - OUTPUT LONGITUDE IN DOUBLE PRECISION RADIANS
c             FROM CENTRAL MERIDIAN.
c    X      - DOUBLE PRECISION UTM METERS FROM THE CENTRAL
c             MERIDIAN.  FALSE EASTING MUST BE
c             REMOVE EXTERNALLY.
c             DABS(X).LE.390000 METERS.
c    Y      - DOUBLE PRECISION UTM METERS FROM THE EQUATOR.
c             FALSE NORTHING (SOUTHERN LATITUDES) MUST BE
c             REMOVED EXTERNALLY.
c             DABS(Y).LE.8900000 METERS.
c    IER    - ERROR RETURN CODE.
c             IER=0 SUCCESSFUL COMPLETION.
c             IER.NE.0 ERROR IN X OR Y INPUT.
c
c  REMARKS..
c    ABSOLUTE ERROR OF PHI AND DLAM LESS THAN 10**-8
c    RADIANS.
c
c  SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED..
c    NONE
c
c  DEVELOPED AND CODED BY..
c    GERALD IAN EVENDEN
c    U. S. GEOLOGICAL SURVEY
c
c
c--------------------------------------------------------------------------
          subroutine utminv(phi,dlam,x,y)
c
          double precision dlam,phi,x,y,x1,x2,y1,y2,
     &       ph1,ph2,dt
c
          equivalence (dt,x1,y1),(x2,y2)
c
          y1=y*1d-7
          y2=y1*y1
c         if (y2.le..7921d0.and.dabs(x).le.390000d0) go to 10
c         ier=-1
c         return
c
c  COMPUTE INVERSE PROJECTION
c 10       ier=0
          ph1=y1*(1.579155027d0+y2*(-.0133273056d0+y2*(
     &         .006813653d0+y2*(-.0018179637d0+y2*(
     &         .000352375d0-y2*.0000494195d0)))))
          ph2=ph1*ph1
          x1=x*1d-7/(.637565512d0+ph2*(-.318782754d0+ph2*(
     &         .0265652212d0+ph2*(-.0008854896d0+ph2*(
     &         .00001579516d0-ph2*.00000016787d0)))))
          x2=x1*x1
          dlam=x1*(.99999998d0+ph2*(-.00338386d0+ph2*(
     &         .001120504d0+ph2*(-.000143997d0+ph2*
     &         .0000081386d0)))
     &      -x2*(.167804d0+ph2*(.162647d0+ph2*(
     &         -.054578d0+ph2*(.0073115d0-ph2*.00045341d0)))
     &      -x2*(.04297d0+ph2*(.1402d0-ph2*.03291d0))))
          dt=-ph1*x2*(.4999998d0+ph2*(-.08671332d0+ph2*(
     &         .00584112d0+ph2*(-.000440204d0+ph2*
     &         .000031613d0)))
     &      -x2*(.20928d0+ph2*(-.0028705d0-ph2*.0081252d0)))
          phi=ph1+dt*(1.0068147d0+ph2*(-.51022062d0+ph2*(
     &         .047620417d0+ph2*(-.003100987d0+ph2*(
     &         .000282232d0-ph2*.0000185462d0))))
     &      +dt*(ph1*(-.51358d0+ph2*(.354256d0+ph2*(
     &         -.0770704d0+ph2*.0073699d0)))))
c
          return
c
          end
c--------------------------------------------------------------------------
          subroutine pstereo(ylat,xlon,x,y)
c
c         polar stereographic projection
c
          double precision ylat,xlon,x,y,sinc,sine,tane1,rad45,e,ee,r,
     &      tanz,polarc,a,dlat2,tane2,dlat,scale
          data e/.08182d0/,r/6378160d0/,rad45/.7854d0/,
     &      ee/.04091d0/,polarc/12713645.1d0/
          dlat=dabs(ylat)
c         polarc=2.d0*r/dsqrt(1.d0-e*e)*((1.d0-e)/(1.d0+e))**ee
          sinc=e*dsin(dlat)
          sine=(1.d0+sinc)/(1.d0-sinc)
          dlat2=dlat*.5d0
          tane2=rad45+dlat2
          tane1=cos(tane2)/sin(tane2)
          tanz=tane1*(sine**ee)
          a=tanz*polarc
          y=-(a*dcos(xlon))
          x=a*dsin(xlon)
          if(ylat.lt.0.d0) y=-y
c         scale=a*dsqrt(1.d0-e*e*dsin(dlat)**2)/dcos(dlat)/r
          return
          end
c--------------------------------------------------------------------------
      function   decdeg(deg,min)
c     Converts degrees and minutes to decimal degrees...
      real decdeg, deg, min
      decdeg = deg + sign(min,deg)/60.
      return
      end
