c  CHESS
c  Combination of programs PRECHESS coded by Tien Grauch and MCHESS
c  coded by Tom Hildenbrand. Program uses the chessboard method of
c  Lin Cordell, modified with Hildenbrand's sliding-rule filter.
c  Reference to the method:
c     Cordell, Lindrith, 1985, Techniques, applications, and 
c        problems of analytical continuation of New Mexico
c	 aeromagnetic data between arbirary surfaces of very
c        high relief [abs.]:  Proceedings of the International
c        Meeting on Potential Fields in Rugged Topography,
c        Institute of Geophysics, University of Lausanne,
c        Switzerland, Bulletin No. 7, p. 96-99.	 
c
c  The chessboard method interpolates between parallel levels or
c  surfaces in order to estimate level-to-surface, surface-to-level or
c  surface-to-surface continuation.  (Because it is an approximate method,
c  users desiring surface-to-surface continuation that involves parallel
c  surfaces will achieve the same but faster results if 
c  program NFFTFIL is used in up/downward continuation mode.  The
c  amount to continue is the distance between the parallel surfaces.)
c
c  The level-to-surface option of the chessboard method is straightforward,
c  but the other two options require fooling the program.  In these
c  two cases, the program constructs a surface grid p to enter for the
c  surface-to-level operation in the following way:
c                    p = 2*l - surf,
c  where l=level to continue to, surf=surface where data are now.
c   OR for the surface-to-surface operation in the following way:
c	             p = newsurf - oldsurf + l,
c  where the data are being continued from oldsurf to newsurf, and l
c  is a level somewhere in between the surfaces to fool CHESS with.
c  (although it is a lie, it needs to be told)
c
      dimension work(2048),a1(2,1024,16),fz(1024,25)
      dimension z(1024)
      character crfile*12,iunit*1
      character*50 tname,coname,fname,flnam,fl2nam
      common/main/id(14),pgm(2),nz,yo,xo,dx,dy,iw,kr,ny,nadd,ir
      common/parm1/fname,coname,zlev(25),nlev,inocal,barlev,ifac,
     1tname
      data lnri/16/,dvaltst/1.0e+28/,ndim/1024/,dval/'ffff7fff'x/
	ir=5
	iw=6
	kr=11
c sm is slop allowed when comparing grid specs
	sm=.000001
	print*                              
5 	write(6,888)
888	format(' 1 - level to surface continuation'/
     1 ' 2 - surface to level continuation'/' 3 - ',
     2 'surface to surface continuation'/5x,'Enter option: ',$)
	read*,iopt
	if(iopt.le.0.or.iopt.gt.3) go to 5
	go to (7,8,8), iopt
7     write(iw,803)
803	format(' Enter input magnetic data (observed on level)')
	read(ir,10) fname
	open(17,file=fname,status='old',form='unformatted',readonly)
      read(17)id,pgm,ncol,nrow,nz,xo,dx,yo,dy
	close(17)
	if(ncol.gt.ndim) then
		write(iw,810), ndim
810		format(' Max no. cols = ',i4)
		stop
	endif
      write(iw,804)
804	format(' Enter surface grid that you wish to continue to')
      read(5,10) flnam
10    format(a50)
	open(10,file=flnam,status='old',form='unformatted',readonly)
      read(10)id,pgm,ncol2,nrow2,nz,xo2,dx2,yo2,dy2
	if(abs(xo2-xo).gt.sm.or.abs(yo2-yo).gt.sm) go to 995
	if(ncol2.ne.ncol.or.nrow2.ne.nrow) go to 995
	if(abs(dx2-dx).gt.sm.or.abs(dy2-dy).gt.sm) then
	  write(iw,807) flnam
807   format(' WARNING--dx and dy of file may not match input mag:'
     1 /a50)
	endif
	write(iw,811)
811     format(' Enter observ. level (in same units as surface ',
     1 'elev data): ',$)
	read*,barlev
      go to 110
8     write(iw,823)
823	format(' Enter input magnetic data (observed on surface)')
	read(ir,10) fname
	open(17,file=fname,status='old',form='unformatted',readonly)
      read(17)id,pgm,ncol,nrow,nz,xo,dx,yo,dy
	close(17)
	if(ncol.gt.ndim) then
		write(iw,810), ndim
		stop
	endif
      write(iw,805)
805	format(' Enter surface grid that data are observed on: ')
      read(5,10) flnam
	open(10,file=flnam,status='old',form='unformatted',readonly)
      read(10)id,pgm,ncol2,nrow2,nz,xo2,dx2,yo2,dy2
	if(abs(xo2-xo).gt.sm.or.abs(yo2-yo).gt.sm) go to 995
	if(ncol2.ne.ncol.or.nrow2.ne.nrow) go to 995
	if(abs(dx2-dx).gt.sm.or.abs(dy2-dy).gt.sm) then
	  write(iw,807) flnam
	endif
	if(iopt.eq.3) go to 110
	write(iw,812)
812	format(' Enter level to continue to (in same units as observ.',
     1 ' surf. data): ',$)
	read*,barlev
110	write(6,801) 
801	format(' Enter the constant to convert surf. data into',
     1 ' the same units as dx & dy '/' with z positive up',
     1 ' (e.g. .001 converts meters a.s.l. to km) : ',$)
	read*,conv
	call minmax(10,ncol,nrow,z,conv,xmin,xmax,xmean)
	write(6,891) xmin,xmax,xmean
891	format(' Surface minimum= ',g14.6,' maximum= ',g14.6,
     1 ' mean= ',g14.6/
     2 '  in same units as dx & dy'/)
	rewind 10
      read(10)id,pgm,ncol,nrow,nz,xo,dx,yo,dy
	go to (15,15,12),iopt
12	write(iw,806)
806	format(' Enter file of surface to continue to.')
       read(5,10) fl2nam
	open(11,file=fl2nam,status='old',form='unformatted',readonly)
       read(11)id,pgm,ncol2,nrow2,nz,xo2,dx2,yo2,dy2
	if(abs(xo2-xo).gt.sm.or.abs(yo2-yo).gt.sm) go to 995
	if(ncol2.ne.ncol.or.nrow2.ne.nrow) go to 995
	if(abs(dx2-dx).gt.sm.or.abs(dy2-dy).gt.sm) then
	  write(iw,807) fl2nam
	endif
	write(6,801) 
	read*,conv3
	call minmax(11,ncol2,nrow2,work,conv3,xmin2,xmax2,xmean2)
	write(6,891) xmin2,xmax2,xmean2
c set arbitrary barlev between the two surfaces (part of fooling schess)
	txmin=amin1(xmin,xmin2)
	txmax=amax1(xmax,xmax2)
	barlev= txmin + (txmax - txmin) * 0.5e0
	rewind 11
        read(11)id,pgm,ncol2,nrow2,nz,xo2,dx2,yo2,dy2
15	barlev=barlev*conv
	barlev2=barlev*2.e0
	write(iw,817)
817	format(' Enter no. of levels for interpolation: ',$)
	read(ir,*) nlev
	if(nlev.lt.7.or.nlev.gt.25) then
		write(iw,818)
818		format(' Max no. levels=25, minimum=7')
		go to 15
	endif
	write(iw,815)
815	format(' How should data be filtered in the downward ',
     1  'continuation?'/,
     2  5x,' 0 - no filtering',/5x,' 1 - slight lowpass',/5x,
     3  ' 2 - moderate lowpass',/5x,' 3 - severe lowpass'/,
     4  ' Enter filtering factor: ',$)
	read(ir,*) ifac
	ifac=ifac-1
	write(iw,816)
816	format(' no. of grid points by which to augment rows '
     1 '& cols in FFT (nadd): ',$)
	read(ir,*) nadd
	write(iw,813)
813	format(' Enter output file name')
	read 10,tname
	write(iw,814)
814	format(' Enter title')
	read 802,id
802	format(14a4)
	open(12,file='mchessz.tmp',status='new',form='unformatted')
       write(12)id,pgm,ncol,nrow,nz,xo,dx,yo,dy
	go to (20,30,130),iopt
c
c  LEVEL TO DRAPE
c
20	do 25 j=1,nrow
	read(10) dum,(z(i),i=1,ncol)
	do 22 i=1,ncol
	if(z(i).ge.dvaltst) go to 22
	z(i)=z(i)*conv
22      continue
	write(12) dum,(z(i),i=1,ncol)
25	continue
	go to 300
c
c  DRAPE TO LEVEL
c
30	xmin=dvaltst
	xmax=-dvaltst
c find xmin, xmax of new surface while creating it
	do 100 j=1,nrow
	read(10) dum,(z(i),i=1,ncol)
	do 50 i=1,ncol
	if(z(i).ge.dvaltst) go to 50
	z(i)=barlev2-(z(i)*conv)	
	if(z(i).lt.xmin) xmin=z(i)
	if(z(i).gt.xmax) xmax=z(i)
50      continue
	write(12) dum,(z(i),i=1,ncol)
100	continue
	go to 300
c
c  DRAPE TO DRAPE
c
130	xmin=dvaltst
	xmax=-dvaltst
c find xmin, xmax of new surface while creating it
	do 200 j=1,nrow
	read(10) dum,(z(i),i=1,ncol)
	read(11) dum,(work(i),i=1,ncol)
	do 150 i=1,ncol
	if(z(i).ge.dvaltst.or.work(i).ge.dvaltst) then
		z(i)=dval
		go to 150
	endif
	z(i)=(work(i)*conv3)-(z(i)*conv)+barlev
	if(z(i).lt.xmin) xmin=z(i)
	if(z(i).gt.xmax) xmax=z(i)
150      continue
	write(12) dum,(z(i),i=1,ncol)
200	continue
	go to 300
995	print*,'Grids mismatch'
300	close(11)
	close(10)
      close(12)
	call schess(ndim,xmin,xmax,xmean,z,work,a1,fz)
      stop
      end
c ***************************************************************
      subroutine minmax(iunit,ncol,nrow,z,conv,zmin,zmax,zmean)
c  finds min and max of file in converted units and stores converted
c  values back into z
c  the file is already attached to iunit and the header has been read
      dimension z(ncol),id(14),pgm(2)
      zmax=-1.e38
      zmin=1.e38
c
c  calculate min, max and mean and convert at same time
c
      sumz=0.0
      en=0.0
      do 20 j=1,nrow
      read(iunit,end=80,err=70) dum,z
      do 10 i=1,ncol
c calculate mean
      if(z(i).ge.1.e+38) go to 10
      z(i)=z(i)*conv
      sumz=sumz+z(i)
      if(z(i).gt.zmax) zmax=z(i)
      if(z(i).lt.zmin) zmin=z(i)
      en=en+1.0
10    continue
20    continue
	if(en.eq.0.0) stop 'found no valid points'
      zmean=sumz/en
	return
70 	print*,'Error in reading the file'
	stop
80	print*,'Found end of file, check pass of ncol & nrow'
	print*,'Has the header been read?'
	stop
	end
c****************************************************************
c    automatic version of chessboard of lin cordell
c    programmed by tom hildenbrand
	subroutine schess(ndim,xmin,xmax,xmean,z,work,a1,fz)
      dimension a1(2,ndim,16),work(2*ndim),title(14),pgmd(2)
      dimension fz(ndim,25),z(ndim)
      character*50 tname,coname,fname
	character crfile*12
      common/main/id(14),pgm(2),nz,yo,xo,dx,dy,iw,kr,ny,nadd,ir
      common/parm1/fname,coname,zlev(25),nlev,inocal,barlev,ifac,tname
      data lnri/16/,dval/1.0e+28/
   20 format(a50)
      if(barlev.lt.xmin) go to 134
      zlev(2)=xmin-0.001
      cmax=barlev
      if(barlev.lt.xmax)cmax=xmax
      xcal=float(nlev-3)
      xlevl=cmax-xmin+0.01
      xdel=xlevl/xcal
      zlev(1)=zlev(2)-xdel/3.
      inocal=nlev-1
      do 138 i=3,nlev-2
      zlev(i)=zlev(i-1)+xdel
138   continue
      go to 139
134   cmax=xmax
      zlev(2)=(xmin-barlev)/2.+barlev
      zlev(1)=barlev
      xcal=float(nlev-4)
      xlevl=xmax-xmin+0.0001
      xdel=xlevl/xcal
      inocal=1
      zlev(3)=xmin
      do 135 i=4,nlev-2
      zlev(i)=zlev(i-1)+xdel
135   continue
139   if(barlev.lt.xmax) go to 137
      zlev(nlev)=xdel+barlev
      zlev(nlev-1)=barlev
      go to 140
137   dif=xmax-barlev
      if(dif.gt.xdel/2.) go to 149
      n=nlev-2
      zlev(nlev-1)=barlev+xdel
      zlev(nlev)=barlev +2.*xdel
      inocal=n
      go to 146
149   zlev(nlev)=xmax+xdel
      zlev(nlev-1)=xmax+0.001
      if(inocal.eq.1) go to 140
      n=nlev-1
144   n=n-1
      inocal=n
      dif=abs(barlev-zlev(n))
      if(dif.lt.xdel/2.or.n.eq.3) go to 146
      go to 144
146   zlev(n)=barlev
140   write(iw,151) (zlev(j),j=1,nlev)
151   format(' levels are as follows:'/,5(5(g15.4,1x),/))
c  read header record of input file.
  180 open(11,status='old',form='unformatted',file=fname)
c  note that x & y have been swicthed from normal (usgs) grid specficati
c  ...convention used is x-north y-east and z-down
      read(kr)title,pgm,n2,n1,nz,yop,dy,xop,dx
      no=inocal+9
      write(crfile,161)no
161   format('mchess',i2,'.tmp')
      open(21,status='new',form='unformatted',file=crfile)
      write(21)title,pgm,n2,n1,nz,yop,dy,xop,dx
      do 160 i=1,n1
      read(kr)dum,(z(j),j=1,n2)
      write(21)dum,(z(j),j=1,n2)
160   continue
      close(11)
      close(21)
      open(11,status='old',form='unformatted',file=fname)
      read(kr)title,pgm,n2,n1,nz,yop,dy,xop,dx
      pgm(1)='auto'
      pgm(2)='ches'
      nx=n1
      ny=n2
      l=lnri+1
      lnri21=lnri/2+1
c  set no. of rows for fft: need m=l*2**k, m.gt. or .eq. nx+2*nadd.
c  m=no. of rows, l=no. from 9-16, k=interger.
      n1=n1+2*nadd
  190 l=l-1
      if(l.lt.lnri21) go to 260
      mr=n1/l+0.0000001
      k=1
      idiv=2
  200 iquot=mr/idiv+0.0000001
      if(iquot.lt.idiv) go to 210
      k=k+1
      mr=iquot
      go to 200
  210 k=k+1
      m=l*2**k
  220 mtest=l*2**(k-1)
      if(mtest.lt.n1) go to 230
      k=k-1
      m=mtest
      if(k.eq.0) go to 230
      go to 220
  230 lnxa=m-n1
      if(l.ne.lnri) go to 250
      nxa16=lnxa
  240 nri=l
      nxa=lnxa
      go to 190
  250 if(lnxa.ge.nxa) go to 190
      go to 240
  260 n1=n1+nxa
c  check to see if row block size of 16 will be more efficient
      n116=n1-nxa+nxa16
      ntest=0.9*n116
      if(ntest.gt.n1.or.n116.gt.1028) go to 270
      n1=n1-nxa+nxa16
      nxa=nxa16
      nri=16
  270 n2=n2+2*nadd
      if(n1.gt.1028.or.n2.gt.1028) go to 320
      nxa=nxa+2*nadd
      id2=n1
      if(n2.gt.n1)id2=n2
      id2=2*id2
c  input parameters for fft are set: call main program.
	call sfftfil(a1,work,nx,nri,nxa,n1,n2,id2)
c  Interpolates a function between levels, given a sequence of grids at 
c  successive vertical levels. Z direction + downward, input grids
c  in monotonic-increasing-Z sequence only.  All grid specs identical.
c  Extra top and bottom level grids are created.
      open(10,status='old',form='unformatted',file='mchessz.tmp')
      read(10) title,pgmd,ncol,nrow,nz,x0,dx,y0,dy
      do 4 i=1,nlev
      no=9+i
      write(crfile,888)no
888   format('mchess',i2,'.tmp')
      iunit=i+11
      open(unit=iunit,status='old',form='unformatted',file=crfile)
      read(iunit) title,pgmd,ncol1,nrow1,nz1,x01,dx1,y01,dy1
4     continue
      open(11,status='new',form='unformatted',file=tname)
      write(11) id,pgm,ncol,nrow,nz,x0,dx,y0,dy
      do 400 irow=1,nrow
      read(10)dum,(z(k),k=1,ncol)
      do 19 n=1,nlev
      iunit=n+11
19    read(iunit) dum,(fz(j,n),j=1,ncol)
      do 18 j=1,ncol
      if(z(j).gt.dval) go to 18
      if(fz(j,4).lt.dval) go to 665
      z(j)=fz(j,4)
      go to 18
665   zj=z(j)
c  Note: Assume 1<iclose<nlev, no checks !
      do 7 i=2,nlev-1
      if(zlev(i).gt.zj) go to 8
      if(zlev(i).lt.zj) go to 7
      z(j)=fz(j,i)
      go to 18
7     continue
8     iclose=i-1
      Z2=zlev(iclose)
      x1=zlev(iclose-1)-Z2
      x3=zlev(iclose+1)-Z2
      x4=zlev(iclose+2)-Z2
      x1sq=x1*x1
      x3sq=x3*x3
      x4sq=x4*x4
      f2=fz(j,iclose)
      if(x1.eq.0..or.x3.eq.0..or.x4.eq.0.) go to 620
      q1=(fz(j,iclose-1)-f2)/x1
      q3=(fz(j,iclose+1)-f2)/x3
      q4=(fz(j,iclose+2)-f2)/x4
      denom=(x3*x4sq-x4*x3sq)-x1*(x4sq-x3sq)+x1sq*(x4-x3)
      if(denom.eq.0.) go to 630
      a=(q1*(x3*x4sq-x4*x3sq)-x1*(q3*x4sq-q4*x3sq)
     1+x1sq*(q3*x4-q4*x3))/denom
      b=(q3*x4sq-q4*x3sq-q1*(x4sq-x3sq)+x1sq*(q4-q3))/denom
      c=(q4*x3-q3*x4-x1*(q4-q3)+q1*(x4-x3))/denom
      x=zj-Z2
      xsq=x*x
      z(j)=f2+a*x+b*xsq+c*xsq*x
      go to 18
620     write(6,640)irow,j
640     format(' x1,x3,or x4=0.: row & col= ',2i4)
      z(j)=0.
      go to 18
630   write(6,650)irow,j
650   format(' denom=0.: row & col= ',2i4)
      z(j)=0.
18    continue
      write(11)dum,(z(k),k=1,ncol)
400    continue
900   continue
      close(10,dispose='delete')
      close(11)
      do 21 i=1,nlev
      iunit=i+11
21    close(unit=iunit,dispose='delete')
      go to 350
c  errors resulting in job abortion.
  280 write(iw,290)
  290 format(' #parameter error detected---case run aborted')
      go to 350
  320 write(iw,330)nx,ny,n1,n2
  330 format(' #no. of extended rows or columns exceeds 1028:'/
     1' input no. of rows and columns='2i4/
     2' no. of rows and columns required for filtering=',2i4,/)
  340 close(11)
  350 write(iw,360)
  360 format(' end of job')
      stop
      end
c   subroutine 'sfftfil' removes flagged values from grid, transforms
c   data, calls routines to apply filter, and writes filtered grid
c***********************************************************************
          subroutine sfftfil(a1,work,nx,nri,nxa,n1,n2,id2)
          dimension a1(2,n2,nri),work(id2),jflag(2,20),title(14)
      character crfile*12
          character*50 coname,fname,fileo
          common/main/id(14),pgm(2),nz,yo,xo,dx,dy,iw,kr,ny,nadd,ir
          common/parm1/fname,coname,zlev(25),nlev,inocal,barlev,ifac,tname
          data iunita/10/,iunitb/12/,dval/'ffff7fff'x/
          data pi2/6.28318531/,pi/3.141592654/,iunitc/15/
      nlevel=1
      z=barlev-zlev(1)
      coname='mchess10.tmp'
      w1=0.
      w2=0.
      if(ifac.lt.0) go to 2
      del=(dx+dy)/2.
      flim=del*2.5
      fac=(ifac+1.5)*del
      wavlen=z*1.25+fac
      if(wavlen.lt.flim)wavlen=flim
      wdel=.5*del+0.25*z
      w1=wavlen-wdel
      w2=wavlen+wdel
      f4=1./w1
      f3=1./w2
      btan34=1./(f4-f3)
      go to 4
2     f4=0.
4         ncase=0
      n22=2*n2
c  compute beginning and end column & row in original grid.
   20     do 30 i=1,2
          do 30 j=1,n2
   30     a1(2,j,i)=0.0
          nap1=nadd+1
          n2a=n2-nadd
          nxap1=nxa*0.5+1
          fxap1=float(nxa)/2.+1.
          if(abs(fxap1-float(nxap1)).gt.0.0001)nxap1=nxap1+1
          n1a=nxap1-1+nx
c  remove flagged values and add '2*nadd' rows & columns to
c  reduce the effect of gibbs phenomena.
         open(10,access='direct',status='new',form='unformatted',
     1   file='slave1.tmp',recl=n22,maxrec=n1,
     1 organization='relative',recordtype='fixed')
          open(14,status='new',form='unformatted',file='flag.loc')
          irr=1
          iwr=0
          jr=0
   40     jr=jr+1
          if(jr.gt.1) go to 140
c   read 1st row & remove flagged values.
          read(kr)dum,(a1(1,i,1),i=nap1,n2a)
          read(kr)dum,(a1(1,i,2),i=nap1,n2a)
          nflag=1
          i=nap1
   50     jflag(1,1)=nap1
          if(a1(1,nap1,1).le.1.0e38) go to 90
          do 60 i=nap1,n2a
   60     if(a1(1,i,1).le.1.0e38) go to 70
          go to 1270
   70     jflag(1,1)=i
          aval=a1(1,i,1)
          do 80 ii=nap1,i-1
   80     a1(1,ii,1)=aval
   90     if(i.ge.n2a) go to 100
          do 100 ii=i+1,n2a
          if(a1(1,ii,1).le.1.0e38) go to 100
          jflag(2,nflag)=ii-1
          go to 110
  100     continue
          jflag(2,nflag)=n2a
          write(14)nflag,(jflag(1,j),jflag(2,j),j=1,nflag)
          go to 350
  110     aval=a1(1,ii-1,1)
          do 120 i=ii,n2a
          if(a1(1,i,1).le.1.0e38) go to 130
  120     a1(1,i,1)=aval
          write(14)nflag,(jflag(1,j),jflag(2,j),j=1,nflag)
          go to 350
  130     a1(1,i-1,1)=(a1(1,i-1,1)+a1(1,i,1))*0.5
          nflag=nflag+1
          jflag(1,nflag)=i
c   1st row completed: now extend grid & write to disk.
          go to 90
c   remove flagged values from remaining rows.
  140     if(jr.ge.nx) go to 150
          read(kr)dum,(a1(1,i,3),i=nap1,n2a)
          go to 170
  150     do 160 i=nap1,n2a
  160     a1(1,i,3)=dval
  170     nflag=1
          i=nap1
          jflag(1,1)=nap1
          if(a1(1,nap1,2).le.1.0e38) go to 220
          do 180 i=nap1,n2a
  180     if(a1(1,i,2).le.1.0e38) go to 190
          i=nap1
          jflag(1,1)=n2a+1
          if(a1(1,nap1,3).ge.1.0e38)a1(1,nap1,2)=a1(1,nap1,1)
          if(a1(1,nap1,3).le.1.0e38)a1(1,nap1,2)=(a1(1,nap1,1)+
     1    a1(1,nap1,3))*0.5
          go to 220
  190     jflag(1,1)=i
          ii=i
          do 210 irev=nap1,i-1
          ii=ii-1
          if(a1(1,ii,3).ge.1.0e38) go to 200
          a1(1,ii,2)=(a1(1,ii,1)+a1(1,ii+1,2)+a1(1,ii,3))*0.3333333
          go to 210
  200     a1(1,ii,2)=(a1(1,ii,1)+a1(1,ii+1,2))*0.5
  210     continue
  220     if(i.ge.n2a) go to 230
          do 230 ii=i+1,n2a
          if(a1(1,ii,2).le.1.0e38) go to 230
          jflag(2,nflag)=ii-1
          go to 240
  230     continue
          jflag(2,nflag)=n2a
          write(14)nflag,(jflag(1,j),jflag(2,j),j=1,nflag)
          go to 390
  240     do 290 i=ii,n2a
          if(i.eq.n2a) go to 250
          if(a1(1,i+1,2).le.1.0e38) go to 270
  250     if(a1(1,i,3).ge.1.0e38) go to 260
          a1(1,i,2)=(a1(1,i,1)+a1(1,i-1,2)+a1(1,i,3))*0.3333333
          go to 290
  260     a1(1,i,2)=(a1(1,i,1)+a1(1,i-1,2))*0.5
          go to 290
  270     if(a1(1,i,3).ge.1.0e38) go to 280
          a1(1,i,2)=(a1(1,i,1)+a1(1,i-1,2)+a1(1,i+1,2)+a1(1,i,3))*0.25
          go to 300
  280     a1(1,i,2)=(a1(1,i,1)+a1(1,i-1,2)+a1(1,i+1,2))*0.3333333
          go to 300
  290     continue
          write(14)nflag,(jflag(1,j),jflag(2,j),j=1,nflag)
          go to 390
  300     nflag=nflag+1
          i=i+1
          jflag(1,nflag)=i
          go to 220
c   extend row & write to disk.
  350     do 360 i=1,nap1
  360     a1(1,i,1)=a1(1,nap1,1)
          do 370 i=n2a,n2
  370     a1(1,i,1)=a1(1,n2a,1)
  380     iwr=iwr+1
          call swrda(iunita,iwr,a1,n22)
          if(iwr.eq.nxap1) go to 40
          go to 380
  390     do 400 i=1,nap1
  400     a1(1,i,2)=a1(1,nap1,2)
          do 410 i=n2a,n2
  410     a1(1,i,2)=a1(1,n2a,2)
          iwr=iwr+1
          call swrda(iunita,iwr,a1(1,1,2),n22)
          if(iwr.ge.n1a) go to 430
          do 420 i=nap1,n2a
          a1(1,i,1)=a1(1,i,2)
  420     a1(1,i,2)=a1(1,i,3)
          go to 40
  430     if(iwr.ge.n1) go to 450
  440     iwr=iwr+1
          call swrda(iunita,iwr,a1(1,1,2),n22)
          if(iwr.lt.n1) go to 440
  450 close(11)
      close(14)
  460     format(i5)
c  write headings
  470     if(ncase.eq.0) go to 590
      nlevel=nlevel+1
      if(nlevel.ne.inocal) go to 520
472   w1=0.
      w2=0.
       write(6,1115)nlevel,w1,w2
      ncase=1
      go to 470
520   z=barlev-zlev(nlevel)
      if(ifac.lt.0.or.z.lt.0.) go to 521
      wavlen=z*1.25+fac
      if(wavlen.lt.flim)wavlen=flim
      wdel=0.5*del+0.25*z
      w1=wavlen-wdel
      w2=wavlen+wdel
      f4=1./w1
      f3=1./w2
      btan34=1./(f4-f3)
521   no=nlevel+9
      write(crfile,560)no
560    format('mchess',i2,'.tmp')
      coname=crfile
590       pid=pi/180.
          rn1=1./(float(n1)*dx)
          rn2=1./(float(n2)*dy)
          c2=pi2*z
c  obtain complex fourier coefficients (f.f.t. a1).
          if(ncase.eq.0) go to 980
          jr=0
          do 970 i=1,n1
          jr=jr+1
          call srdda(iunitc,jr,a1,n22)
  970     call swrda(iunita,jr,a1,n22)
          go to 1000
  980     isign=-1
          open(12,access='direct',status='new',form='unformatted',
     1    file='slave2.tmp',recl=n22,maxrec=n1
     1 ,organization='relative',recordtype='fixed')
          call sfftmg(iunita,iunitb,n2,n1,a1,nri,isign,work)
          open(15,access='direct',status='new',form='unformatted',
     1    file='fftfil.cof',recl=n22,maxrec=n1,
     1 organization='relative',recordtype='fixed')
          jr=0
          do 990 i=1,n1
          jr=jr+1
          call srdda(iunita,jr,a1,n22)
  990     call swrda(iunitc,jr,a1,n22)
c  a1 array now holds complex fourier coefficients.
      if(zlev(1).eq.barlev) go to 472
 1000     n1p1=n1+1
          n2p1=n2+1
          nnh1=(n1/2)+1
          nnh2=(n2/2)+1
c  operating on a1 fourier coefficients.
          jr=0
          jmr=n1+1
 1010     jr=jr+1
          if(jr.gt.nnh1) go to 1110
          jj=jr-1
          x=float(jj)*rn1
          xsq=x*x
          call srdda(iunita,jr,a1,n22)
          do 1090 i=1,n2
          ii=i-1
          if(i.gt.nnh2)ii=-(n2p1-i)
          y=float(ii)*rn2
          xysq=xsq+y*y
 1030     call scont(c2,xysq,a1(1,i,1),a1(2,i,1),f3,f4,btan34)
1090  continue
          call swrda(iunita,jr,a1,n22)
          if(jr.eq.1) go to 1010
          jmr=jmr-1
          if(jr.eq.jmr) go to 1110
          a1(2,1,1)=-a1(2,1,1)
          iwr=n2p1
          do 1100 i=2,nnh2
          iwr=iwr-1
          x=a1(2,i,1)
          y=a1(1,i,1)
          a1(2,i,1)=-a1(2,iwr,1)
          a1(1,i,1)=a1(1,iwr,1)
          a1(1,iwr,1)=y
          a1(2,iwr,1)=-x
 1100     continue
          call swrda(iunita,jmr,a1,n22)
          go to 1010
c  a1 array now holds complex coefficients operated on by filter
c  compute desired anomaly = inv. f.f.t. of a1
 1110     isign=1
      write(iw,1115)nlevel,w1,w2
 1115 format(' completed continuing to level ',i3,' with w1&w2=',2f10.2)
          call sfftmg(iunita,iunitb,n2,n1,a1,nri,isign,work)
          area=1./float(n1*n2)
c  a1 array now holds computed anomaly on datum = z.
c  create standard output file
 1120     open(11,status='new',form='unformatted',file=coname)
          write(kr)id,pgm,ny,nx,nz,yo,dy,xo,dx
          open(14,status='old',form='unformatted',
     1    file='flag.loc')
          dum=0.0
          iwr=0
          do 1210 jr=nxap1,n1a
          iwr=iwr+1
          call srdda(iunita,jr,a1,n22)
          read(14)nflag,(jflag(1,j),jflag(2,j),j=1,nflag)
          j=1
c   restore shape of original grid (i.e. insert flagged values).
          do 1160 i=nap1,n2a
          if(i.lt.jflag(1,j)) go to 1130
          if(i.eq.jflag(2,j)) go to 1140
          go to 1150
 1130     a1(1,i,1)=dval
          go to 1160
 1140     j=j+1
          if(j.le.nflag) go to 1150
          j=j-1
          jflag(1,j)=1028
 1150     a1(1,i,1)=a1(1,i,1)*area
 1160     continue
          go to 1190
 1170     do 1180 i=nap1,n2a
 1180     a1(1,i,1)=a1(1,i,1)*area
 1190     write(kr)dum,(a1(1,i,1),i=nap1,n2a)
 1210     continue
          close(11)
          ncase=1
      if(nlevel.eq.nlev) go to 1250
      close(14)
          go to 470
1250       close(14,disp='delete')
      close(15,disp='delete')
        close(10,disp='delete')
          close(12,disp='delete')
          go to 1290
 1270     write(6,1280)
 1280     format(' #entire first row is flagged values-program aborted')
          close(11)
          close(10)
          close(14)
 1290     return
          end
c***********************************************************************
      subroutine sfftmg(unita,unitb,n,m,f,l,isign,work)
c     performs 2-dimensional fast fourier transform using disk
c     arguments -
c       unita - unit number of open random access disk file containing
c               one complex data row per record.
c               file is used for input to fftmg and output
c               of the transform from fftmg.
c       unitb - unit number of open random access work file
c               the same as the file of unita.
c       n     - number of y columns = length of x rows.
c       m     - number of x rows = length of y columns,
c               must be of the form l*2**k (k integer).
c       f     - complex work array for core manipulations.
c       l     - number of x rows which can be held in f.
c       isign - +1 or -1 for forward or reverse transform.
c       work  - work array for in-core transform routine sfourt.
c               work must be complex, of length max(n,l).
c   developed and coded by ray watts--usgs,denver
c   note: randin,randou,inset, and outset are entry points
c   in subroutine sdummy
c***********************************************************************
      integer unita,unitb,output,ndim(2)
      complex f(n,l),twidle,twidlf,w,work(1)
      nfourt=2*n*l
      nwork=2*n
      if(l.gt.n)nwork=2*l
c     **********************************************************
c     *    the number of disk accesses (reads+writes) used by  *
c     *    this program is m*(4*log2(m/l)+3).                  *
c     **********************************************************
c     --------------------------------------------------------
c     check arguments
      if(l*(m/l).ne.m)stop 'l not a factor of m in fftmg'
      ml=m/l
      mt=1
   10 if(mt-ml)20,40,30
   20 mt=2*mt
      go to 10
   30 stop 'm/l not a power of 2 in fftmg'
   40 continue
c     ------------------------------------------------------------
c     -------------------------------------------------------------
c     start processing
c     skip decimation if l .eq. m
      if(l.eq.m)go to 80
c     set starting input and output units
      input=unita
      output=unitb
c     set starting blocksize for decimation in input domain
      iblock=m
c     -------------------------------------------------------
c     decimate this blocksize
c     set up sub-block size
   50 isub=iblock/2
c     set up for random i/0
      call inset(input)
      call outset(output)
c     for each block of this size
      do 60 iblst=1,m,iblock
c     copy even members into first sub-block, odd members into second
      do 60 i=1,isub
      call randin(n,f,2*(i-1)+iblst)
      call randou(n,f,i-1+iblst)
      call randin(n,f,2*i-1+iblst)
   60 call randou(n,f,i-1+iblst+isub)
c     swap input/output functions
      inputt=input
      input=output
      output=inputt
c     next smaller blocksize
      iblock=isub
c     loop if not done
      if(iblock.gt.l)go to 50
c     decimation complete
c     ---------------------------------------------------------
c     ---------------------------------------------------------
c     ensure that data set is back on unita
      if(input.eq.unita)go to 80
      call inset(input)
      call outset(output)
      do 70 i=1,m
      call randin(n,f,i)
   70 call randou(n,f,i)
   80 continue
c     --------------------------------------------------------
c     start transform.
c     set block size
      iblock=l
c     set up for random i/o
      call inset(unita)
      call outset(unita)
c     run through the blocks
      do 100 istart=1,m,iblock
      into=1
c     run through the records in a block
      do 90 irecrd=istart,istart+iblock-1
c     read the record
      call randin(n,f(1,into),irecrd)
c     proceed with next record in block
   90 into=into+1
c     fourier transform in both directions
      ndim(1)=n
      ndim(2)=l
      call sfourt(f,ndim,2,isign,1,work,nfourt,nwork)
c     ******************************************************
c     write the block back onto disk
      ifrom=1
      do 100 irecrd=istart,istart+iblock-1
      call randou(n,f(1,ifrom),irecrd)
  100 ifrom=ifrom+1
c     if l.eq.m, the transform is now complete.
      if(l.eq.m)return
c     done with first-step processing
c     ---------------------------------------------------------
c     finish the processing by pairs of rows, applying twiddle factors
c     set the twiddle generator
      twidlf=cexp(cmplx(0.,3.141593*isign/(iblock)))
c     run through the blocks by pairs
  110 do 140 istart=1,m,iblock*2
c     start the twiddle factor
      twidle=(1.,0.)
c     scan the block pair
      do 130 irecrd=istart,istart+iblock-1
      jrecrd=irecrd+iblock
      call randin(n,f,irecrd)
      call randin(n,f(1,2),jrecrd)
c     combine, using twiddle factor
      do 120 i=1,n
      w=twidle*f(i,2)
      f(i,2)=f(i,1)-w
  120 f(i,1)=f(i,1)+w
c     write back onto disk
      call randou(n,f,irecrd)
      call randou(n,f(1,2),jrecrd)
c     increment the twiddle factor
  130 twidle=twidle*twidlf
c     go on to next block pair
  140 continue
c     increase the block size, change twiddle increment
      iblock=2*iblock
      twidlf=csqrt(twidlf)
      if(isign*aimag(twidlf).lt.0)twidlf=-twidlf
c     check if done
      if(iblock.lt.m)go to 110
c     ----------------------------------------------------------
c     done processing
      end
c***********************************************************************
      subroutine sdummy
c     random access i/o routine
c***********************************************************************
      integer out
      complex f(n),g(n)
c     entry to receive input setup
      entry inset(iin)
      in=iin
      return
c     entry to receive output setup
      entry outset(iout)
      out=iout
      return
c     entry to do input
      entry randin(n,f,nrec)
      read(in'nrec)f
      return
c     entry to do output
      entry randou(n,g,nrec)
      write(out'nrec)g
      return
      end
c***********************************************************************
      subroutine sfourt(data,nn,ndim,isign,iform,work,nfourt,nwork1)
c  note that 'nfourt' and 'nwork1' in subroutine call statement are
c  revised features of 'sfourt'. they are used as dynamic dimensioning
c  for the 'data' and 'work' arrays
c     the cooley-tukey fast fourier transform in usasi basic fortran
c     transform(j1,j2,) = sum(data(i1,i2,)*w1**((i1-1)*(j1-1))
c                                 *w2**((i2-1)*(j2-1))*,),
c     where i1 and j1 run from 1 to nn(1) and w1=exp(isign*2*pi*
c     sqrt(-1)/nn(1)), etc.  there is no limit on the dimensionality
c     (number of subscripts) of the data array.  if an inverse
c     transform (isign=+1) is performed upon an array of transformed
c     (isign=-1) data, the original data will reappear.
c     multiplied by nn(1)*nn(2)*,  the array of input data must be
c     in complex format.  however, if all imaginary parts are zero (i.e.
c     the data are disguised real) running time is cut up to forty per-
c     cent.  (for fastest transform of real data, nn(1) should be even.)
c     the transform values are always complex and are returned in the
c     original array of data, replacing the input data.  the length
c     of each dimension of the data array may be any integer.  the
c     program runs faster on composite integers than on primes, and is
c     particularly fast on numbers rich in factors of two.
c     timing is in fact given by the following formula.  let ntot be the
c     total number of points (real or complex) in the data array, that
c     is, ntot=nn(1)*nn(2)*...  decompose ntot into its prime factors,
c     such as 2**k2 * 3**k3 * 5**k5 * ...  let sum2 be the sum of all
c     the factors of two in ntot, that is, sum2 = 2*k2.  let sumf be
c     the sum of all other factors of ntot, that is, sumf = 3*k3*5*k5*..
c     the time taken by a multidimensional transform on these ntot data
c     is t = t0 + ntot*(t1+t2*sum2+t3*sumf).  on the cdc 3300 (floating
c     point add time = six microseconds), t = 3000 + ntot*(600+40*sum2+
c     175*sumf) microseconds on complex data.
c     implementation of the definition by summation will run in a time
c     proportional to ntot*(nn(1)+nn(2)+...).  for highly composite ntot
c     the savings offered by this program can be dramatic.  a one-dimen-
c     sional array 4000 in length will be transformed in 4000*(600+
c     40*(2+2+2+2+2)+175*(5+5+5)) = 14.5 seconds versus about 4000*
c     4000*175 = 2800 seconds for the straightforward technique.
c     the fast fourier transform places three restrictions upon the
c     data.
c     1.  the number of input data and the number of transform values
c     must be the same.
c     2.  both the input data and the transform values must represent
c     equispaced points in their respective domains of time and
c     frequency.  calling these spacings deltat and deltaf, it must be
c     true that deltaf=2*pi/(nn(i)*deltat).  of course, deltat need not
c     be the same for every dimension.
c     3.  conceptually at least, the input data and the transform output
c     represent single cycles of periodic functions.
c     the calling sequence is--
c     call fourt(data,nn,ndim,isign,iform,work)
c     data is the array used to hold the real and imaginary parts
c     of the data on input and the transform values on output.  it
c     is a multidimensional floating point array, with the real and
c     imaginary parts of a datum stored immediately adjacent in storage
c     (such as fortran iv places them).  normal fortran ordering is
c     expected, the first subscript changing fastest.  the dimensions
c     are given in the integer array nn, of length ndim.  isign is -1
c     to indicate a forward transform (exponential sign is -) and +1
c     for an inverse transform (sign is +).  iform is +1 if the data are
c     complex, 0 if the data are real.  if it is 0, the imaginary
c     parts of the data must be set to zero.  as explained above, the
c     transform values are always complex and are stored in array data.
c     work is an array used for working storage.  it is floating point
c     real, one dimensional of length equal to twice the largest array
c     dimension nn(i) that is not a power of two.  if all nn(i) are
c     powers of two, it is not needed and may be replaced by zero in the
c     calling sequence.  thus, for a one-dimensional array, nn(1) odd,
c     work occupies as many storage locations as data.  if supplied,
c     work must not be the same array as data.  all subscripts of all
c     arrays begin at one.
c     example 1.  three-dimensional forward fourier transform of a
c     complex array dimensioned 32 by 25 by 13 in fortran iv.
c     dimension data(32,25,13),work(50),nn(3)
c     complex data
c     data nn/32,25,13/
c     do 1 i=1,32
c     do 1 j=1,25
c     do 1 k=1,13
c  1  data(i,j,k)=complex value
c     call fourt(data,nn,3,-1,1,work)
c     example 2.  one-dimensional forward transform of a real array of
c     length 64 in fortran ii,
c     dimension data(2,64)
c     do 2 i=1,64
c     data(1,i)=real part
c  2  data(2,i)=0.
c     call fourt(data,64,1,-1,0,0)
c     there are no error messages or error halts in this program.  the
c     program returns immediately if ndim or any nn(i) is less than one.
c     program by norman brenner from the basic program by charles
c     rader,  june 1967.  the idea for the digit reversal was
c     suggested by ralph alter.
c     this is the fastest and most versatile version of the fft known
c     to the author.  a program called four2 is available that also
c     performs the fast fourier transform and is written in usasi basic
c     fortran.  it is about one third as long and restricts the
c     dimensions of the input array (which must be complex) to be powers
c     of two.  another program, called four1, is one tenth as long and
c     runs two thirds as fast on a one-dimensional complex array whose
c     length is a power of two.
c     reference--
c     ieee audio transactions (june 1967), special issue on the fft.
c***********************************************************************
      dimension data(nfourt),nn(2),ifact(32),work(nwork1)
      data twopi/6.2831853071796/,rthlf/0.70710678118655/
      if(ndim-1)1190,10,10
   10 ntot=2
      do 20 idim=1,ndim
      if(nn(idim))1190,1190,20
   20 ntot=ntot*nn(idim)
c     main loop for each dimension
      np1=2
      do 1180 idim=1,ndim
      n=nn(idim)
      np2=np1*n
      if(n-1)1190,1170,30
c     is n a power of two and if not, what are its factors
   30 m=n
      ntwo=np1
      if=1
      idiv=2
   40 iquot=m/idiv
      irem=m-idiv*iquot
      if(iquot-idiv)120,50,50
   50 if(irem)70,60,70
   60 ntwo=ntwo+ntwo
      ifact(if)=idiv
      if=if+1
      m=iquot
      go to 40
   70 idiv=3
      inon2=if
   80 iquot=m/idiv
      irem=m-idiv*iquot
      if(iquot-idiv)140,90,90
   90 if(irem)110,100,110
  100 ifact(if)=idiv
      if=if+1
      m=iquot
      go to 80
  110 idiv=idiv+2
      go to 80
  120 inon2=if
      if(irem)140,130,140
  130 ntwo=ntwo+ntwo
      go to 150
  140 ifact(if)=m
c     separate four cases--
c        1. complex transform or real transform for the 4th, 9th,etc.
c           dimensions.
c        2. real transform for the 2nd or 3rd dimension.  method--
c           transform half the data, supplying the other half by con-
c           jugate symmetry.
c        3. real transform for the 1st dimension, n odd.  method--
c           set the imaginary parts to zero.
c        4. real transform for the 1st dimension, n even.  method--
c           transform a complex array of length n/2 whose real parts
c           are the even numbered real values and whose imaginary parts
c           are the odd numbered real values.  separate and supply
c           the second half by conjugate symmetry.
  150 icase=1
      ifmin=1
      i1rng=np1
      if(idim-4)160,210,210
  160 if(iform)170,170,210
  170 icase=2
      i1rng=np0*(1+nprev/2)
      if(idim-1)180,180,210
  180 icase=3
      i1rng=np1
      if(ntwo-np1)210,210,190
  190 icase=4
      ifmin=2
      ntwo=ntwo/2
      n=n/2
      np2=np2/2
      ntot=ntot/2
      i=1
      do 200 j=1,ntot
      data(j)=data(i)
  200 i=i+2
c     shuffle data by bit reversal, since n=2**k.  as the shuffling
c     can be done by simple interchange, no working array is needed
  210 if(ntwo-np2)290,220,220
  220 np2hf=np2/2
      j=1
      do 280 i2=1,np2,np1
      if(j-i2)230,250,250
  230 i1max=i2+np1-2
      do 240 i1=i2,i1max,2
      do 240 i3=i1,ntot,np2
      j3=j+i3-i2
      tempr=data(i3)
      tempi=data(i3+1)
      data(i3)=data(j3)
      data(i3+1)=data(j3+1)
      data(j3)=tempr
  240 data(j3+1)=tempi
  250 m=np2hf
  260 if(j-m)280,280,270
  270 j=j-m
      m=m/2
      if(m-np1)280,260,260
  280 j=j+m
      go to 370
c     shuffle data by digit reversal for general n
  290 nwork=2*n
      do 360 i1=1,np1,2
      do 360 i3=i1,ntot,np2
      j=i3
      do 350 i=1,nwork,2
      if(icase-3)300,310,300
  300 work(i)=data(j)
      work(i+1)=data(j+1)
      go to 320
  310 work(i)=data(j)
      work(i+1)=0.
  320 ifp2=np2
      if=ifmin
  330 ifp1=ifp2/ifact(if)
      j=j+ifp1
      if(j-i3-ifp2)350,340,340
  340 j=j-ifp2
      ifp2=ifp1
      if=if+1
      if(ifp2-np1)350,350,330
  350 continue
      i2max=i3+np2-np1
      i=1
      do 360 i2=i3,i2max,np1
      data(i2)=work(i)
      data(i2+1)=work(i+1)
  360 i=i+2
c     main loop for factors of two.  perform fourier transforms of
c     length four, with one of length two if needed.  the twiddle factor
c     w=exp(isign*2*pi*sqrt(-1)*m/(4*mmax)).  check for w=isign*sqrt(-1)
c     and repeat for w=w*(1+isign*sqrt(-1))/sqrt(2).
  370 if(ntwo-np1)680,680,380
  380 np1tw=np1+np1
      ipar=ntwo/np1
  390 if(ipar-2)430,410,400
  400 ipar=ipar/4
      go to 390
  410 do 420 i1=1,i1rng,2
      do 420 k1=i1,ntot,np1tw
      k2=k1+np1
      tempr=data(k2)
      tempi=data(k2+1)
      data(k2)=data(k1)-tempr
      data(k2+1)=data(k1+1)-tempi
      data(k1)=data(k1)+tempr
  420 data(k1+1)=data(k1+1)+tempi
  430 mmax=np1
  440 if(mmax-ntwo/2)450,680,680
  450 lmax=max0(np1tw,mmax/2)
      do 670 l=np1,lmax,np1tw
      m=l
      if(mmax-np1)500,500,460
  460 theta=-twopi*float(l)/float(4*mmax)
      if(isign)480,470,470
  470 theta=-theta
  480 wr=cos(theta)
      wi=sin(theta)
  490 w2r=wr*wr-wi*wi
      w2i=2.*wr*wi
      w3r=w2r*wr-w2i*wi
      w3i=w2r*wi+w2i*wr
  500 do 630 i1=1,i1rng,2
      kmin=i1+ipar*m
      if(mmax-np1)510,510,520
  510 kmin=i1
  520 kdif=ipar*mmax
  530 kstep=4*kdif
      if(kstep-ntwo)540,540,630
  540 do 620 k1=kmin,ntot,kstep
      k2=k1+kdif
      k3=k2+kdif
      k4=k3+kdif
      if(mmax-np1)550,550,580
  550 u1r=data(k1)+data(k2)
      u1i=data(k1+1)+data(k2+1)
      u2r=data(k3)+data(k4)
      u2i=data(k3+1)+data(k4+1)
      u3r=data(k1)-data(k2)
      u3i=data(k1+1)-data(k2+1)
      if(isign)560,570,570
  560 u4r=data(k3+1)-data(k4+1)
      u4i=data(k4)-data(k3)
      go to 610
  570 u4r=data(k4+1)-data(k3+1)
      u4i=data(k3)-data(k4)
      go to 610
  580 t2r=w2r*data(k2)-w2i*data(k2+1)
      t2i=w2r*data(k2+1)+w2i*data(k2)
      t3r=wr*data(k3)-wi*data(k3+1)
      t3i=wr*data(k3+1)+wi*data(k3)
      t4r=w3r*data(k4)-w3i*data(k4+1)
      t4i=w3r*data(k4+1)+w3i*data(k4)
      u1r=data(k1)+t2r
      u1i=data(k1+1)+t2i
      u2r=t3r+t4r
      u2i=t3i+t4i
      u3r=data(k1)-t2r
      u3i=data(k1+1)-t2i
      if(isign)590,600,600
  590 u4r=t3i-t4i
      u4i=t4r-t3r
      go to 610
  600 u4r=t4i-t3i
      u4i=t3r-t4r
  610 data(k1)=u1r+u2r
      data(k1+1)=u1i+u2i
      data(k2)=u3r+u4r
      data(k2+1)=u3i+u4i
      data(k3)=u1r-u2r
      data(k3+1)=u1i-u2i
      data(k4)=u3r-u4r
  620 data(k4+1)=u3i-u4i
      kdif=kstep
      kmin=4*(kmin-i1)+i1
      go to 530
  630 continue
      m=m+lmax
      if(m-mmax)640,640,670
  640 if(isign)650,660,660
  650 tempr=wr
      wr=(wr+wi)*rthlf
      wi=(wi-tempr)*rthlf
      go to 490
  660 tempr=wr
      wr=(wr-wi)*rthlf
      wi=(tempr+wi)*rthlf
      go to 490
  670 continue
      ipar=3-ipar
      mmax=mmax+mmax
      go to 440
c     main loop for factors not equal to two.  apply the twiddle factor
c     w=exp(isign*2*pi*sqrt(-1)*(j1-1)*(j2-j1)/(ifp1+ifp2)), then
c     perform a fourier transform of length ifact(if), making use of
c     conjugate symmetries.
  680 if(ntwo-np2)690,900,900
  690 ifp1=ntwo
      if=inon2
      np1hf=np1/2
  700 ifp2=ifact(if)*ifp1
      j1min=np1+1
      if(j1min-ifp1)710,710,760
  710 do 750 j1=j1min,ifp1,np1
      theta=-twopi*float(j1-1)/float(ifp2)
      if(isign)730,720,720
  720 theta=-theta
  730 wstpr=cos(theta)
      wstpi=sin(theta)
      wr=wstpr
      wi=wstpi
      j2min=j1+ifp1
      j2max=j1+ifp2-ifp1
      do 750 j2=j2min,j2max,ifp1
      i1max=j2+i1rng-2
      do 740 i1=j2,i1max,2
      do 740 j3=i1,ntot,ifp2
      tempr=data(j3)
      data(j3)=data(j3)*wr-data(j3+1)*wi
  740 data(j3+1)=tempr*wi+data(j3+1)*wr
      tempr=wr
      wr=wr*wstpr-wi*wstpi
  750 wi=tempr*wstpi+wi*wstpr
  760 theta=-twopi/float(ifact(if))
      if(isign)780,770,770
  770 theta=-theta
  780 wstpr=cos(theta)
      wstpi=sin(theta)
      j2rng=ifp1*(1+ifact(if)/2)
      do 890 i1=1,i1rng,2
      do 890 i3=i1,ntot,np2
      j2max=i3+j2rng-ifp1
      do 880 j2=i3,j2max,ifp1
      j1max=j2+ifp1-np1
      do 850 j1=j2,j1max,np1
      j3max=j1+np2-ifp2
      do 850 j3=j1,j3max,ifp2
      jmin=j3-j2+i3
      jmax=jmin+ifp2-ifp1
      i=1+(j3-i3)/np1hf
      if(j2-i3)790,790,820
  790 sumr=0.
      sumi=0.
      do 810 j=jmin,jmax,ifp1
  800 sumr=sumr+data(j)
  810 sumi=sumi+data(j+1)
      work(i)=sumr
      work(i+1)=sumi
      go to 850
  820 iconj=1+(ifp2-2*j2+i3+j3)/np1hf
      j=jmax
      sumr=data(j)
      sumi=data(j+1)
      oldsr=0.
      oldsi=0.
      j=j-ifp1
  830 tempr=sumr
      tempi=sumi
      sumr=twowr*sumr-oldsr+data(j)
      sumi=twowr*sumi-oldsi+data(j+1)
      oldsr=tempr
      oldsi=tempi
      j=j-ifp1
      if(j-jmin)840,840,830
  840 tempr=wr*sumr-oldsr+data(j)
      tempi=wi*sumi
      work(i)=tempr-tempi
      work(iconj)=tempr+tempi
      tempr=wr*sumi-oldsi+data(j+1)
      tempi=wi*sumr
      work(i+1)=tempr+tempi
      work(iconj+1)=tempr-tempi
  850 continue
      if(j2-i3)860,860,870
  860 wr=wstpr
      wi=wstpi
      go to 880
  870 tempr=wr
      wr=wr*wstpr-wi*wstpi
      wi=tempr*wstpi+wi*wstpr
  880 twowr=wr+wr
      i=1
      i2max=i3+np2-np1
      do 890 i2=i3,i2max,np1
      data(i2)=work(i)
      data(i2+1)=work(i+1)
  890 i=i+2
      if=if+1
      ifp1=ifp2
      if(ifp1-np2)700,900,900
c     complete a real transform in the 1st dimension, n even, by con-
c     jugate symmetries.
  900 go to (1170,1090,1170,910),icase
  910 nhalf=n
      n=n+n
      theta=-twopi/float(n)
      if(isign)930,920,920
  920 theta=-theta
  930 wstpr=cos(theta)
      wstpi=sin(theta)
      wr=wstpr
      wi=wstpi
      imin=3
      jmin=2*nhalf-1
      go to 960
  940 j=jmin
      do 950 i=imin,ntot,np2
      sumr=(data(i)+data(j))/2.
      sumi=(data(i+1)+data(j+1))/2.
      difr=(data(i)-data(j))/2.
      difi=(data(i+1)-data(j+1))/2.
      tempr=wr*sumi+wi*difr
      tempi=wi*sumi-wr*difr
      data(i)=sumr+tempr
      data(i+1)=difi+tempi
      data(j)=sumr-tempr
      data(j+1)=-difi+tempi
  950 j=j+np2
      imin=imin+2
      jmin=jmin-2
      tempr=wr
      wr=wr*wstpr-wi*wstpi
      wi=tempr*wstpi+wi*wstpr
  960 if(imin-jmin)940,970,1000
  970 if(isign)980,1000,1000
  980 do 990 i=imin,ntot,np2
  990 data(i+1)=-data(i+1)
 1000 np2=np2+np2
      ntot=ntot+ntot
      j=ntot+1
      imax=ntot/2+1
 1010 imin=imax-2*nhalf
      i=imin
      go to 1030
 1020 data(j)=data(i)
      data(j+1)=-data(i+1)
 1030 i=i+2
      j=j-2
      if(i-imax)1020,1040,1040
 1040 data(j)=data(imin)-data(imin+1)
      data(j+1)=0.
      if(i-j)1060,1080,1080
 1050 data(j)=data(i)
      data(j+1)=data(i+1)
 1060 i=i-2
      j=j-2
      if(i-imin)1070,1070,1050
 1070 data(j)=data(imin)+data(imin+1)
      data(j+1)=0.
      imax=imin
      go to 1010
 1080 data(1)=data(1)+data(2)
      data(2)=0.
      go to 1170
c     complete a real transform for the 2nd or 3rd dimension by
c     conjugate symmetries.
 1090 if(i1rng-np1)1100,1170,1170
 1100 do 1160 i3=1,ntot,np2
      i2max=i3+np2-np1
      do 1160 i2=i3,i2max,np1
      imin=i2+i1rng
      imax=i2+np1-2
      jmax=2*i3+np1-imin
      if(i2-i3)1120,1120,1110
 1110 jmax=jmax+np2
 1120 if(idim-2)1150,1150,1130
 1130 j=jmax+np0
      do 1140 i=imin,imax,2
      data(i)=data(j)
      data(i+1)=-data(j+1)
 1140 j=j-2
 1150 j=jmax
      do 1160 i=imin,imax,np0
      data(i)=data(j)
      data(i+1)=-data(j+1)
 1160 j=j-np0
c     end of loop on each dimension
 1170 np0=np1
      np1=np2
 1180 nprev=n
 1190 return
      end
c***********************************************************************
       subroutine scont(c2,xysq,ar,ai,f3,f4,btan34)
c  subroutine 'cont' upward or downward continues input fourier
c  coefficients.
c***********************************************************************
      f=1.
      rad=sqrt(xysq)
      e=exp(c2*rad)
      if(f4.eq.0..or.c2.lt.0.) go to 50
      if(rad.gt.f4) go to 10
      go to 20
10    ar=0.
      ai=0.
      go to 60
20    if(rad.le.f3) go to 50
      f=(f4-rad)*btan34
50    ar=ar*e*f
      ai=ai*e*f
60    return
      end
c***********************************************************************
      subroutine srdbin(no,dat,n)
c       subroutine 'srdbin' reads standard grids (binary).
c***********************************************************************
      dimension dat(n)
      read(no)dum,dat
      return
      end
c***********************************************************************
      subroutine swrbin(no,dat,n)
c       subroutine 'swrbin' writes standard grids (binary).
c***********************************************************************
      dimension dat(n)
      data dum/0.0/
      write(no)dum,dat
      return
      end
c***********************************************************************
       subroutine srdda(no,ipos,dat,n)
c       subroutine 'srrda' reads keyed sequential files.
c***********************************************************************
      dimension dat(n)
      read(no'ipos)dat
      return
      end
c***********************************************************************
      subroutine swrda(no,ipos,dat,n)
c       subroutine 'swrda' writes keyed sequential files.
c***********************************************************************
      dimension dat(n)
      write(no'ipos)dat
      return
      end
