c
c             program expcnv
c  convert packed to unpacked records and vice versa.
c
	parameter   ( namax=1024, ntout=509, nchmax=50 )
	dimension   intid(2), z(nchmax), ia(namax), a(namax)
	character   ifile*80, ofile*80, id*8, suffix*3
	character   itype*4, otype*4

	logical     profil
	equivalence ( intid, id ), ( ia, a )

	iunit  = 10
	junit  = 11
	npostx = nchmax + 2

10	type *, 'enter input filename'
	accept 1, ifile
1	format(a)
	open( iunit, file=ifile, status='old', form='unformatted', 
	1     access='sequential', readonly, iostat=ios )
	if ( ios .ne. 0 ) then
	  type *, 'input file not found'
	  go to 10
	endif

c  derive input file type

	call lenexp( iunit, nword, ntotal, ia, namax )
	if ( ntotal .gt. 0 ) then
	  lenphy = ntotal + 3	
	  itype  = 'exp '
	  type *, ' input is express'

	  else
	  lenphy = lenrec( iunit, ia, npostx )
	  if ( lenphy .gt. 3 ) then
	    itype = 'post'
	    else
	    itype = 'xyz '
	    type *, ' input is not post'
	  endif
	endif
	type *,' physical record length =', lenphy

c  check for valid id 

	if ( itype .eq. 'post' ) then
	  rewind( iunit )
	  read( iunit ) intid
	  call printc( intid, 2, ierr )
	  if ( ierr .eq. 0 ) then
	    nchan = lenphy - 2	
	    type *, ' input is post file, nchannel =', nchan
	    type *, ' first id :', id, '.'
	    else
	    itype = 'xyz'
	    type *, ' input is not post'
	  endif
	endif

c  set output file type

	otype  = 'exp '
	suffix = 'exp'
	if ( itype .eq. 'exp '  ) then
c         output is not express
	  if ( nword .le. 3 ) then
	    otype  = 'xyz '
	    suffix = 'xyz'
	    else
	    otype  = 'post'
	    suffix = 'pos'
	  endif
	endif

c  open output file

	call getfn( ifile, suffix, ofile )
	type *
	type *, ' output file is'
	type *, ofile
	type *, ' is this ok ? [ y ]'

	if ( noyes() .eq. 0 ) then
	  type *, ' enter output filename'
	  accept 1, ofile
	endif

	if ( otype .eq. 'exp ' ) then
	  ntotal = ntout
	  nword  = lenphy
	  length = 3 + ntotal
	  open( junit, file = ofile, status='new', form='unformatted',
	1       recl=length, access='direct' )
	  else
	  open( junit, file=ofile, status='new', form='unformatted',
	1       access='sequential' )
	endif


c  pack data file

	maxlog = ntotal / nword

	if ( itype .eq. 'xyx' ) then
	  type * 
	  type *, ' working...'
	  call xyzexp( iunit, junit, nword, maxlog, ntotal, ia )
	endif

	if ( itype .eq. 'post' ) then
100	  profil = .false.
	  type *, ' is this profile data, as opposed to random ( y or n )?'
	  ians = noyes()
	  if ( ians .eq.  1 ) profil = .true.
	  if ( ians .eq. -1 ) go to 100
	  call pstexp( profil, iunit, junit, nword, maxlog, ntotal, ia )
	endif

c  unpack an express data file
	if ( itype .eq. 'exp ' ) then
	  type * 
	  type *, ' working...'

	  if ( otype .eq. 'xyz ' ) call expxyz( iunit, junit, 
	1      nword, maxlog, ntotal, a )

	  if ( otype .eq. 'post' ) call exppst( iunit, junit, 
	1      nword, maxlog, ntotal, ia )

	endif

	stop
	end
c*******************************************************************************
	subroutine expxyz( iu, ju, nword, maxlog, ntotal, a )
c  convert express to xyz.
	dimension a(ntotal)
	inp  = 0
	iout = 0

1	read( iu, err = 2, iostat=ios, end = 99 ) nw, nl, nt, a
2	inp = inp + 1
	if ( nw * nl .gt. nt   .or.   nt .ne. ntotal   .or.
	1    ios .ne. 0 ) then
	  type *, 'expxyz: error at record', inp
	  type *, 'nword, nline, ntotal =', nw, nl, nt	  
	  go to 1
	endif
	iz  = 0

	do iline = 1, nl 
	  ix = iz + 1
	  iy = ix + 1
	  iz = iy + 1
	  write( ju ) a(ix), a(iy), a(iz)
	  iout = iout + 1
	enddo	  
	go to 1

99	type *,  inp, ' input records'
	type *, iout, ' output records'
	return
	end
c*******************************************************************************
	subroutine xyzexp( iu,ju, nword, maxlog, ntotal, a )
c  convert xyz to express.
	dimension a(ntotal)
	data dval /'ffffffff'x/
	inp  = 0
	jrec = 0

1	iz    = 0
	idata = 0

10	ix = iz + 1
	iy = ix + 1
	iz = iy + 1
	read( iu, end = 20 ) a(ix), a(iy), a(iz)
	inp   = inp   + 1
	idata = idata + 1
	if ( idata .eq. maxlog ) then
	  write( ju ) nword, idata, ntotal, a
	  iout = iout + 1
	  go to 1
	endif
	go to 10

20	if ( idata .gt. 0 ) then
	  do i = ix, ntotal
	    a(i) = dval
	  enddo
	  jrec = jrec + 1
	  write( ju, rec=jrec ) nword, idata, ntotal, a
	endif

	type *, inp, ' input records'
	type *, jrec, ' output records'
	return
	end
c*******************************************************************************
	subroutine exppst( iu, ju, nword, maxlog, ntotal, ia )
c  convert express to post.
	dimension   intid(2), ia(ntotal)
	character   id*8
	equivalence ( intid, id )
	inp         = 0
	iout        = 0
	nchan       = nword - 2

1	read( iu, err = 2, iostat=ios, end = 99 ) nw, nd, nt, ia
2	inp = inp + 1
	if ( nw * nd .gt. nt   .or.   nt .ne. ntotal   .or.
	1    ios .ne. 0 ) then
	  type *, 'expxyz: error at record', inp
	  type *, 'nword, ndata, ntotal =', nw, nd, nt	  
	  go to 1
	endif

	iptr = 1
	do idata = 1, nd
	  izptr    = iptr + 2 
	  intid(1) = ia(iptr)
	  intid(2) = ia(iptr+1)
	  call wrtpst( ju, id, ia(izptr), nchan )
	  iptr = iptr + nword
	  iout = iout + 1
	enddo	  
	go to 1

99	type *,  inp, ' input records'
	type *, iout, ' output records'
	return
	end
c*******************************************************************************
	subroutine wrtpst( ju, id, z, nz )
	dimension z(nz)
	character id*8
	write(ju) id, z
	return
	end	
c*******************************************************************************
	subroutine rdpst( iu, iz, nw, iend )
	dimension iz(nw)
	iend = 0
	read( iu, end=9 ) iz
	return
9	iend = 1
	end	
c*******************************************************************************
	subroutine pstexp( profil, iu,ju, nword, maxlog, ntotal, ia )
c  convert post to express.
c  the first 4 id characters are line number, and the second ignored.

	dimension   intid(2), isave(52), ia(ntotal)
	character   id*4, currnt*4
	logical     profil
	equivalence ( intid, id )
	data        idval /'ffffffff'x/

	inp   = 0
	jrec  = 0
	itcnt = 0
	ntcnt = 0

	read( iu, end = 30 ) ( isave(i) , i=1, nword )
	inp      = inp + 1
	intid(1) = isave(1)
	intid(2) = isave(2)
	currnt   = id

c  begin profile loop, random data is effectively one profile
1	if ( profil ) type *, 'current line=', currnt
	do i = 1, ntotal
	  ia(i) = idval
	enddo
	do i = 1, nword
	  ia(i) = isave(i)
	enddo
	iptr  = nword + 1
	icnt  = 1
	ndata = 1

c     begin read loop

10	call rdpst( iu, ia(iptr), nword, iend )
	if ( iend .eq. 1 ) go to 20
	inp = inp + 1

	if ( profil ) then
c  check profile id for end-of-line
	  intid(1) = ia(iptr)
	  intid(2) = ia(iptr+1)
	  if ( id .eq. currnt ) then
	    icnt = icnt + 1
	  else
	    ip = iptr
	    do i = 1, nword
	      isave(i) = ia(ip)
	      ia(ip)   = idval
	      ip       = ip + 1
	    enddo
	    go to 20
	  endif
	endif

	ndata = ndata + 1
	if ( ndata .eq. maxlog ) then
	  jrec = jrec + 1
	  write( ju, rec=jrec ) nword, ndata, ntotal, ia
	  iptr  = 1
	  ndata = 0
	  else
	  iptr = iptr + nword
	endif
	go to 10
c  end read loop

c      close out line 

20	if ( profil ) then
	  type *, ' number of entries =', icnt
c  trap nonprofile data
	  itcnt = itcnt + icnt
	  ntcnt = ntcnt + 1
	  if ( ntcnt .gt. 5 ) then
	    iave = itcnt / ntcnt
	    if ( iave .lt. 5 ) then
	      type *, ' fewer than 5 points per profile...aborting'
	      stop
	    endif
	  endif 
	endif

	if ( ndata .gt. 0 ) then
	  do i = iptr + nword - 1, ntotal
	    ia(i) = idval
	  enddo
	  jrec = jrec + 1
	  write( ju, rec=jrec ) nword, ndata, ntotal, ia
	endif
	if ( iend .eq. 0 ) then
	  currnt = id
	  go to 1
	endif

30	type *
	type *, inp, ' input records'
	type *, jrec, ' output records'
	return
	end
c*******************************************************************************
	subroutine lenexp( iunit, nword, ntotal, ia, na )

c  check file for express format
c  return ntotal and the logical record length for the first data record.

	dimension ia(na)

c check first record, set access parameter.
	irand = 1
	read( iunit, rec=1, err=10, iostat=ios ) nword, nlog, ntotal
10	if ( ios .ne. 0 ) then
	  rewind( iunit )
	  read( iunit, err=100 ) nword, nlog, ntotal
	  irand = 0
	endif

c  primary tests.
	if (  nword .lt. 1  .or.  nword .gt. ntotal ) go to 100
	if ( ntotal .lt. 1  .or. ntotal .gt. 10000  ) go to 100
	if (   nword * iabs( nlog )  .gt.   ntotal  ) go to 100

c  find data record nword.
	if ( nlog .lt. 0 ) then
	do 30 irec = 2, 10
	  if ( irand .eq. 1 ) then
	    read( iunit, rec=irec, err=100 ) nword, nlog, nt
	    else
	    read( iunit, end=100 ) nword, nlog, nt
	  endif
	  if ( nword * iabs( nlog )   .gt.   ntotal ) then
	    type *, ' %%lenexp: parameter error: nword * |nlog| > ntotal'
	    type *, ' nword=', nword, ' nlog=', nlog, ' ntotal=', ntotal
	    stop
	  endif
	  if ( nlog .gt. 0 ) go to 50
30	continue
	endif

c  check data buffer length
50	if ( ntotal .gt. na ) go to 999
	if ( irand .eq. 1 ) then
	    read( iunit, rec=1, err=60, iostat=ios ) nw, nl, nt, 
	1   ( ia(i), i=1, ntotal )
	    else
	    rewind( iunit )
	    read( iunit, err=60, iostat=ios ) nw, nl, nt, 
	1   ( ia(i), i=1, ntotal )
	endif
	go to 999

60	type *, ' %%lenexp: physical record too short'
	type *, ' %%lenexp: parameters from record 1'
	type *, ' nword=', nword, ' nlog=', nlog, ' ntotal=', ntotal
	stop

c  not express 
100	nword  = 0
	ntotal = 0
999	return
	end
c******************************************************************************
	function lenrec( iswt, irec, nwork )
c  brute force recovery of record length
	dimension irec(nwork)

	istep = 10
	if ( nwork .gt. 100 ) istep = 20
	if ( nwork .gt. 500 ) istep = 50
	rewind iswt

	do max = istep, nwork, istep
	  read( iswt, err=10 ) ( irec(i), i=1, max )
	  rewind iswt
	enddo
	lenrec = -1
	go to 99

10	rewind iswt
	do lenrec = max, max - istep, -1
	  read( iswt, err=20 ) ( irec(i), i=1, lenrec )
	  go to 99
20	  rewind iswt
	enddo

99	rewind iswt
	return
	end
c*******************************************************************************
	subroutine printc( intgr, nw, ierr )
c  are the bytes in INTGR printing characters ?
	dimension intgr(nw)
	character ch*4
	equivalence ( itest, ch )
	ierr = 0

	do 20 iw = 1, nw
	  itest = intgr(iw)
	  do 10 i = 1, 4
	    ich = ichar( ch(i:i) )
	    if ( ich .lt. 32  .or.  ich .gt. 126 ) then
	      ierr = i
	      return
	    endif
10	  continue
20	continue

	return
	end
c******************************************************************************
	subroutine getfn(ifile,suffix,ofile)
c  strip off old extension and specified device/directory and add new suffix.
c  VAX filename conventions:    device:[directory]filename.extension
c  directory is level(n).level(n+1).level(n+2)...
	character ifile*(*),suffix*(*),ofile*(*)

	lenin = len(ifile)
	lenou = len(ofile)
	ofile = ' '
c  strip off device/directory and default to the working directory
	m = index(ifile,']') + 1
	n = lenin - m + 1
	ofile(1:n) = ifile(m:lenin)

c  find nonblank length
10	lennb=0
	do i = n, 1, -1
	  if(ofile(i:i).ne.' ' .and. lennb.eq.0) lennb=i
	enddo
	if(lennb.eq.0) then
	  type *,' getfn: cannot parse filename, enter new prefix'
	  type *,'        to go with the suffix ',suffix
	  read(5,*,err=10) ofile
	  go to 10
	endif

c  terminate prefix with period.
	n = index(ofile,'.')
	if (n .eq. 0) then
	  n = lennb + 1
	  ofile(n:n)='.'
	endif	

	m = n + 1
	lens = len(suffix)
	n = n + lens
	ofile(m:n) = suffix(1:lens)

	do i = n+1, lenou
	  ofile(i:i) = ' '
	enddo
	return
	end
c******************************************************************************
	function noyes()
c  returns -1 for indeterminate, 0 for no, 1 for yes
	character  inp*4, blank*4
	data       blank/ '    ' /

	icnt  = 0
	noyes = -1

1	inp   = ' '
	read ( 5, 2, end=5, err=5 ) inp
2	format( a )
	if ( inp .eq. blank ) return

	do 3 n = 1, 4
	  if ( inp(n:n) .ne. ' ' ) then
	    if ( inp(n:n).eq.'y' .or. inp(n:n).eq.'Y' ) noyes = 1
	    if ( inp(n:n).eq.'n' .or. inp(n:n).eq.'N' ) noyes = 0
	    if ( noyes .gt. -1 ) return
	    go to 5
	  endif
3	continue

5	if ( icnt .lt. 3 ) then
	  icnt = icnt + 1
	  write( 6, * ) ' <cr>, no, or yes :'
	  go to 1
	endif

	write( 6, * ) ' count exceeded, returning indeterminate state '
9	return
	end
