C
C________________________________________________________________
C
C     PROGRAM  R E M P O L E X P
C  shortened name:  EXPREMD
C________________________________________________________________
C
C PROGRAM REMPOLEXP IS A MODIFIED VERSION OF REMPOLDAT.FOR
C ORIGINALLY WRITTEN BY TOM HILDENBRAND.  REMPOLEXP HAS
C ADDITIONS OF 1) OPTIONAL EXPRESS FILE IO  AND 2) A SLIGHTLY FASTER
C POLYGON ROUTINE.
C
c
c    program 'rem_poldat'
c    removes a simply-connected polygon from formatted
c    or unformatted files. Maximum number of points in polygon is
c    2000.
c    Note: 1st and last point must be the same.
c
c    input: command segment containing namelist parameters
c           and data in various forms
c
c    structure of command segment:
c       &parms infil='test.grv',ldat=1,.......(ect.),&
c
c    namelist parameters:
c      (note: user may specify command segment 'term' in which
c             case the parameters are inputed from the terminal.
c             useful when a few parameters are to be changed from
c             a previous run)
c
c      infil = input segment
c      outfil = output segment
c      polyseg = segment containing x,y points of polygon
c                note: NO NEED TO repeat first point of polygon
C                      at end of file
c                note: points on boundary of polygon are
c                      considered outside
c                default: polyseg='rempoldat.asc'
c      ifmt = format of input segment(default: ifmt='(3f10.2)')
c            note: 'i' formats will cause errors
c      pform = format of data in 'polyseg'
c              (default: pform='(2f10.2)')
c             note: 'i' formats will cause errors
c             (For usgs standard gravity files:
c             lat & long must be in deg.,dec. min.(include neg.
c             signs)
c             note: longitude must be supplied first and then
c                   latitude read in 'f' format
c                   (eq. pform='(2f9.0)' ))
c      title = 80 characters of indentification for usgs standard
c              gravity files
c      irem = 0 remove polygon as output (default)
c             1 remove points outside polygon as output
c      ldat = -1 x,y points of polygon in 'polyseg' (default)
c              0 use points from previous run
c              1 points of polygon entered from terminal
c            (note: data must be entered in clockwise or
c                   counter-clockwise order and include negative
c                   signs!)
c      irec = input record type
c             1 binary lon,lat,z
c             2 binary .post   id,lon,lat,z(1-6)
c             3 formatted lon,lat,z (default)
c             4 formatted .post (id,lon,lat,z(1-6)
c             5 standard usgs gravity file (3 header lines)
c             6 binary EXPRESS FILE
c
c    coded by tom hildenbrand - usgs, denver.
c
c
      dimension x(2001),y(2001),POST(32)
      character*50 infil,outfil,polyseg,cmdseg
      character blank*2,exit*2,title*80,pform*80,t1*80,t2*80,
     1 t3*80
      character stuff1*21,stuff2*44,ifmt*80,id*8
      LOGICAL FUNCTION INSIDE
      namelist/parms/infil,outfil,polyseg,ldat,pform,title,
     1irec,irem,ifmt
      data ir/5/,iw/6/,idat/10/,iout/12/
      data exit/'ex'/,blank/'  '/,dval/'ffff7fff'x/
c
c    default values of parameters
c
      ldat=-1
      ifmt='(3f10.2)'
      title=' created using rem_poldat'
      irec=3
      polyseg='rempoldat.asc'
      pform='(2f10.2)'
      irem=0
c
c    read command segment
c
1     write(iw,2)
2     format(' enter command segment: ')
      read(ir,5)cmdseg
5     format(a50)
      if(cmdseg.eq.exit.or.cmdseg.eq.blank) go to 99
      if(cmdseg.eq.'term') go to 10
      open(10,status='old',form='formatted',file=cmdseg)
      read(idat,parms,end=98)
      close(10)
      go to 15
c
c    command seg='term': read parms from terminal
c
10    write(iw,12)
12    format(' enter parms')
      read(ir,parms,end=98)
c
c    read header records of input segment if irec=5
c
15    if(irec.lt.3.OR.IREC.EQ.6) go to 14
      open(10,status='old',form='formatted',file=infil,readonly)
      go to 13
14    open(10,status='old',form='unformatted',file=infil,
     1 readonly)
13    if(irec.ne.5) go to 18
      read(10,16)t1,t2,t3
      t1=title
16    format(a80)
   18 IF(IREC.EQ.6) THEN
        CALL OPENEXP(IDAT,POST,INFO,NWORD0,LENREC)
        IF(INFO.EQ.1.OR.INFO.EQ.-5) GOTO 94
        IF(INFO.EQ.2) LENREC=512
      END IF
      if(ldat)30,40,20
c
c    enter polygon from terminal
c
20    write(iw,22)
22    format(' enter lon,lat (x,y) pairs of polygon')
      write(iw,24)
24    format(' enter point: ')
      read(ir,*)x(1),y(1)
      ic=1
26    ic=ic+1
      write(iw,24)
      read(ir,*)x(ic),y(ic)
      if(x(ic).eq.x(1).and.y(ic).eq.y(1)) go to 40
      go to 26
c
c   polygon read from 'polyseg'
c
30    open(14,status='old',form='formatted',file=polyseg)
      read(14,pform,end=96)x(1),y(1)
      ic=1
32    ic=ic+1
      read(14,pform,end=35)x(ic),y(ic)
      if(x(ic).eq.x(1).and.y(ic).eq.y(1)) go to 35
      go to 32
35    close(14)
c
c    determine min & max lat & long in polygon
c
40    n=ic-1
      xmin=x(1)
      xmax=x(1)
      ymin=y(1)
      ymax=y(1)
      do 42 i=2,n
      if(x(i).lt.xmin)xmin=x(i)
      if(x(i).gt.xmax)xmax=x(i)
      if(y(i).lt.ymin)ymin=y(i)
42    if(y(i).gt.ymax)ymax=y(i)
      if(irec.lt.3.OR.IREC.EQ.6) go to 45
      open(12,status='new',form='formatted',file=outfil)
      if(irec.eq.5) go to 52
      go to 49
   45 IF(IREC.EQ.6) THEN
        OPEN(IOUT,FILE=OUTFIL,FORM='UNFORMATTED',STATUS='NEW',
     1   RECL=LENREC,RECORDTYPE='FIXED',ACCESS='SEQUENTIAL')
        CALL OPENEXP(IOUT,POST,INFO,NWORD0,LENREC)
        IF(INFO.NE.1) GOTO 92
      ELSE
        open(12,status='new',form='unformatted',file=outfil)
      END IF
49    go to (100,110,115,130,60,140),irec
52    write(12,16)t1,t2,t3
c
c   read data
c
65    format(a21,i7,i8,a44)
60    read(10,65,end=120) stuff1,iyd,ixd,stuff2
      xd=float(ixd)
      yd=float(iyd)
      go to 59
100   read(10,end=120)xd,yd,z
      go to 59
110   read(10,end=120)id,xd,yd,z1,z2,z3,z4,z5,z6
      go to 59
115   read(10,ifmt,end=120)xd,yd,z
      go to 59
130   read(10,ifmt,end=120)id,xd,yd,z1,z2,z3,z4,z5,z6
      GOTO 59
  140 CALL READEXP(IDAT,POST,IEND,NWORD0)
      IF(IEND.NE.0) GOTO 121
      IF(NWORD0.LT.5) THEN
        XD=POST(1)
        YD=POST(2)
      ELSE
        XD=POST(3)
        YD=POST(4)
      END IF
c
c    check to see if data are in polygon
c
59    if(yd.lt.ymin.or.yd.gt.ymax) go to 77
      if(xd.lt.xmin.or.xd.gt.xmax) go to 77
      IF(INSIDE(X,Y,N,XD,YD)) THEN
        IF(IREM.EQ.1) GOTO 49
      ELSE
        IF(IREM.EQ.0) GOTO 49
      END IF
      GOTO 190
77     if(irem.eq.0) go to 49
C
C59    if(yd.lt.ymin.or.yd.gt.ymax) go to 77
C      if(xd.lt.xmin.or.xd.gt.xmax) go to 77
C      call spttest (n,x,y,xd,yd,icode)
C       go to 78
C77     if(irem.eq.0) go to 49
C       go to 190
C78    if(irem.eq.0.and.icode.eq.-1) go to 49
C      if(irem.eq.1.and.icode.eq.1) go to 49
c
c   write output file
c
190   go to (200,210,220,230,240,250),irec
200   write(12)xd,yd,z
      go to 100
210   write(12)id,xd,yd,z1,z2,z3,z4,z5,z6
      go to 110
220   write(12,ifmt)xd,yd,z
      go to 115
230   write(12,ifmt)id,xd,yd,z1,z2,z3,z4,z5,z6
      go to 130
240   write(12,65)stuff1,iyd,ixd,stuff2
      go to 60
  250 CALL WRITEXP(IOUT,POST,IERR)
      GOTO 140
c
c    clean-up,error messages, and exit
c
  121 CALL CLOSEXP(IDAT,INFO)
      CALL CLOSEXP(IOUT,INFO)
120   close(12)
      close(10)
      go to 1
   92 WRITE(IW,91)
   91 FORMAT(X,'ERROR IN WRITING TO EXPRESS UNIT')
      GOTO 121
   94 WRITE(IW,93)
   93 FORMAT(X,'ERROR IN READING FROM EXPRESS UNIT')
      GOTO 121
96    write(iw,95)
95    format(' error in reading polyseg...end pt. & beg pt. the',
     1 ' same?')
      close(14)
      go to 1
98    write(iw,97)
97    format(' %odd EOF in command segment')
      close(10)
      go to 1
99    stop
      end
c    spttest
c
c    subroutine 'spttest' determines if a point (x1,y1) is inside
c    a polygon of n points stored in array x(i) and y(i). The
c    subroutine requires that x(n+1)=x(1) and y(n+1)=y(1). points
c    may be in clockwise or counter-clockwise order.
c
c
      subroutine spttest(n,x,y,x1,y1,icode)
      dimension x(1),y(1)
      logical ya,yb
      ib=1
      do 10 i=1,n
      if(y(i).eq.y(i+1)) go to 10
      ya=y1 .le. y(i)
      yb=y1 .gt. y(i+1)
      if((ya.and.yb.or.(.not.ya.and..not.yb)).and.
     1x1-x(i)-(y1-y(i))*(x(i+1)-x(i))/(y(i+1)-y(i)).lt.0.0)ib=-ib
c
c    the above 'if' statements check to see if two conditions are
c    met. condition one is that y1 is between the end values of
c    the end points of a line segment. condition two will be true
c    if first one is met and x1 is to the left of the line
c    segment. in this way the algorithm counts the number of line
c    segments that go past point (x1,y1) on its right. if there
c    is an odd number of such segments the point is inside the
c    polygon.
c
c    points on the boundary of the polygon are considered
c    outside. to include points on the boundary as inside the
c    polygon change the .lt. condition in the last if statement
c    to .le.
c
10    continue
      icode=-ib
      return
      end
C
C________________________________________________________________
C
C     FUNCTION  I N S I D E
C________________________________________________________________
C
C FUNCTION INSIDE IS A LOGICAL FUNCTION WHICH IS TRUE WHEN A TEST
C POINT IS WITHIN THE BOUNDARIES OF A POLYGON AND FALSE WHEN THE
C TEST POINT IS OUTSIDE THE POLYGON. A POINT ON THE BOUNDARY IS
C CONSIDERED INSIDE.
C
C   X      - REAL*4 ARRAY.  X COORDINATES OF A POLYGON.
C   Y      - REAL*4 ARRAY.  Y COORDINATES
C   NPTS   - INTEGER*4.  NUMBER OF POINTS IN THE POLYGON.
C            (NOTE: THE FIRST AND LAST POINT OF THE POLYGON
C                   DO NOT NEED TO HAVE THE SAME COORDINATES.)
C   XTEST  - REAL*4.  X COORDINATE OF THE TEST POINT.
C   YTEST  - REAL*4.  Y COORDINATE
C
C FUNCTION INSIDE WRITTEN BY ROB BRACKEN, USGS, 31MAR87
C
C
      function inside(x,y,npts,xtest,ytest)
C
C VARIABLE DECLARATION
C
      logical*4 inside
      real*4 x(1),y(1)
C
C INITIALIZE TEST POINT OUTSIDE POLYGON
C
      inside=.false.
C
C INITIALIZE EDGE MARKER OFF OF POLYGON BORDER LINE
C
      inedge=.false.
C
C INITIALIZE X-AXIS CROSSING COUNTER
C
      ncross=0
C
C INITIALIZE I (CURRENT POINTER) AND J (PREVIOUS POINTER)
C
      i=1
      j=npts
C
C POLYGON ENTRY PROCEDURE
C
      if(y(j)-ytest) 101,102,103
C --- POINT J BELOW X-AXIS ---
  101 if(y(i)-ytest) 111,130,120
C --- POINT J ON X-AXIS ---
  102 j=j-1
      if(j.lt.1) goto 130
      if(y(j)-ytest) 104,102,105
C --- X-AXIS APPROACHED FROM BELOW ---
  104 i=0
      if(x(npts)-xtest) 131,132,133
C --- X-AXIS APPROACHED FROM ABOVE ---
  105 i=0
      if(x(npts)-xtest) 141,142,143
C --- POINT J ABOVE X-AXIS ---
  103 if(y(i)-ytest) 110,140,121
C
C I BELOW X-AXIS. J ON OR ABOVE X-AXIS.
C
  110 if(x(i).gt.xtest) then
        if(x(j).ge.xtest) then
C ---     SEGMENT CROSSES POSITIVE X-AXIS ---
          ncross=ncross+1
          goto 111
        end if
      else
        if(x(j).lt.xtest) then
C ---     SEGMENT CROSSES NEGATIVE X-AXIS ---
          goto 111
        end if
      end if
C --- SEGMENT CROSSES X-AXIS VERY NEAR TEST POINT ---
C --- VECTOR PRODUCT:  POINT BELOW X POINT ABOVE ---
      vprod=(x(i)-xtest)*(y(j)-ytest)-(y(i)-ytest)*(x(j)-xtest)
      if(vprod.gt.0.) then
C ---   SEGMENT CROSSES POSITIVE X-AXIS ---
        ncross=ncross+1
      else if(vprod.eq.0.) then
C ---   SEGMENT CROSSES TEST POINT ---
        inedge=.true.
      end if
C --- FOLLOW POLYGON EDGE ---
  111 do i=i+1,npts
        if(y(i).ge.ytest) then
          if(y(i).eq.ytest) goto 130
C ---     POINT I IS ABOVE X-AXIS ---
          j=i-1
          goto 120
        end if
      end do
      goto 190
C
C I ABOVE X-AXIS. J ON OR BELOW X-AXIS.
C
  120 if(x(j).gt.xtest) then
        if(x(i).ge.xtest) then
C ---     SEGMENT CROSSES POSITIVE X-AXIS ---
          ncross=ncross+1
          goto 121
        end if
      else
        if(x(i).lt.xtest) then
C ---     SEGMENT CROSSES NEGATIVE X-AXIS ---
          goto 121
        end if
      end if
C --- SEGMENT CROSSES X-AXIS VERY NEAR TEST POINT ---
C --- VECTOR PRODUCT:  POINT BELOW X POINT ABOVE ---
      vprod=(x(j)-xtest)*(y(i)-ytest)-(y(j)-ytest)*(x(i)-xtest)
      if(vprod.gt.0.) then
C ---   SEGMENT CROSSES POSITIVE X-AXIS ---
        ncross=ncross+1
      else if(vprod.eq.0.) then
C ---   SEGMENT CROSSES TEST POINT ---
        inedge=.true.
      end if
C --- FOLLOW POLYGON EDGE ---
  121 do i=i+1,npts
        if(y(i).le.ytest) then
          if(y(i).eq.ytest) goto 140
C ---     POINT I IS BELOW X-AXIS ---
          j=i-1
          goto 110
        end if
      end do
      goto 190
C
C I ON X-AXIS.  J BELOW X-AXIS.
C
  130 if(x(i)-xtest) 131,132,133
C --- SEGMENT CROSSES TESTPOINT ---
  132 inedge=.true.
C --- POINT I ON NEGATIVE X-AXIS ---
  131 i=i+1
      if(i.gt.npts) goto 190
      if(y(i)-ytest) 111,135,121
C --- NEW POINT I ON X-AXIS ---
  135 if(x(i)-xtest) 131,132,134
C --- SEGMENT CROSSES TEST POINT ---
  134 inedge=.true.
C --- POINT I ON POSITIVE X-AXIS ---
  133 i=i+1
      if(i.gt.npts) goto 190
      if(y(i)-ytest) 111,136,137
C --- NEW POINT I ON X-AXIS ---
  136 if(x(i)-xtest) 132,132,133
C --- NEW POINT I CROSSES POSITIVE X-AXIS ---
  137 ncross=ncross+1
      goto 121
C
C I ON X-AXIS.  J ABOVE X-AXIS.
C
  140 if(x(i)-xtest) 141,142,143
C --- SEGMENT CROSSES TESTPOINT ---
  142 inedge=.true.
C --- POINT I ON NEGATIVE X-AXIS ---
  141 i=i+1
      if(i.gt.npts) goto 190
      if(y(i)-ytest) 111,145,121
C --- NEW POINT I ON X-AXIS ---
  145 if(x(i)-xtest) 141,142,144
C --- SEGMENT CROSSES TEST POINT ---
  144 inedge=.true.
C --- POINT I ON POSITIVE X-AXIS ---
  143 i=i+1
      if(i.gt.npts) goto 190
      if(y(i)-ytest) 147,146,121
C --- NEW POINT I ON X-AXIS ---
  146 if(x(i)-xtest) 142,142,143
C --- NEW POINT I CROSSES POSITIVE X-AXIS ---
  147 ncross=ncross+1
      goto 111
C
C EVALUATE INEDGE AND NCROSS
C
  190 if(inedge) then
        inside=.FALSE.
      else if(jiand(ncross,1).eq.1) then
C --- NCROSS IS ODD ---
        inside=.true.
      end if
C
C RETURN TO CALLING ROUTINE
C
      return
      end
C
C
C >>>>>  BLOCK DATA  B L O C K E X P  <<<<<......................
C
C BLOCK DATA BLOCKEXP CONTAINS THE VARIABLE DECLARATIONS AND
C COMMON STATEMENTS USED THROUGHOUT THE EXPRESS SUBROUTINE
C SERIES.
C
C DATA IS PASSED TO AND FROM THE EXPRESS SUBROUTINES VIA THE CALL
C STATEMENT OR VIA A COMMON BLOCK CALLED /PASBLK/.  IF IT IS
C PASSED VIA THE COMMON BLOCK IT WILL BE NECESSARY TO ELIMINATE
C THE VARIABLE LIST FROM ALL OF THE SUBROUTINE STATEMENTS BEFORE
C COMPILATION.  THE ADVANTAGE OF PASSING THE VARIABLES IN THE
C COMMON BLOCK IS THAT THE EXPRESS SUBROUTINES WILL RUN ABOUT 3%
C FASTER.  HOWEVER, THIS SPEED ADVANTAGE WILL BE NULLIFIED IF
C MORE THAN TWO ASSIGNMENTS PER CALL MUST BE MADE AS A RESULT OF
C NOT PASSING VARIABLES IN THE CALL STATEMENT.
C
C THE PASSED VARIABLES CONTAINED IN PASBLK ARE:
C
C    IUNIT  - UNIT NUMBER OF THE EXPRESS FILE BEING ACCESSED BY 
C             THE PRESENT SUBROUTINE CALL.
C    POST   - A REAL*4 ARRAY OF DIMENSION MXWORD IN WHICH THE
C             DATA IS PASSED TO OR FROM THE EXPRESS FILE IUNIT.
C    INFO   - AN INTEGER CONTAINING INFORMATION PERTAINING TO THE
C             MOST RECENT EXPRESS SUBROUTINE CALL.
C                0 - NORMAL PROCESS OCCURED.
C                1 - END OF FILE ENCOUNTERED DURING A READ.  ZERO
C                    RECORDS PRESENT IN THE EXPRESS FILE.
C                2 - END OF FILE ENCOUNTERED DURING A READ.  ONE
C                    RECORD PRESENT IN THE EXPRESS FILE.
C                3 - END OF FILE ENCOUNTERED DURING A READ.  TWO
C                    RECORDS PRESENT IN THE EXPRESS FILE.  ETC.
C               -4 - WRITE ERROR
C               -5 - UNIT NOT OPENED
C    NWORD0 - NUMBER OF REAL*4 WORDS CONTAINED IN POST.
C    LENREC - DEFAULT MAXIMUM RECORD LENGTH FOR THE EXPRESS FILE.
C             THE DEFAULT IS ACTUALLY SET ONLY IN A CALL TO
C             OPENEXP.  IT APPLIES ONLY WHEN WRITING TO THE FILE
C             IF PREVIOUS READS FROM THE SAME FILE HAVE NOT
C             ALREADY BEEN MADE.  IF PREVIOUS READS HAVE BEEN
C             MADE THE DEFAULT RECORD LENGTH WILL BE EQUAL TO THE
C             LONGEST OF THE TWO MOST RECENT RECORD LENGTHS.
C             WHEN A CALL TO OPENEXP OR TESTEXP IS MADE LENREC
C             RETURNS THE BEST DEFAULT RECORD LENGTH DEPENDING ON
C             ITS INITIAL VALUE AND THE RECORD LENGTHS ALREADY
C             PRESENT IN THE FILE.
C    IPOST  - WHEN A CALL TO TESTEXP IS MADE IPOST IS THE NUMBER
C             OF THE POST GROUP IN THE PRESENT FILE RECORD
C             WHICH WILL BE ACCESSED IN THE NEXT IO OPERATION.
C    IREC   - WHEN A CALL TO TESTEXP IS MADE IREC IS THE EXPRESS
C             FILE RECORD RECORD NUMBER WHERE THE PRESENT
C             IO OPERATIONS ARE OCCURING.
C
C
C BLOCK DATA BLOCKEXP WRITTEN BY ROB BRACKEN, USGS, 10DEC86
C
C
      block data blockexp
C
C VARIABLE DECLARATION
C
C VARIABLES WHICH MAY BE PASSED TO AND FROM THE SUBROUTINES
C VIA COMMON BLOCK /PASBLK/
C
      integer*4 iunit,info,nword0,lenrec,ipost,irec
      parameter(mxword=32)
      real*4 post(mxword)
      common /pasblk/ iunit,post,info,nword0,lenrec,ipost,irec
C
C DIMENSION SIZES WHICH MUST BE USED THROUGHOUT ALL THE 
C EXPRESS SUBROUTINES
C
      integer*2 mxhold,nunit,otty
      parameter(mxhold=2557,nunit=99,otty=6)
C
C VARIABLES WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      logical*1 mode(nunit)
      integer*4 ipoint(nunit)
      integer*2 nword(nunit)
      integer*2 idata(nunit),ndata(nunit),mxdata(nunit)
      integer*2 ntotal(nunit),ntot1(nunit),ntot2(nunit)
      integer*2 ndftot(nunit),mxtot(nunit)
      real*4 hold(mxhold,nunit)
C
C COMMON BLOCKS WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      common /filblk/ mode,ipoint
      common /datblk/ nword,idata,ndata,mxdata
      common /totblk/ ntotal,ntot1,ntot2,ndftot,mxtot
      common /hldblk/ hold
C
C DATA STATEMENTS WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      data mode /nunit*0/
C
C END OF BLOCK DATA
C
      end
C
C
C >>>>>  SUBROUTINE  O P E N E X P  <<<<<........................
C
C SUBROUTINE OPEN EXPRESS INITIALIZES VARIABLES FOR OPENING A NEW
C EXPRESS UNIT FOR EXPRESS FORMAT FILE IO AND RETURNS VALUES FOR
C NWORD0 AND LENREC AND ASSIGNS THESE VALUES FOR FURTHER FILE IO.
C
C SUBROUTINE OPENEXP WRITTEN BY ROB BRACKEN, USGS, 10DEC86
C
C
      subroutine openexp(iunit,post,info,nword0,lenrec)
C
C VARIABLE DECLARATION
C
C VARIABLES WHICH MAY BE PASSED TO AND FROM THE SUBROUTINES
C VIA COMMON BLOCK /PASBLK/
C
      integer*4 iunit,info,nword0,lenrec,ipost,irec
      parameter(mxword=32)
      real*4 post(mxword)
c      common /pasblk/ iunit,post,info,nword0,lenrec,post,irec
C
C DIMENSION SIZES WHICH MUST BE USED THROUGHOUT ALL THE 
C EXPRESS SUBROUTINES
C
      integer*2 mxhold,nunit,otty
      parameter(mxhold=2557,nunit=99,otty=6)
C
C VARIABLES WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      logical*1 mode(nunit)
      integer*4 ipoint(nunit)
      integer*2 nword(nunit)
      integer*2 idata(nunit),ndata(nunit),mxdata(nunit)
      integer*2 ntotal(nunit),ntot1(nunit),ntot2(nunit)
      integer*2 ndftot(nunit),mxtot(nunit)
      real*4 hold(mxhold,nunit)
C
C COMMON BLOCKS WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      common /filblk/ mode,ipoint
      common /datblk/ nword,idata,ndata,mxdata
      common /totblk/ ntotal,ntot1,ntot2,ndftot,mxtot
      common /hldblk/ hold
C
C END OF BLOCK DATA
C
C
C INITIALIZE IUNIT - UNIT NUMBER OF THE EXPRESS UNIT
C
      if(iunit.lt.1.or.iunit.gt.nunit) then
        write(otty,801) iunit
  801   format(x,'*** OPENEXP -- ILLEGAL UNIT NUMBER: ',i4)
        info=-5
        return
      end if
C
C INITIALIZE MODE(IUNIT) - MODE OF PREVIOUS OPERATION
C
C    1 - WRITING MODE
C    2 - OPENING MODE
C    3 - READING MODE
C
      mode(iunit)=2
C
C INITIALIZE IPOINT(IUNIT) - EXPRESS UNIT FILE POINTER INDEX
C
      rewind (unit=iunit)
      ipoint(iunit)=1
C
C INITIALIZE NWORD  - NUMBER OF REAL*4 FIELDS IN EACH POST GROUP
C INITIALIZE LENREC - NUMBER OF REAL*4 FIELDS IN EACH RECORD
C
      call testexp(iunit,post,info,nword0,lenrec,ipost,irec)
      nword(iunit)=nword0
C
C INITIALIZE INFO - INFORMATION VARIABLE GIVING STATUS INFO FOR
C                   CERTAIN OPERATIONS.
C
      info=info
C
C INITIALIZE:
C
C    NDATA(IUNIT)  - HOLDING ARRAY INDEX OF THE FIRST WORD IN THE
C                    RIGHT MOST EXISTING POST GROUP OF THE
C                    CURRENT RECORD.
C    IDATA(IUNIT)  - HOLDING ARRAY INDEX OF THE FIRST WORD IN THE
C                    NEXT POST GROUP TO BE PROCESSED.
C    MXDATA(IUNIT) - HOLDING ARRAY INDEX OF ANY WORD IN THE
C                    RIGHT MOST ALLOWABLE POST GROUP OF THE
C                    CURRENT RECORD.  APPLIES ONLY WHEN WRITING.
C
      ndata(iunit)=1-nword(iunit)
      idata(iunit)=1
      mxdata(iunit)=0
C
C INITIALIZE:
C
C    NTOTAL(IUNIT) - NUMBER OF REAL*4 WORDS IN THE CURRENT
C                    HOLDING ARRAY.
C    NTOT1(IUNIT)  - PREVIOUS NTOTAL(IUNIT)
C    NTOT2(IUNIT)  - PREVIOUS NTOT1(IUNIT)
C    NDFTOT(IUNIT) - DEFAULT VALUE FOR NTOTAL(IUNIT)
C    MXTOT(IUNIT)  - MAXIMUM SIZE OF HOLDING ARRAY
C
      ntotal(iunit)=0
      ntot1(iunit)=0
      ntot2(iunit)=0
      ndftot(iunit)=lenrec-3
      mxtot(iunit)=mxhold
C
C ADJUST NTOT1(IUNIT)
C
      ntot1(iunit)=ndftot(iunit)
C
C FIND NTOTAL(IUNIT) AND MXDATA(IUNIT)
C
      ntotal(iunit)=imax0(ntot1(iunit),ntot2(iunit))
      mxdata(iunit)=ntotal(iunit)-nword(iunit)+1
C
C INITIALIZE POST - REAL*4 ARRAY CONTAINING POST GROUP FIELDS.
C
      do i=1,mxword
        post(i)=post(i)
      end do
C
C INITIALIZE HOLD - REAL*4 ARRAY CONTAINING EACH RECORD READ FROM
C                   THE EXPRESS UNIT.
C
      do i=1,mxtot(iunit)
        hold(i,iunit)=0.e0
      end do
C
C INITIALIZING COMPLETED
C
      return
      end
C
C
C >>>>>  SUBROUTINE  C L O S E X P  <<<<<......................
C
C SUBROUTINE CLOSE EXPRESS OUTPUTS THE FINAL RECORD TO THE
C EXPRESS UNIT.
C
C SUBROUTINE CLOSEXP WRITTEN BY ROB BRACKEN, USGS, 10DEC86
C
C
      subroutine closexp(iunit,info)
C
C VARIABLE DECLARATION
C
C VARIABLES WHICH MAY BE PASSED TO AND FROM THE SUBROUTINES
C VIA COMMON BLOCK /PASBLK/
C
      integer*4 iunit,info
      parameter(mxword=32)
      real*4 post(mxword)
c      common /pasblk/ iunit,post,info
C
C DIMENSION SIZES WHICH MUST BE USED THROUGHOUT ALL THE 
C EXPRESS SUBROUTINES
C
      integer*2 mxhold,nunit,otty
      parameter(mxhold=2557,nunit=99,otty=6)
C
C VARIABLES WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      logical*1 mode(nunit)
      integer*4 ipoint(nunit)
      integer*2 nword(nunit)
      integer*2 idata(nunit),ndata(nunit),mxdata(nunit)
      integer*2 ntotal(nunit),ntot1(nunit),ntot2(nunit)
      integer*2 ndftot(nunit),mxtot(nunit)
      real*4 hold(mxhold,nunit)
C
C COMMON BLOCKS WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      common /filblk/ mode,ipoint
      common /datblk/ nword,idata,ndata,mxdata
      common /totblk/ ntotal,ntot1,ntot2,ndftot,mxtot
      common /hldblk/ hold
C
C END OF BLOCK DATA
C
C
C BRANCH ACCORDING TO THE CURRENT MODE
C
      goto(10,20,30),mode(iunit)
      write(otty,801) iunit
  801 format(x,'*** CLOSEXP -- UNIT',i4,' NOT OPENED')
      info=-5
      return
C
C WRITING MODE - OUTPUT FINAL RECORD
C
   10 if(ndata(iunit).ge.1) then
        call outrec(iunit,nword(iunit),ndata(iunit),
     1              ntotal(iunit),hold(1,iunit),info)
        if(info.ne.0) return
        ipoint(iunit)=ipoint(iunit)+1
        mode(iunit)=0
        info=ipoint(iunit)
        return
      end if
C
C OPENING MODE - CONTINUE TO READING MODE
C
   20 continue
C
C READING MODE - CONTINUE TO RETURN
C
   30 continue
      mode(iunit)=0
      info=0
C
C CLOSING COMPLETED
C
      return
      end
C
C
C >>>>>  SUBROUTINE  W R I T E X P  <<<<<........................
C
C SUBROUTINE WRITE EXPRESS WRITES ONE POST GROUP AT A TIME TO THE
C HOLDING ARRAY AND WRITES THE CONTENTS OF THE HOLDING ARRAY TO
C THE EXPRESS UNIT AT APPROPRIATE TIMES
C
C SUBROUTINE WRITEXP WRITTEN BY ROB BRACKEN, USGS, 10DEC86
C
C
      subroutine writexp(iunit,post,ierr)
C
C VARIABLE DECLARATION
C
C VARIABLES WHICH MAY BE PASSED TO AND FROM THE SUBROUTINES
C VIA COMMON BLOCK /PASBLK/
C
      integer*4 iunit,ierr,nword0
      parameter(mxword=32)
      real*4 post(mxword)
c      common /pasblk/ iunit,post,ierr
C
C DIMENSION SIZES WHICH MUST BE USED THROUGHOUT ALL THE 
C EXPRESS SUBROUTINES
C
      integer*2 mxhold,nunit,otty
      parameter(mxhold=2557,nunit=99,otty=6)
C
C VARIABLES WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      logical*1 mode(nunit)
      integer*4 ipoint(nunit)
      integer*2 nword(nunit)
      integer*2 idata(nunit),ndata(nunit),mxdata(nunit)
      integer*2 ntotal(nunit),ntot1(nunit),ntot2(nunit)
      integer*2 ndftot(nunit),mxtot(nunit)
      real*4 hold(mxhold,nunit)
C
C COMMON BLOCKS WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      common /filblk/ mode,ipoint
      common /datblk/ nword,idata,ndata,mxdata
      common /totblk/ ntotal,ntot1,ntot2,ndftot,mxtot
      common /hldblk/ hold
C
C END OF BLOCK DATA
C
C
C BRANCH ACCORDING TO THE CURRENT MODE(IUNIT)
C
      if(mode(iunit).eq.1) goto 10
      goto(10,20,30),mode(iunit)
      write(otty,801) iunit
  801 format(x,'*** WRITEXP -- UNIT',i4,' NOT OPENED')
      ierr=-5
      return
C
C WRITING MODE - OUTPUT HOLDING RECORD IF IT IS FULL
C
   10 if(idata(iunit).gt.mxdata(iunit)) then
        call outrec(iunit,nword(iunit),ndata(iunit),
     1              ntotal(iunit),hold(1,iunit),ierr)
        if(ierr.ne.0) return
        ipoint(iunit)=ipoint(iunit)+1
   50   idata(iunit)=1
        ndata(iunit)=1-nword(iunit)
      end if
C
C PUT POST GROUP INTO HOLDING ARRAY
C
   40 mpost=idata(iunit)
      nword0=nword(iunit)
      goto(301,302,303,304,305,306,307,308,309,310,311,312),
     1 nword0
      mpost=mpost-1
      do i=nword0,13,-1
        hold(mpost+i,iunit)=post(i)
      end do
      mpost=mpost+1
  312 hold(mpost+11,iunit)=post(12)
  311 hold(mpost+10,iunit)=post(11)
  310 hold(mpost+9,iunit)=post(10)
  309 hold(mpost+8,iunit)=post(9)
  308 hold(mpost+7,iunit)=post(8)
  307 hold(mpost+6,iunit)=post(7)
  306 hold(mpost+5,iunit)=post(6)
  305 hold(mpost+4,iunit)=post(5)
  304 hold(mpost+3,iunit)=post(4)
  303 hold(mpost+2,iunit)=post(3)
  302 hold(mpost+1,iunit)=post(2)
  301 hold(mpost,iunit)=post(1)
C
C INCREMENT IDATA TO NEW POSITION AND RETURN
C
      ndata(iunit)=mpost
      idata(iunit)=mpost+nword0
      ierr=0
      return
C
C OPENING MODE
C
   20 mode(iunit)=1
      goto 40
C
C READING MODE - CONVERT TO WRITING MODE
C
   30 mode(iunit)=1
      ntotal(iunit)=imax0(ntot1(iunit),ntot2(iunit))
      mxdata(iunit)=ntotal(iunit)-nword(iunit)+1
      if(idata(iunit).gt.mxdata(iunit)) goto 50
      backspace iunit
      ipoint(iunit)=ipoint(iunit)-1
      goto 40
      end
C
C
C >>>>>  SUBROUTINE  R E A D E X P  <<<<<........................
C
C SUBROUTINE READ EXPRESS READS A POST GROUP FROM THE HOLDING
C ARRAY AND IF NECESSARY READS THE NEXT RECORD FROM THE EXPRESS
C UNIT INTO THE HOLDING ARRAY
C
C SUBROUTINE READEXP WRITTEN BY ROB BRACKEN, USGS, 10DEC86
C
C
      subroutine readexp(iunit,post,iend,nword0)
c      subroutine readexp
C
C VARIABLE DECLARATION
C
C VARIABLES WHICH MAY BE PASSED TO AND FROM THE SUBROUTINES
C VIA COMMON BLOCK /PASBLK/
C
      integer*4 iunit,iend,nword0
      parameter(mxword=32)
      real*4 post(mxword)
c      common /pasblk/ iunit,post,iend,nword0
C
C DIMENSION SIZES WHICH MUST BE USED THROUGHOUT ALL THE 
C EXPRESS SUBROUTINES
C
      integer*2 mxhold,nunit,otty
      parameter(mxhold=2557,nunit=99,otty=6)
C
C VARIABLES WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      logical*1 mode(nunit)
      integer*4 ipoint(nunit)
      integer*2 nword(nunit)
      integer*2 idata(nunit),ndata(nunit),mxdata(nunit)
      integer*2 ntotal(nunit),ntot1(nunit),ntot2(nunit)
      integer*2 ndftot(nunit),mxtot(nunit)
      real*4 hold(mxhold,nunit)
C
C COMMON BLOCKS WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      common /filblk/ mode,ipoint
      common /datblk/ nword,idata,ndata,mxdata
      common /totblk/ ntotal,ntot1,ntot2,ndftot,mxtot
      common /hldblk/ hold
C
C END OF BLOCK DATA
C
C
C BRANCH ACCORDING TO THE CURRENT MODE(IUNIT)
C
      if(mode(iunit).eq.3) goto 30
      goto(10,20),mode(iunit)
      write(otty,801) iunit
  801 format(x,'*** READEXP -- UNIT',I4,' NOT OPENED')
      iend=-5
      return
C
C READING MODE - YANK NEW RECORD INTO HOLDING ARRAY
C
   30 if(idata(iunit).gt.ndata(iunit)) then
   50   call yank(iunit,nword(iunit),ndata(iunit),ntotal(iunit),
     1            hold(1,iunit),iend)
        if(iend.eq.2) then
          iend=ipoint(iunit)
          return
        end if
        ipoint(iunit)=ipoint(iunit)+1
        idata(iunit)=1
        mxdata(iunit)=ntotal(iunit)-nword(iunit)+1
        ntot2(iunit)=ntot1(iunit)
        ntot1(iunit)=ntotal(iunit)
      end if
C
C TRANSFER POST GROUP FROM HOLDING ARRAY INTO POST ARRAY
C
   40 mpost=idata(iunit)
      nword0=nword(iunit)
      goto(301,302,303,304,305,306,307,308,309,310,311,312),
     1 nword0
      mpost=mpost-1
      do i=nword0,13,-1
        post(i)=hold(mpost+i,iunit)
      end do
      mpost=mpost+1
  312 post(12)=hold(mpost+11,iunit)
  311 post(11)=hold(mpost+10,iunit)
  310 post(10)=hold(mpost+9,iunit)
  309 post(9)=hold(mpost+8,iunit)
  308 post(8)=hold(mpost+7,iunit)
  307 post(7)=hold(mpost+6,iunit)
  306 post(6)=hold(mpost+5,iunit)
  305 post(5)=hold(mpost+4,iunit)
  304 post(4)=hold(mpost+3,iunit)
  303 post(3)=hold(mpost+2,iunit)
  302 post(2)=hold(mpost+1,iunit)
  301 post(1)=hold(mpost,iunit)
C
C INCREMENT IDATA(IUNIT) TO NEW POSITION AND RETURN
C
      idata(iunit)=mpost+nword0
      iend=0
      return
C
C OPENING MODE - YANK NEW RECORD INTO HOLDING ARRAY
C
   20 mode(iunit)=3
      goto 50
C
C WRITING MODE - CONVERT TO READING MODE AND RETURN WITH END
C                STATUS
C
   10 if(idata(iunit).le.ndata(iunit)) goto 40
      call outrec(iunit,nword(iunit),ndata(iunit),ntotal(iunit),
     1            hold(1,iunit),ierr)
      mode(iunit)=3
      nword0=nword(iunit)
      ipoint(iunit)=ipoint(iunit)+1
      iend=ipoint(iunit)
      return
      end
C
C
C >>>>>  SUBROUTINE  R E W I N D E X P  <<<<<..................
C
C SUBROUTINE REWIND EXPRESS REWINDS THE EXPRESS UNIT AND RESETS
C THE APPROPRIATE VARIABLES
C
C SUBROUTINE REWINDEXP WRITTEN BY ROB BRACKEN, USGS, 10DEC86
C
C
      subroutine rewindexp(iunit,info)
C
C VARIABLE DECLARATION
C
C VARIABLES WHICH MAY BE PASSED TO AND FROM THE SUBROUTINES
C VIA COMMON BLOCK /PASBLK/
C
      integer*4 iunit,info,nword0,lenrec,ipost,irec
      parameter(mxword=32)
      real*4 post(mxword)
c      common /pasblk/ iunit,post,info
C
C DIMENSION SIZES WHICH MUST BE USED THROUGHOUT ALL THE 
C EXPRESS SUBROUTINES
C
      integer*2 mxhold,nunit,otty
      parameter(mxhold=2557,nunit=99,otty=6)
C
C VARIABLES WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      logical*1 mode(nunit)
      integer*4 ipoint(nunit)
      integer*2 nword(nunit)
      integer*2 idata(nunit),ndata(nunit),mxdata(nunit)
      integer*2 ntotal(nunit),ntot1(nunit),ntot2(nunit)
      integer*2 ndftot(nunit),mxtot(nunit)
      real*4 hold(mxhold,nunit)
C
C COMMON BLOCKS WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      common /filblk/ mode,ipoint
      common /datblk/ nword,idata,ndata,mxdata
      common /totblk/ ntotal,ntot1,ntot2,ndftot,mxtot
      common /hldblk/ hold
C
C END OF BLOCK DATA
C
C
C BRANCH ACCORDING TO THE CURRENT MODE(IUNIT)
C
      goto(10,20,30),mode(iunit)
      write(otty,801) iunit
  801 format(x,'*** REWINDEXP -- UNIT',i4,' NOT OPENED')
      info=-5
      return
C
C READING MODE - REWIND EXPRESS UNIT AND RESET VARIABLES
C
   30 nword0=nword(iunit)
      lenrec=imax0(ntot1(iunit),ntot2(iunit),ndftot(iunit))+3
      call openexp(iunit,post,info,nword0,lenrec,ipost,irec)
      return
C
C OPENING MODE - NO ACTION NECESSARY
C
   20 info=0
      return
C
C WRITING MODE - CONVERT TO READING MODE
C
   10 call outrec(iunit,nword(iunit),ndata(iunit),ntotal(iunit),
     1            hold(1,iunit),ierr)
      goto 30
      end
C
C
C >>>>>  SUBROUTINE  B A C K S P A C E X P  <<<<<..............
C
C SUBROUTINE BACKSPACE EXPRESS BACKSPACES THE POINTER IN THE
C HOLDING ARRAY AND IF NECESSARY BACKSPACES THE EXPRESS UNIT
C
C SUBROUTINE BACKSPACEXP WRITTEN BY ROB BRACKEN, USGS, 10DEC86
C
C
      subroutine backspacexp(iunit,info)
C
C VARIABLE DECLARATION
C
C VARIABLES WHICH MAY BE PASSED TO AND FROM THE SUBROUTINES
C VIA COMMON BLOCK /PASBLK/
C
      integer*4 iunit,info
      parameter(mxword=32)
      real*4 post(mxword)
c      common /pasblk/ iunit,post,info
C
C DIMENSION SIZES WHICH MUST BE USED THROUGHOUT ALL THE 
C EXPRESS SUBROUTINES
C
      integer*2 mxhold,nunit,otty
      parameter(mxhold=2557,nunit=99,otty=6)
C
C VARIABLES WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      logical*1 mode(nunit)
      integer*4 ipoint(nunit)
      integer*2 nword(nunit)
      integer*2 idata(nunit),ndata(nunit),mxdata(nunit)
      integer*2 ntotal(nunit),ntot1(nunit),ntot2(nunit)
      integer*2 ndftot(nunit),mxtot(nunit)
      real*4 hold(mxhold,nunit)
C
C COMMON BLOCKS WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      common /filblk/ mode,ipoint
      common /datblk/ nword,idata,ndata,mxdata
      common /totblk/ ntotal,ntot1,ntot2,ndftot,mxtot
      common /hldblk/ hold
C
C END OF BLOCK DATA
C
C
C BRANCH ACCORDING TO CURRENT MODE(IUNIT)
C
      goto(10,20,30),mode(iunit)
      write(otty,801) iunit
  801 format(x,'*** BACKSPACEXP -- UNIT',i4,' NOT OPENED')
      info=-5
      return
C
C READING MODE - DECREMENT HOLDING ARRAY INDEX
C
   30 idata(iunit)=idata(iunit)-nword(iunit)
      info=0
      if(idata(iunit).ge.1) return
C
C IF IDATA(IUNIT) LESS THAN 1 BACKSPACE EXPRESS UNIT
C
      if(ipoint(iunit).le.2) then
        call rewindexp(iunit,info)
        return
      end if
      backspace iunit
      backspace iunit
      call yank(iunit,nword(iunit),ndata(iunit),ntotal(iunit),
     1          hold(1,iunit),info)
      ipoint(iunit)=ipoint(iunit)-1
      idata(iunit)=ndata(iunit)
C
C OPENING MODE - NO ACTION NECESSARY
C
   20 return
C
C WRITING MODE - DECREMENT HOLDING ARRAY INDEX
C
   10 idata(iunit)=idata(iunit)-nword(iunit)
      info=0
      if(idata(iunit).ge.1) return
C
C IF IDATA(IUNIT) LESS THAN 1 BACKSPACE EXPRESS UNIT
C
      if(ipoint(iunit).le.1) then
        call rewindexp(iunit,info)
        return
      end if
      if(ndata(iunit).ge.1) then
        call outrec(iunit,nword(iunit),ndata(iunit),
     1              ntotal(iunit),hold(1,iunit),ierr)
        backspace iunit
      end if
      backspace iunit
      call yank(iunit,nword(iunit),ndata(iunit),ntotal(iunit),
     1          hold(1,iunit),info)
      mode(iunit)=3
      idata(iunit)=ndata(iunit)
      ntot2(iunit)=ntot1(iunit)
      ntot1(iunit)=ntotal(iunit)
      return
      end
C
C
C >>>>>  SUBROUTINE  T E S T E X P <<<<<.......................
C
C SUBROUTINE TEST EXPRESS CHECKS THE EXPRESS UNIT PARAMETERS
C AND THEN RETURNS THE BEST VALUES FOR THE FOLLOWING:
C
C    NWORD0 - FOR FILES WITH ONE OR MORE RECORDS THE VALUE FOR
C             NWORD0 IN THE FIRST RECORD IS RETURNED.
C             FOR FILES WITH NO RECORDS THE INITIAL VALUE.
C    LENREC - FOR FILES WITH 2 OR MORE RECORDS THE LONGEST OF THE
C             FIRST TWO RECORD LENGTHS IS RETURNED.
C             FOR FILES WITH 1 RECORD THE LARGEST OF THE INITIAL
C             VALUE OR THE LENGTH OF THE RECORD.
C             FOR FILES WITH NO RECORDS THE INITIAL VALUE.
C    IPOST  - THE NUMBER OF THE POST GROUP ABOUT TO BE READ OR
C             WRITTEN TO.  (THE FIRST POST GROUP IN EACH RECORD
C             IS NUMBER 1).
C    IREC   - THE EXPRESS FILE POINTER POSITION.  (THE FIRST
C             RECORD IN THE EXPRESS FILE IS NUMBER 1).
C
C IT ALSO RETURNS INFORMATION IN VARIABLE INFO:
C
C    INFO=0  - PROCEDURE COMPLETED.
C    INFO=1  - PROCEDURE COMPLETED.  FILE HAS NO RECORDS.
C    INFO=2  - PROCEDURE COMPLETED.  FILE HAS ONE RECORD.
C    INFO=-5 - IUNIT IS NOT OPENED.
C
C SUBROUTINE TESTEXP WRITTEN BY ROB BRACKEN, USGS, 10DEC86
C
C
      subroutine testexp(iunit,post,info,nword0,lenrec,ipost,
     1                   irec)
C
C VARIABLE DECLARATION
C
C VARIABLES WHICH MAY BE PASSED TO AND FROM THE SUBROUTINES
C VIA COMMON BLOCK /PASBLK/
C
      integer*4 iunit,info,nword0,lenrec,ipost,irec
      parameter(mxword=32)
      real*4 post(mxword)
c      common /pasblk/ iunit,post,info,nword0,lenrec,ipost,irec
C
C DIMENSION SIZES WHICH MUST BE USED THROUGHOUT ALL THE 
C EXPRESS SUBROUTINES
C
      integer*2 mxhold,nunit,otty
      parameter(mxhold=2557,nunit=99,otty=6)
C
C VARIABLES WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      logical*1 mode(nunit)
      integer*4 ipoint(nunit)
      integer*2 nword(nunit)
      integer*2 idata(nunit),ndata(nunit),mxdata(nunit)
      integer*2 ntotal(nunit),ntot1(nunit),ntot2(nunit)
      integer*2 ndftot(nunit),mxtot(nunit)
      real*4 hold(mxhold,nunit)
C
C COMMON BLOCKS WHICH ARE INTERNAL TO THE EXPRESS SUBROUTINES
C
      common /filblk/ mode,ipoint
      common /datblk/ nword,idata,ndata,mxdata
      common /totblk/ ntotal,ntot1,ntot2,ndftot,mxtot
      common /hldblk/ hold
C
C END OF BLOCK DATA
C
C
C BRANCH ACCORDING TO MODE
C
      goto(10,20,30),mode(iunit)
      write(otty,801) iunit
  801 format(x,'*** TESTEXP -- UNIT',i4,' NOT OPENED')
      info=-5
      return
C
C WRITING MODE - RETURN PRESENT PARAMETERS
C
   10 nword0=nword(iunit)
      lenrec=ntotal(iunit)+3
      ipost=(idata(iunit)+nword(iunit)-1)/nword(iunit)
      irec=ipoint(iunit)
      info=0
      return
C
C READING MODE - RETURN PRESENT PARAMETERS
C
   30 nword0=nword(iunit)
      lenrec=ntotal(iunit)+3
      ipost=(idata(iunit)+nword(iunit)-1)/nword(iunit)
      irec=ipoint(iunit)-1
      info=0
      return
C
C OPENING MODE - SET PRELIMINARY NWORD0 AND LENREC
C
   20 if(nword0.lt.1) nword0=3
      if(nword0.gt.mxword) nword0=mxword
      if(lenrec.lt.nword0+3) lenrec=nword0+3
      if(lenrec.gt.mxhold+3) lenrec=mxhold+3
      ipost=1
      irec=1
      info=1
C
C READ BEGINNING OF FIRST RECORD IN EXPRESS FILE
C
      ntot1(iunit)=lenrec-3
      ntotal(iunit)=1
      call inrec(iunit,nword(iunit),ndata(iunit),ntotal(iunit),
     1           hold(1,iunit),ierr,iend)
      if(ierr.eq.-4) return
      if(iend.eq.2) return
      info=2
C
C SET NWORD0 AND LENREC BASED ON FIRST RECORD ONLY
C
      nword0=nword(iunit)
      lenrec=imax0(ntotal(iunit),ntot1(iunit))+3
C
C READ BEGINNING OF SECOND RECORD IN EXPRESS FILE
C
      ntot1(iunit)=ntotal(iunit)
      ntotal(iunit)=1
      call inrec(iunit,nword(iunit),ndata(iunit),ntotal(iunit),
     1           hold(1,iunit),ierr,iend)
      if(ierr.eq.-4) return
      rewind iunit
      if(iend.eq.2) return
C
C SET LENREC BASED ON FIRST TWO RECORDS
C
      lenrec=imax0(ntotal(iunit),ntot1(iunit))+3
      info=0
      return
      end
C
C
C >>>>>  SUBROUTINE  O U T R E C  <<<<<........................
C
C SUBROUTINE OUTPUT RECORD OUTPUTS ONE RECORD TO THE EXPRESS
C UNIT
C
C SUBROUTINE OUTREC WRITTEN BY ROB BRACKEN, USGS, 10DEC86
C
C
      subroutine outrec(iunit,nword,ndata,ntotal,hold,ierr)
C
C VARIABLE DECLARATION
C
      integer*4 iunit,ierr,iend
      integer*2 nword,ndata,ntotal
      integer*4 nword4,ndata4,ntot4
      dimension hold(ntotal)
C
C WRITE ONE RECORD
C
      ierr=0
      nword4=nword
      ndata4=(ndata-1+nword)/nword
      ntot4=ntotal
      write(iunit,err=901) nword4,ndata4,ntot4,hold
      return
  901 ierr=-4
      return
      end
C
C
C >>>>>  SUBROUTINE  Y A N K  <<<<<............................
C
C SUBROUTINE YANK READS A RECORD FROM THE EXPRESS UNIT AND
C DETERMINES THE SIZE OF THE RECORD.  THEN IT ADJUSTS THE SIZE
C OF THE RECORD AND RE-READS IT IF NECESSARY.
C
C SUBROUTINE YANK WRITTEN BY ROB BRACKEN, USGS, 10DEC86
C
C
      subroutine yank(iunit,nword,ndata,ntotal,hold,iend)
C
C VARIABLE DECLARATION
C
      integer*4 iunit,iend
      integer*2 nword,ndata,ntotal,ntot0
      parameter(mxhold=2557)
      dimension hold(mxhold)
C
C YANK RECORD
C
      icount=1
   10 ntot0=ntotal
      call inrec(iunit,nword,ndata,ntotal,hold,ierr,iend)
      if(iend.eq.2) return
      if(ntotal.le.ntot0) return
      if(icount.ge.2) then
        iend=ierr
        return
      end if
      icount=icount+1
      backspace iunit
      goto 10
      end
C
C
C >>>>>  SUBROUTINE  I N R E C  <<<<<..........................
C
C SUBROUTINE INPUT RECORD READS ONE RECORD FROM THE EXPRESS
C UNIT
C
C SUBROUTINE INREC WRITTEN BY ROB BRACKEN, USGS, 10DEC86
C
C
      subroutine inrec(iunit,nword,ndata,ntotal,hold,ierr,iend)
C
C VARIABLE DECLARATION
C
      integer*4 iunit,ierr,iend
      integer*2 nword,ndata,ntotal
      integer*4 nword4,ndata4,ntot4
      dimension hold(ntotal)
C
C READ IN RECORD
C
      ierr=0
      iend=0
      read(iunit,err=901,end=902) nword4,ndata4,ntot4,hold
   10 nword=nword4
      ndata=ndata4*nword4-nword4+1
      ntotal=ntot4
      return
  901 ierr=-4
      goto 10
  902 iend=2
      return
      end
