c*******************************************************************************
c Notes for future revision:  make psdmag,psdgrv,redpol, & vertot itypes=1-4
c       program fftfil
c   filter operations employing f.f.t.
c   coded by T.G. Hildenbrand--usgs Denver 303-234-5464
c   Added function vertot, coded by Hildenbrand.  T. Grauch  Sept. 20, 1988
c        (parameters xinc,dec,bdec,binc required for vertot)
c   (subroutines 'sfftmg' & 'sdummy' coded by R. Watts)
c   structure of command file--
c    card 1 : operation in coded name from list below
c              psdmag    pseudo-magnetic transformation
c              psdgrv    pseudo-gravity transformation
c              redpol    reduction of magnetics to the north pole
c              upcont    upward continuation
c              dncont    downward continuation
c              1stver    1st-vertical derivative of the input field
c              2ndver    2nd-vertical derivative of the input field
c              banpas    bandpass filter
c              strike    directional filtering.
c	       vertot    change vertical-component data to total-field
c              nofilt    no filtering
c    card 2 : input file name.ext.
c    card 3 : output file name.ext.
c    card 4 : title (col. 1-56).
c    card 6 : &parms
c    card 6 :  list parameters--example: iopt1=0,iopt2=1,den=.3,.....
c    card 7 : &end
c    the following queries are asked after filter operation
c    has been completed. purpose of queries are to continue filtering
c    input file with different filters.
c    query 1 : additional filter to be applied? (y or n) - if 'y'
c              the program asks queries 2 thru 6. if 'n' the
c              program starts over by asking for the command segment.
c    query 2 : new operator? format same as in card 1 response.
c    query 3 : new output file name.ext?
c    query 4 : new title?
c    query 5 : parmameter change? (y or n) - if 'n' the filtering
c              computations are started. if 'y' the user enters the
c              parmeters to be changed in a namelist
c              (eg. &parms thet1=-90.,thet2=0.,&)
c              (note the following paramters cannot be changed-nadd,
c              iopt2,idval,ddx,ddy,xo,yo).
c    query 1 is then repeated for additional filter operations.
c   input parameters:
c    iopt1 = 0 no printed output (default: iopt1=0)
c            6 output printed on terminal
c           -1 output printed on disk.
c    iopt2 = -1 no removal of mean from input array (default iopt2=-1)
c             0 remove mean using boundary values
c             1 remove mean and save grid, file name: con.grd.
c    nadd = no. of rows or columns added to each side of grid
c           to reduce the effects of gibbs phenomena (default nadd=0).
c    w1,w2,w3 & w4 - wavelengths used in bandpass filtering.
c                    (default w1=w2=0. w3=w4=1.0e+30 infinte wavelength)
c    den - density contrast, gm/cc (default den=1.).
c    bmag - magnetization contrast, gammas (default bmag=1.).
c    dec & xinc - declination and inclination of earth''s field,degrees.
c                 (default dec=0. xinc=90.)
c    bdec & binc - declination and inclination of magnetization vector.
c                  (default bdec=0. binc=90.)
c    idval = 0 no flagged grid points in input data (default idval=0)
c            1 flagged grid points in input data
c           -1 flagged values removed and locations in file 'flag.loc'.
c    icoef = 1 save fourier coefficients in segment 'fftfil.cof' for
c              later use but perform designated filter operation
c              (note: if data contains flagged values a segment
c              called 'flag.loc' containing their locations is also
c              created in users disk area.)
c            0 fourier coefficients not saved (default icoef=0).
c           -1 fourier coefficients in segment 'fftfil.cof' are used
c              as input. note that if flagged values are present in the
c              data, segment 'flag.loc' is required and 'idval' must be
c              equal to -1. in addition, the parameter 'nadd' must be
c              indentical to its assigned value when the fourier
c              coefficients were saved.
c     z = continuation distance (must be in grid units) (default z=0.)
c         for z>0 downward continuation, for z<0 upward continuation
c    thet1 & thet2 - angles from geographic north that form a pie-slice
c                    filter for directional filtering(-90.ge.thet.le.
c                    +90.;thet2.gt.thet1). (default thet1=0. thet2=90.)
c    istr = -1 reject trends between thet1 and thet2.
c           +1 pass (default istr=1).
c   (following parameters can be used if spacing and origin
c   of grid want to be changed. note that these changes
c   only occur in the header of the output filtered grid and no
c   subset of the input grid is extracted).
c    ddx - new grid spacing in x-direction.
c    ddy - new grid spacing in y-direction.
c    xo - new origin of rows.
c    yo - new origin of coloumns. (default ddx=ddy=xo=yo=0.)
c***********************************************************************
      dimension a1(2,2048,64),work(4096),title(14)
      character*50 tname,coname,fname,conout
      character help*4,blank*2,exit*2,fopr*6,flist(11)*6,iz*12,blid*4
      common/main/id(14),pgm(2),nz,yo,xo,dx,dy,iw,kr,ny,nadd,ir
      common/parm1/fname,coname,iopt1,iopt2,idval,icoef,ddx,ddy
      common/parm2/den,dec,xinc,bmag,bdec,binc,z,thet1,thet2,istr,w1,w2,
     1w3,w4
      equivalence (id(1),blid)
      data icmd/10/,lnri/64/
      data nlist,flist/11,'psdgrv','redpol','upcont','dncont','2ndver',
     1'strike','banpas','psdmag','1stver','vertot','nofilt'/
      data blank/'  '/,exit/'ex'/,help/'help'/
      kr=11
      iw=6
      ir=5
c read command file and header record of input file holding data array.
	print*,'** New FFTFIL version 1, 9/20/88'
	print*,'new operator vertot changes vertical-comp. to total-field'
	print*
   10 write(iw,15)
   15 format(' Command file (car ret to exit): ',$)
      read(ir,20)tname
   20 format(a50)
      if(tname.eq.exit.or.tname.eq.blank) go to 350
c  default values of input parameters.
      iopt1=0
      iopt2=-1
      icoef=0
      nadd=0
      w1=0.
      w2=0.
      w3=1.0e+30
      w4=1.0e+30
      den=1.
      z=1.
      dec=0.0
      xinc=90.
      bmag=1.
      bdec=0.0
      binc=90.
      xo=0.0
      yo=0.0
      ddx=0.0
      ddy=0.0
      thet1=0.0
      thet2=90.
      istr=1
      idval=0
      z=0.0
c  read command file.
      open(10,status='old',form='formatted',file=tname,readonly)
      read(icmd,30)fopr
   30 format(a6)
      if(fopr.ne.blank) go to 50
   40 write(iw,45)
   45 format(' enter operator: ',$)
      read(ir,30)fopr
   50 if(fopr.eq.help) go to 80
      do 60 i=1,nlist
      if(fopr.ne.flist(i)) go to 60
      itype=i
      go to 110
   60 continue
      write(iw,70)
   70 format(' #invalid operator')
   80 write(iw,90)
   90 format(' quick list of options')
      write(iw,100)flist
  100 format(2x,a6)
      go to 40
  110 read(icmd,20)fname
      if(fname.ne.blank) go to 120
      write(iw,115)
  115 format(' enter input file name: ',$)
      read(ir,20)fname
  120 read(icmd,20)coname
      if(coname.ne.blank) go to 130
      write(iw,125)
  125 format(' enter output file name: ',$)
      read(ir,20)coname
  130 read(icmd,140)id
      if(blid.ne.blank) go to 150
      write(iw,135)
  135 format(' enter title: ')
      read(ir,140)id
  140 format(14a4)
  150 call snamel(icmd)
      close(10)
c  test for parameter errors.
      if(z.ge.0.0.and.fopr.eq.'upcont') go to 300
      if(z.le.0.0.and.fopr.eq.'dncont') go to 300
      if(fopr.eq.'nofilt'.and.icoef.eq.0) go to 280
      if(abs(thet1).gt.90..or.abs(thet2).gt.90.) go to 280
      if(thet1.gt.thet2) go to 280
      if(w1.gt.w2.or.w2.gt.w3.or.w3.gt.w4) go to 280
      if(abs(binc).gt.90..or.abs(xinc).gt.90.) go to 280
      if(iopt1)170,180,160
  160 if(iopt1.ne.6) go to 280
      go to 180
  170 iopt1=20
      conout='fftfil.out'
      open(20,status='new',form='formatted',file=conout)
c  read header record of input file.
  180 open(11,status='old',form='unformatted',file=fname,readonly)
c  note that x & y have been switched from normal (usgs) grid specficati
c  ...convention used is x-north y-east and z-down
      read(kr)title,pgm,n2,n1,nz,yop,dy,xop,dx
      if(icoef.eq.-1.and.fopr.eq.'nofilt')itype=12
      pgm(1)='fftf'
      pgm(2)='il**'
      nx=n1
      ny=n2
      if(ddx.ne.0.0)dx=ddx
      if(ddy.ne.0.0)dy=ddy
      if(xo.ne.0.0)xop=xo
      if(yo.ne.0.0)yop=yo
      xo=xop
      yo=yop
      l=lnri+1
      lnri21=lnri/2+1
c  set no. of rows for fft: need m=l*2**k, m.gt. or .eq. nx+2*nadd.
c  m=no. of rows, l=no. from 9-64, k=interger.
      n1=n1+2*nadd
  190 l=l-1
      if(l.lt.lnri21) go to 260
      mr=n1/l+0.0000001
      k=1
      idiv=2
  200 iquot=mr/idiv+0.0000001
      if(iquot.lt.idiv) go to 210
      k=k+1
      mr=iquot
      go to 200
  210 k=k+1
      m=l*2**k
  220 mtest=l*2**(k-1)
      if(mtest.lt.n1) go to 230
      k=k-1
      m=mtest
      if(k.eq.0) go to 230
      go to 220
  230 lnxa=m-n1
      if(l.ne.lnri) go to 250
      nxa64=lnxa
  240 nri=l
      nxa=lnxa
      go to 190
  250 if(lnxa.ge.(nxa-1)) go to 190
      go to 240
  260 n1=n1+nxa
c  check to see if row block size of 16 will be more efficient
      n164=n1-nxa+nxa64
      ntest=0.9*n164
      if(ntest.gt.n1.or.n164.gt.2048) go to 270
      n1=n1-nxa+nxa64
      nxa=nxa64
      nri=64
  270 n2=n2+2*nadd
      write(iw,275)nri
  275 format(' blocking rows =',i3)
      if(n1.gt.2048.or.n2.gt.2048) go to 320
      nxa=nxa+2*nadd
      id2=n1
      if(n2.gt.n1)id2=n2
      id2=2*id2
c  input parameters for fft are set: call main program.
      call sfftfil(a1,work,nx,nri,itype,nxa,n1,n2,id2)
      if(iopt1.eq.20)close(20)
      go to 10
c  errors resulting in job abortion.
  280 write(iw,290)
  290 format(' #parameter error detected---case run aborted')
      go to 10
  300 write(iw,310)
  310 format(' #sign of z does not conform to continuation operator')
      go to 10
  320 write(iw,330)nx,ny,n1,n2
  330 format(' #no. of extended rows or columns exceeds 2048:'/
     1' input no. of rows and columns='2i4/
     2' no. of rows and columns required for filtering=',2i4,/)
  340 close(11)
      go to 10
  350 write(iw,360)
  360 format(' end of job')
      stop
      end
c***********************************************************************
c    subroutine 'snamel' reads parms from namelist section
c    iread=5 (terminal read) or 10 (read from file10)
c***********************************************************************
      subroutine snamel(iread)
      character*50 fname,coname
      common/main/id(14),pgm(2),nz,yo,xo,dx,dy,iw,kr,ny,nadd,ir
      common/parm1/fname,coname,iopt1,iopt2,idval,icoef,ddx,ddy
      common/parm2/den,dec,xinc,bmag,bdec,binc,z,thet1,thet2,istr,w1,w2,
     1w3,w4
      namelist/parms/iopt1,iopt2,den,dec,xinc,bmag,bdec,binc,idval,ddx,
     1ddy,xo,yo,thet1,thet2,istr,nadd,w1,w2,w3,w4,icoef,z
      read(iread,parms,end=10)
      return
   10 write(iw,20)
   20 format('#odd eof in parms command file--program aborted')
      if(iread.eq.10)close(10)
      stop
      end
c***********************************************************************
c   subroutine 'sfftfil' removes flagged values from grid, transforms
c   data, calls routines to apply filter, and writes filtered grid
c***********************************************************************
          subroutine sfftfil(a1,work,nx,nri,itype,nxa,n1,n2,id2)
          dimension a1(2,n2,nri),work(id2),jflag(2,20),title(14)
          character flist(10)*6,help*4,quer*1,fopr*6
          character*50 coname,fname,fileo
          common/main/id(14),pgm(2),nz,yo,xo,dx,dy,iw,kr,ny,nadd,ir
          common/basic/c1,el,em,en,bl,bm,bn
          common/parm1/fname,coname,iopt1,iopt2,idval,icoef,ddx,ddy
          common/parm2/den,dec,xinc,bmag,bdec,binc,z,thet1,thet2,istr,w1
     1    ,w2,w3,w4
          common/tr/prad,p,q,th1p,th2p
          data help/'help'/,nlist,flist/10,'psdgrv','redpol','upcont',
     1   'dncont','2ndver','strike','banpas','psdmag','1stver','nofilt'/
          data iunita/10/,iunitb/12/,dval/'ffff7fff'x/
          data gamma/6.67323/,pi2/6.28318531/,pi/3.141592654/,iunitc/15/
          ncase=0
      n22=2*n2
          if(icoef.ne.-1) go to 10
          ncase=1
         open(15,access='direct',status='old',form='unformatted',
     1   file='fftfil.cof',recl=n22,maxrec=n1
     1 ,organization='relative',recordtype='fixed')
         open(12,access='direct',status='new',form='unformatted',
     1   file='slave2.tmp',recl=n22,maxrec=n1,
     1 organization='relative',recordtype='fixed')
          close(11)
          go to 20
c   remove mean using boundary values.
   10     if(iopt2.lt.0) go to 20
          call srmean(nx,ny,a1)
          fileo='fftfil.grd'
          open(11,status='old',form='unformatted',file=fileo)
          read(kr)title,pgm,ny,nx,nz,yo,dy,xo,dx
c  compute beginning and end column & row in original grid.
   20     do 30 i=1,2
          do 30 j=1,n2
   30     a1(2,j,i)=0.0
          nap1=nadd+1
          n2a=n2-nadd
          nxap1=nxa*0.5+1
          fxap1=float(nxa)/2.+1.
          if(abs(fxap1-float(nxap1)).gt.0.0001)nxap1=nxap1+1
          n1a=nxap1-1+nx
c  remove flagged values and add '2*nadd' rows & columns to
c  reduce the effect of gibbs phenomena.
         open(10,access='direct',status='new',form='unformatted',
     1   file='slave1.tmp',recl=n22,maxrec=n1,
     1 organization='relative',recordtype='fixed')
      if(ncase.eq.1) go to 590
          if(idval.eq.1)open(14,status='new',form='unformatted',
     1    file='flag.loc')
          irr=1
          iwr=0
          jr=0
   40     jr=jr+1
          if(idval.lt.1) go to 310
c   idval>0: removal of flagged values.
          if(jr.gt.1) go to 140
c   read 1st row & remove flagged values.
          read(kr)dum,(a1(1,i,1),i=nap1,n2a)
          read(kr)dum,(a1(1,i,2),i=nap1,n2a)
          nflag=1
          i=nap1
   50     jflag(1,1)=nap1
          if(a1(1,nap1,1).le.1.0e38) go to 90
          do 60 i=nap1,n2a
   60     if(a1(1,i,1).le.1.0e38) go to 70
          go to 1270
   70     jflag(1,1)=i
          aval=a1(1,i,1)
          do 80 ii=nap1,i-1
   80     a1(1,ii,1)=aval
   90     if(i.ge.n2a) go to 100
          do 100 ii=i+1,n2a
          if(a1(1,ii,1).le.1.0e38) go to 100
          jflag(2,nflag)=ii-1
          go to 110
  100     continue
          jflag(2,nflag)=n2a
          write(14)nflag,(jflag(1,j),jflag(2,j),j=1,nflag)
          go to 350
  110     aval=a1(1,ii-1,1)
          do 120 i=ii,n2a
          if(a1(1,i,1).le.1.0e38) go to 130
  120     a1(1,i,1)=aval
          write(14)nflag,(jflag(1,j),jflag(2,j),j=1,nflag)
          go to 350
  130     a1(1,i-1,1)=(a1(1,i-1,1)+a1(1,i,1))*0.5
          nflag=nflag+1
          jflag(1,nflag)=i
c   1st row completed: now extend grid & write to disk.
          go to 90
c   remove flagged values from remaining rows.
  140     if(jr.ge.nx) go to 150
          read(kr)dum,(a1(1,i,3),i=nap1,n2a)
          go to 170
  150     do 160 i=nap1,n2a
  160     a1(1,i,3)=dval
  170     nflag=1
          i=nap1
          jflag(1,1)=nap1
          if(a1(1,nap1,2).le.1.0e38) go to 220
          do 180 i=nap1,n2a
  180     if(a1(1,i,2).le.1.0e38) go to 190
          i=nap1
          jflag(1,1)=n2a+1
          if(a1(1,nap1,3).ge.1.0e38)a1(1,nap1,2)=a1(1,nap1,1)
          if(a1(1,nap1,3).le.1.0e38)a1(1,nap1,2)=(a1(1,nap1,1)+
     1    a1(1,nap1,3))*0.5
          go to 220
  190     jflag(1,1)=i
          ii=i
          do 210 irev=nap1,i-1
          ii=ii-1
          if(a1(1,ii,3).ge.1.0e38) go to 200
          a1(1,ii,2)=(a1(1,ii,1)+a1(1,ii+1,2)+a1(1,ii,3))*0.3333333
          go to 210
  200     a1(1,ii,2)=(a1(1,ii,1)+a1(1,ii+1,2))*0.5
  210     continue
  220     if(i.ge.n2a) go to 230
          do 230 ii=i+1,n2a
          if(a1(1,ii,2).le.1.0e38) go to 230
          jflag(2,nflag)=ii-1
          go to 240
  230     continue
          jflag(2,nflag)=n2a
          write(14)nflag,(jflag(1,j),jflag(2,j),j=1,nflag)
          go to 390
  240     do 290 i=ii,n2a
          if(i.eq.n2a) go to 250
          if(a1(1,i+1,2).le.1.0e38) go to 270
  250     if(a1(1,i,3).ge.1.0e38) go to 260
          a1(1,i,2)=(a1(1,i,1)+a1(1,i-1,2)+a1(1,i,3))*0.3333333
          go to 290
  260     a1(1,i,2)=(a1(1,i,1)+a1(1,i-1,2))*0.5
          go to 290
  270     if(a1(1,i,3).ge.1.0e38) go to 280
          a1(1,i,2)=(a1(1,i,1)+a1(1,i-1,2)+a1(1,i+1,2)+a1(1,i,3))*0.25
          go to 300
  280     a1(1,i,2)=(a1(1,i,1)+a1(1,i-1,2)+a1(1,i+1,2))*0.3333333
          go to 300
  290     continue
          write(14)nflag,(jflag(1,j),jflag(2,j),j=1,nflag)
          go to 390
  300     nflag=nflag+1
          i=i+1
          jflag(1,nflag)=i
          go to 220
c   idval.eq.0: read & check input data.
  310     if(jr.ne.1)irr=2
          read(kr)dum,(a1(1,i,irr),i=nap1,n2a)
          do 320 ii=nap1,n2a
  320     if(a1(1,ii,irr).ge.1.0e38) go to 330
          if(jr.ne.1) go to 390
          go to 350
  330     write(iw,340)jr,ii
  340     format(' #flagged value row=',i4,3x,'col=',i4,' may be more?')
          close(11)
          close(10)
          return
c   extend row & write to disk.
  350     do 360 i=1,nap1
  360     a1(1,i,1)=a1(1,nap1,1)
          do 370 i=n2a,n2
  370     a1(1,i,1)=a1(1,n2a,1)
  380     iwr=iwr+1
          call swrda(iunita,iwr,a1,n22)
          if(iwr.eq.nxap1) go to 40
          go to 380
  390     do 400 i=1,nap1
  400     a1(1,i,2)=a1(1,nap1,2)
          do 410 i=n2a,n2
  410     a1(1,i,2)=a1(1,n2a,2)
          iwr=iwr+1
          call swrda(iunita,iwr,a1(1,1,2),n22)
          if(iwr.ge.n1a) go to 430
          if(idval.lt.1) go to 40
          do 420 i=nap1,n2a
          a1(1,i,1)=a1(1,i,2)
  420     a1(1,i,2)=a1(1,i,3)
          go to 40
  430     if(iwr.ge.n1) go to 450
  440     iwr=iwr+1
          call swrda(iunita,iwr,a1(1,1,2),n22)
          if(iwr.lt.n1) go to 440
  450 if(iopt2.ne.0)close(11)
      if(iopt2.eq.0)close(11,disp='delete')
      if(idval.eq.1)close(14)
  460     format(i5)
c  write headings
  470     if(ncase.eq.0) go to 590
  480 write(iw,485)
  485 format(' enter new operator: ',$)
          read(ir,490)fopr
  490     format(a6)
  500     if(fopr.eq.help) go to 530
          do 510 i=1,nlist
          if(fopr.ne.flist(i)) go to 510
          itype=i
          go to 560
  510     continue
          write(iw,520)
  520     format(' #invalid operator')
  530     write(iw,540)
  540     format(' quick list of options')
          write(iw,550)flist
  550     format(2x,a6)
          go to 480
  560 write(iw,565)
  565 format(' enter new output file name: ',$)
          read(ir,570)coname
  570     format(a50)
      write(iw,575)
575   format(' enter new title: ',$)
          read(ir,580)id
  580     format(14a4)
      write(iw,585)
  585 format(' parm change? if y enter parameters via &parms): ',$)
          read(ir,570)quer
          if(quer.ne.'y') go to 590
          call snamel(ir)
  590     ip=iopt1
          ih=iw
          if(ip.ne.0)ih=ip
          pid=pi/180.
          rn1=1./(float(n1)*dx)
          rn2=1./(float(n2)*dy)
          c2=pi2*z
          go to (620,640,670,690,710,730,780,600,890,595,930,910),itype
  595	  write(ih,597)
  597     format(1h1,'transformation of vertical-component magnetic ',
     1    'data to total-field data,',/,' by means of the fast fourier',
     2    ' transform and poissons equation.',/)
          go to 640
  600     write(ih,610)
  610     format(1h1,'transformation of the gravity field to the ',
     1    'pseudo-magnetic field,',/,' by means of the fast fourier',
     2    ' transform and poissons equation.',/)
          go to 640
  620     write(ih,630)
  630     format(1h1,'transformation of total magnetic intensity field',
     1    ' to pseudo-gravitational field,',/,' by means of the fast ',
     2    'fourier transform and the poisson equation.',/)
  640     rbdec=bdec*pid
          rbinc=binc*pid
          cosrb=cos(rbinc)
          bl=cosrb*cos(rbdec)
          bm=cosrb*sin(rbdec)
          bn=sin(rbinc)
          rdec=dec*pid
          rinc=xinc*pid
          cosr=cos(rinc)
          el=cosr*cos(rdec)
          em=cosr*sin(rdec)
          en=sin(rinc)
          if(itype.eq.2) go to 650
          c1=(gamma*den)/(pi2*bmag)
          go to 950
  650     write(ih,660)
  660     format(1h1,'transformation of the total magnetic field ',
     1    'intensity to the pole,',/,' by means of the fast fourier ',
     2    'transform and the poisson equation.',/)
          go to 950
  670     write(ih,680)
  680     format(1h1,'upward continuation of field values,',/,
     1    ' by means of the fast fourier transform.',/)
          go to 950
  690     write(ih,700)
  700     format(1h1,'downward continuation of field values,',/,
     1    ' by means of the fast fourier transform.',/)
          go to 950
  710     write(ih,720)
  720     format(1h1,'second vertical derivative operation,',/,
     1    ' by means of the fast fourier transform.',/)
          c4=4.*pi*pi
          go to 950
  730     write(ih,740)
  740     format(1h1,'strike sensitive filtering using the fast ',
     1    'fourier transform.',/)
          p=1.
          q=0.0
          if(istr.ne.1) go to 760
          if(thet1.lt.0.0.and.thet2.ge.0.0) go to 750
          go to 770
  750     p=0.0
          q=1.
          go to 770
  760     if(thet1.lt.0.0.and.thet2.ge.0.0) go to 770
          p=0.0
          q=1.
  770     th1p=180.-thet1
          th2p=180.-thet2
          if(th1p.gt.180.)th1p=th1p-180.
          if(th2p.gt.180.)th2p=th2p-180.
          prad=180./pi
          go to 950
  780     write(ih,790)
  790     format(1h1,'band-pass filter applied to data',/)
          if(w1.ne.0.0) go to 800
          f4=1.0e+30
          go to 810
  800     f4=1./w1
  810     if(w2.ne.0.0) go to 820
          f3=1.0e+30
          go to 830
  820     f3=1./w2
  830     if(w3.ne.1.0e+30) go to 840
          f2=0.
          go to 850
  840     f2=1./w3
  850     if(w4.ne.1.0e+30) go to 860
          f1=0.
          go to 870
  860     f1=1./w4
  870     if(w3.eq.w4) go to 880
          btan12=1./(f2-f1)
  880     if(w1.eq.w2) go to 950
          btan34=1./(f4-f3)
          go to 950
  890     write(ih,900)
  900     format(1h1,'first vertical derivative operation,',/,
     1    ' by means of the fast fourier transform.',/)
          c4=pi2
          go to 950
  910     write(ih,920)
  920     format(1h1,' no filter operation performed: fourier coeff.''s',
     1    ' are inputed and inverse transformed',/)
          go to 950
  930     write(ih,940)
  940     format(1h1,' no filter operation performed: only fourier ',
     1    'coef.s are outputed in fftfil.cof',/)
  950     write(ih,960)id,den,z,bmag,xinc,dec,binc,bdec,thet1,thet2,
     1     istr,w1,w2,w3,w4,dx,dy,xo,yo,ny,nx,nadd,n2,n1,fname,coname
  960     format(' title: ',14a4,/' parameters: density=',f7.3,5x,
     1    'datum level z=',f7.3/13x,'intensity of magnetization=', f9.3
     2    /13x,'inclination & declination of geomagnetic field=',2f9.3
     3    /13x,'inclination & declination of magnetization vector=',
     4    2f9.3/13x,'thet1=',f9.3,5x,'thet2=',f9.3,5x,'istr=',i2/13x,
     5    'w1,w2,w3&w4=',4e11.4/13x,'dx & dy=',2e14.6/13x,
     6    'origin of grid(xo & yo)=',2e14.6/13x,'no. of columns & rows='
     7    ,2i5/13x,'nadd=',i5,'  extended columns & rows=',2i5/13x,
     8    'input file name: ',a50/13x,'output file name: ',a50,/)
c  obtain complex fourier coefficients (f.f.t. a1).
          if(ncase.eq.0) go to 980
          jr=0
          do 970 i=1,n1
          jr=jr+1
          call srdda(iunitc,jr,a1,n22)
  970     call swrda(iunita,jr,a1,n22)
          if(itype.eq.12) go to 1110
          go to 1000
  980     isign=-1
          open(12,access='direct',status='new',form='unformatted',
     1    file='slave2.tmp',recl=n22,maxrec=n1
     1 ,organization='relative',recordtype='fixed')
          call sfftmg(iunita,iunitb,n2,n1,a1,nri,isign,work)
          open(15,access='direct',status='new',form='unformatted',
     1    file='fftfil.cof',recl=n22,maxrec=n1,
     1 organization='relative',recordtype='fixed')
          jr=0
          do 990 i=1,n1
          jr=jr+1
          call srdda(iunita,jr,a1,n22)
  990     call swrda(iunitc,jr,a1,n22)
          if(itype.eq.11) go to 1250
      write(iw,995)
  995 format(' forward transform completed')
c  a1 array now holds complex fourier coefficients.
 1000     n1p1=n1+1
          n2p1=n2+1
          nnh1=(n1/2)+1
          nnh2=(n2/2)+1
c  operating on a1 fourier coefficients.
          jr=0
          jmr=n1+1
 1010     jr=jr+1
          if(jr.gt.nnh1) go to 1110
          jj=jr-1
          x=float(jj)*rn1
          xsq=x*x
          call srdda(iunita,jr,a1,n22)
          do 1090 i=1,n2
          ii=i-1
          if(i.gt.nnh2)ii=-(n2p1-i)
          y=float(ii)*rn2
          xysq=xsq+y*y
          go to (1020,1020,1030,1030,1040,1050,1060,1020,1070,1020),
     1  itype
 1020     if(xysq.eq.0.0) go to 1080
          call spspol(itype,x,y,xysq,a1(1,i,1),a1(2,i,1))
          go to 1090
 1030     call scont(c2,xysq,a1(1,i,1),a1(2,i,1))
          go to 1090
 1040     call ssvert(c4,xysq,a1(1,i,1),a1(2,i,1))
          go to 1090
 1050     if(xysq.eq.0.0) go to 1090
          call strend(x,y,a1(1,i,1),a1(2,i,1))
          go to 1090
 1060     call sbpass(a1(1,i,1),a1(2,i,1),f1,f2,f3,f4,btan12,btan34,
     1    xysq)
          go to 1090
 1070     call sfvert(c4,xysq,a1(1,i,1),a1(2,i,1))
          go to 1090
 1080     a1(1,1,1)=0.0
          a1(2,1,1)=0.0
 1090     continue
          call swrda(iunita,jr,a1,n22)
          if(jr.eq.1) go to 1010
          jmr=jmr-1
          if(jr.eq.jmr) go to 1110
          a1(2,1,1)=-a1(2,1,1)
          iwr=n2p1
          do 1100 i=2,nnh2
          iwr=iwr-1
          x=a1(2,i,1)
          y=a1(1,i,1)
          a1(2,i,1)=-a1(2,iwr,1)
          a1(1,i,1)=a1(1,iwr,1)
          a1(1,iwr,1)=y
          a1(2,iwr,1)=-x
 1100     continue
          call swrda(iunita,jmr,a1,n22)
          go to 1010
c  a1 array now holds complex coefficients operated on by filter
c  compute desired anomaly = inv. f.f.t. of a1
 1110     isign=1
      write(iw,1115)
 1115 format(' completed applying filter to coefficients')
          call sfftmg(iunita,iunitb,n2,n1,a1,nri,isign,work)
      write(iw,1118)
 1118 format(' inverse transform completed')
          area=1./float(n1*n2)
c  a1 array now holds computed anomaly on datum = z.
c  create standard output file and list output.
          if(iopt1.eq.0) go to 1120
          write(ip,1240)id,z
          f16=float(ny)/16.
          i16=ny/16
          if(abs(f16-float(i16)).gt.0.0001)i16=i16+1
 1120     open(11,status='new',form='unformatted',file=coname)
          write(kr)id,pgm,ny,nx,nz,yo,dy,xo,dx
          if(idval.eq.1)open(14,status='old',form='unformatted',
     1    file='flag.loc')
          if(idval.lt.0)open(14,status='old',form='unformatted',
     1    file='flag.loc')
          dum=0.0
          iwr=0
          do 1210 jr=nxap1,n1a
          iwr=iwr+1
          call srdda(iunita,jr,a1,n22)
          if(idval.eq.0) go to 1170
          read(14)nflag,(jflag(1,j),jflag(2,j),j=1,nflag)
          j=1
c   restore shape of original grid (i.e. insert flagged values).
          do 1160 i=nap1,n2a
          if(i.lt.jflag(1,j)) go to 1130
          if(i.eq.jflag(2,j)) go to 1140
          go to 1150
 1130     a1(1,i,1)=dval
          go to 1160
 1140     j=j+1
          if(j.le.nflag) go to 1150
          j=j-1
          jflag(1,j)=2048
 1150     a1(1,i,1)=a1(1,i,1)*area
 1160     continue
          go to 1190
 1170     do 1180 i=nap1,n2a
 1180     a1(1,i,1)=a1(1,i,1)*area
 1190     write(kr)dum,(a1(1,i,1),i=nap1,n2a)
          if(iopt1.eq.0) go to 1210
          write(ip,1220)iwr
          ih=nap1-1
          do 1200 ii=1,i16
          iout=ih+1
          ih=iout+15
          if(ih.gt.n2a)ih=n2a
 1200     write(ip,1230)(a1(1,i,1),i=iout,ih)
 1210     continue
 1220     format(/,1x,'row ',i3)
 1230     format(3x,16f7.1)
 1240     format(1h1,14a4,//' calculated anomaly on level z=',f7.3,//)
          close(11)
      write(iw,1245)
 1245 format(' additional filtering to original grid?(y or n) ',$)
          read(ir,570)quer
          if(quer.ne.'y') go to 1250
      if(idval.ne.0)close(14)
          ncase=1
          go to 470
 1250     if(idval.ne.1) go to 1255
          if(icoef.eq.0)close(14,disp='delete')
          if(icoef.ne.0)close(14)
      go to 1260
 1255 if(idval.eq.0) go to 1260
      close(14)
 1260     close(10,disp='delete')
          close(12,disp='delete')
      if(icoef.eq.0)close(15,disp='delete')
      if(icoef.ne.0)close(15)
          go to 1290
 1270     write(6,1280)
 1280     format(' #entire first row is flagged values-program aborted')
          close(11)
          close(10)
          if(idval.eq.1)close(14)
 1290     return
          end
c***********************************************************************
      subroutine sfftmg(unita,unitb,n,m,f,l,isign,work)
c     performs 2-dimensional fast fourier transform using disk
c     arguments -
c       unita - unit number of open random access disk file containing
c               one complex data row per record.
c               this file is used for input to fftmg and output
c               of the transform from fftmg.
c       unitb - unit number of open random access work file
c               the same as the file of unita.
c       n     - number of y columns = length of x rows.
c       m     - number of x rows = length of y columns,
c               must be of the form l*2**k (k integer).
c       f     - complex work array for core manipulations.
c       l     - number of x rows which can be held in f.
c       isign - +1 or -1 for forward or reverse transform.
c       work  - work array for in-core transform routine sfourt.
c               work must be complex, of length max(n,l).
c   developed and coded by ray watts--usgs,denver
c   note: randin,randou,inset, and outset are entry points
c   in subroutine sdummy
c***********************************************************************
      integer unita,unitb,output,ndim(2)
      complex f(n,l),twidle,twidlf,w,work(1)
      nfourt=2*n*l
      nwork=2*n
      if(l.gt.n)nwork=2*l
c     **********************************************************
c     *    the number of disk accesses (reads+writes) used by  *
c     *    this program is m*(4*log2(m/l)+3).                  *
c     **********************************************************
c     --------------------------------------------------------
c     check arguments
      if(l*(m/l).ne.m)stop 'l not a factor of m in fftmg'
      ml=m/l
      mt=1
   10 if(mt-ml)20,40,30
   20 mt=2*mt
      go to 10
   30 stop 'm/l not a power of 2 in fftmg'
   40 continue
c     ------------------------------------------------------------
c     -------------------------------------------------------------
c     start processing
c     skip decimation if l .eq. m
      if(l.eq.m)go to 80
c     set starting input and output units
      input=unita
      output=unitb
c     set starting blocksize for decimation in input domain
      iblock=m
c     -------------------------------------------------------
c     decimate this blocksize
c     set up sub-block size
   50 isub=iblock/2
c     set up for random i/0
      call inset(input)
      call outset(output)
c     for each block of this size
      do 60 iblst=1,m,iblock
c     copy even members into first sub-block, odd members into second
      do 60 i=1,isub
      call randin(n,f,2*(i-1)+iblst)
      call randou(n,f,i-1+iblst)
      call randin(n,f,2*i-1+iblst)
   60 call randou(n,f,i-1+iblst+isub)
c     swap input/output functions
      inputt=input
      input=output
      output=inputt
c     next smaller blocksize
      iblock=isub
c     loop if not done
      if(iblock.gt.l)go to 50
c     decimation complete
c     ---------------------------------------------------------
c     ---------------------------------------------------------
c     ensure that data set is back on unita
      if(input.eq.unita)go to 80
      call inset(input)
      call outset(output)
      do 70 i=1,m
      call randin(n,f,i)
   70 call randou(n,f,i)
   80 continue
c     --------------------------------------------------------
c     start transform.
c     set block size
      iblock=l
c     set up for random i/o
      call inset(unita)
      call outset(unita)
c     run through the blocks
      do 100 istart=1,m,iblock
      into=1
c     run through the records in a block
      do 90 irecrd=istart,istart+iblock-1
c     read the record
      call randin(n,f(1,into),irecrd)
c     proceed with next record in block
   90 into=into+1
c     fourier transform in both directions
      ndim(1)=n
      ndim(2)=l
      call sfourt(f,ndim,2,isign,1,work,nfourt,nwork)
c     ******************************************************
c     write the block back onto disk
      ifrom=1
      do 100 irecrd=istart,istart+iblock-1
      call randou(n,f(1,ifrom),irecrd)
  100 ifrom=ifrom+1
c     if l.eq.m, the transform is now complete.
      if(l.eq.m)return
c     done with first-step processing
c     ---------------------------------------------------------
c     finish the processing by pairs of rows, applying twiddle factors
c     set the twiddle generator
      twidlf=cexp(cmplx(0.,3.141593*isign/(iblock)))
c     run through the blocks by pairs
  110 do 140 istart=1,m,iblock*2
c     start the twiddle factor
      twidle=(1.,0.)
c     scan the block pair
      do 130 irecrd=istart,istart+iblock-1
      jrecrd=irecrd+iblock
      call randin(n,f,irecrd)
      call randin(n,f(1,2),jrecrd)
c     combine, using twiddle factor
      do 120 i=1,n
      w=twidle*f(i,2)
      f(i,2)=f(i,1)-w
  120 f(i,1)=f(i,1)+w
c     write back onto disk
      call randou(n,f,irecrd)
      call randou(n,f(1,2),jrecrd)
c     increment the twiddle factor
  130 twidle=twidle*twidlf
c     go on to next block pair
  140 continue
c     increase the block size, change twiddle increment
      iblock=2*iblock
      twidlf=csqrt(twidlf)
      if(isign*aimag(twidlf).lt.0)twidlf=-twidlf
c     check if done
      if(iblock.lt.m)go to 110
c     ----------------------------------------------------------
c     done processing
      end
c***********************************************************************
      subroutine sdummy
c     random access i/o routine
c***********************************************************************
      integer out
      complex f(n),g(n)
c     entry to receive input setup
      entry inset(iin)
      in=iin
      return
c     entry to receive output setup
      entry outset(iout)
      out=iout
      return
c     entry to do input
      entry randin(n,f,nrec)
      read(in'nrec)f
      return
c     entry to do output
      entry randou(n,g,nrec)
      write(out'nrec)g
      return
      end
c***********************************************************************
      subroutine sfourt(data,nn,ndim,isign,iform,work,nfourt,nwork1)
c  note that 'nfourt' and 'nwork1' in subroutine call statement are
c  revised features of 'sfourt'. they are used as dynamic dimensioning
c  for the 'data' and 'work' arrays
c     the cooley-tukey fast fourier transform in usasi basic fortran
c     transform(j1,j2,) = sum(data(i1,i2,)*w1**((i1-1)*(j1-1))
c                                 *w2**((i2-1)*(j2-1))*,),
c     where i1 and j1 run from 1 to nn(1) and w1=exp(isign*2*pi*
c     sqrt(-1)/nn(1)), etc.  there is no limit on the dimensionality
c     (number of subscripts) of the data array.  if an inverse
c     transform (isign=+1) is performed upon an array of transformed
c     (isign=-1) data, the original data will reappear.
c     multiplied by nn(1)*nn(2)*,  the array of input data must be
c     in complex format.  however, if all imaginary parts are zero (i.e.
c     the data are disguised real) running time is cut up to forty per-
c     cent.  (for fastest transform of real data, nn(1) should be even.)
c     the transform values are always complex and are returned in the
c     original array of data, replacing the input data.  the length
c     of each dimension of the data array may be any integer.  the
c     program runs faster on composite integers than on primes, and is
c     particularly fast on numbers rich in factors of two.
c     timing is in fact given by the following formula.  let ntot be the
c     total number of points (real or complex) in the data array, that
c     is, ntot=nn(1)*nn(2)*...  decompose ntot into its prime factors,
c     such as 2**k2 * 3**k3 * 5**k5 * ...  let sum2 be the sum of all
c     the factors of two in ntot, that is, sum2 = 2*k2.  let sumf be
c     the sum of all other factors of ntot, that is, sumf = 3*k3*5*k5*..
c     the time taken by a multidimensional transform on these ntot data
c     is t = t0 + ntot*(t1+t2*sum2+t3*sumf).  on the cdc 3300 (floating
c     point add time = six microseconds), t = 3000 + ntot*(600+40*sum2+
c     175*sumf) microseconds on complex data.
c     implementation of the definition by summation will run in a time
c     proportional to ntot*(nn(1)+nn(2)+...).  for highly composite ntot
c     the savings offered by this program can be dramatic.  a one-dimen-
c     sional array 4000 in length will be transformed in 4000*(600+
c     40*(2+2+2+2+2)+175*(5+5+5)) = 14.5 seconds versus about 4000*
c     4000*175 = 2800 seconds for the straightforward technique.
c     the fast fourier transform places three restrictions upon the
c     data.
c     1.  the number of input data and the number of transform values
c     must be the same.
c     2.  both the input data and the transform values must represent
c     equispaced points in their respective domains of time and
c     frequency.  calling these spacings deltat and deltaf, it must be
c     true that deltaf=2*pi/(nn(i)*deltat).  of course, deltat need not
c     be the same for every dimension.
c     3.  conceptually at least, the input data and the transform output
c     represent single cycles of periodic functions.
c     the calling sequence is--
c     call fourt(data,nn,ndim,isign,iform,work)
c     data is the array used to hold the real and imaginary parts
c     of the data on input and the transform values on output.  it
c     is a multidimensional floating point array, with the real and
c     imaginary parts of a datum stored immediately adjacent in storage
c     (such as fortran iv places them).  normal fortran ordering is
c     expected, the first subscript changing fastest.  the dimensions
c     are given in the integer array nn, of length ndim.  isign is -1
c     to indicate a forward transform (exponential sign is -) and +1
c     for an inverse transform (sign is +).  iform is +1 if the data are
c     complex, 0 if the data are real.  if it is 0, the imaginary
c     parts of the data must be set to zero.  as explained above, the
c     transform values are always complex and are stored in array data.
c     work is an array used for working storage.  it is floating point
c     real, one dimensional of length equal to twice the largest array
c     dimension nn(i) that is not a power of two.  if all nn(i) are
c     powers of two, it is not needed and may be replaced by zero in the
c     calling sequence.  thus, for a one-dimensional array, nn(1) odd,
c     work occupies as many storage locations as data.  if supplied,
c     work must not be the same array as data.  all subscripts of all
c     arrays begin at one.
c     example 1.  three-dimensional forward fourier transform of a
c     complex array dimensioned 32 by 25 by 13 in fortran iv.
c     dimension data(32,25,13),work(50),nn(3)
c     complex data
c     data nn/32,25,13/
c     do 1 i=1,32
c     do 1 j=1,25
c     do 1 k=1,13
c  1  data(i,j,k)=complex value
c     call fourt(data,nn,3,-1,1,work)
c     example 2.  one-dimensional forward transform of a real array of
c     length 64 in fortran ii,
c     dimension data(2,64)
c     do 2 i=1,64
c     data(1,i)=real part
c  2  data(2,i)=0.
c     call fourt(data,64,1,-1,0,0)
c     there are no error messages or error halts in this program.  the
c     program returns immediately if ndim or any nn(i) is less than one.
c     program by norman brenner from the basic program by charles
c     rader,  june 1967.  the idea for the digit reversal was
c     suggested by ralph alter.
c     this is the fastest and most versatile version of the fft known
c     to the author.  a program called four2 is available that also
c     performs the fast fourier transform and is written in usasi basic
c     fortran.  it is about one third as long and restricts the
c     dimensions of the input array (which must be complex) to be powers
c     of two.  another program, called four1, is one tenth as long and
c     runs two thirds as fast on a one-dimensional complex array whose
c     length is a power of two.
c     reference--
c     ieee audio transactions (june 1967), special issue on the fft.
c***********************************************************************
      dimension data(nfourt),nn(2),ifact(32),work(nwork1)
      data twopi/6.2831853071796/,rthlf/0.70710678118655/
      if(ndim-1)1190,10,10
   10 ntot=2
      do 20 idim=1,ndim
      if(nn(idim))1190,1190,20
   20 ntot=ntot*nn(idim)
c     main loop for each dimension
      np1=2
      do 1180 idim=1,ndim
      n=nn(idim)
      np2=np1*n
      if(n-1)1190,1170,30
c     is n a power of two and if not, what are its factors
   30 m=n
      ntwo=np1
      if=1
      idiv=2
   40 iquot=m/idiv
      irem=m-idiv*iquot
      if(iquot-idiv)120,50,50
   50 if(irem)70,60,70
   60 ntwo=ntwo+ntwo
      ifact(if)=idiv
      if=if+1
      m=iquot
      go to 40
   70 idiv=3
      inon2=if
   80 iquot=m/idiv
      irem=m-idiv*iquot
      if(iquot-idiv)140,90,90
   90 if(irem)110,100,110
  100 ifact(if)=idiv
      if=if+1
      m=iquot
      go to 80
  110 idiv=idiv+2
      go to 80
  120 inon2=if
      if(irem)140,130,140
  130 ntwo=ntwo+ntwo
      go to 150
  140 ifact(if)=m
c     separate four cases--
c        1. complex transform or real transform for the 4th, 9th,etc.
c           dimensions.
c        2. real transform for the 2nd or 3rd dimension.  method--
c           transform half the data, supplying the other half by con-
c           jugate symmetry.
c        3. real transform for the 1st dimension, n odd.  method--
c           set the imaginary parts to zero.
c        4. real transform for the 1st dimension, n even.  method--
c           transform a complex array of length n/2 whose real parts
c           are the even numbered real values and whose imaginary parts
c           are the odd numbered real values.  separate and supply
c           the second half by conjugate symmetry.
  150 icase=1
      ifmin=1
      i1rng=np1
      if(idim-4)160,210,210
  160 if(iform)170,170,210
  170 icase=2
      i1rng=np0*(1+nprev/2)
      if(idim-1)180,180,210
  180 icase=3
      i1rng=np1
      if(ntwo-np1)210,210,190
  190 icase=4
      ifmin=2
      ntwo=ntwo/2
      n=n/2
      np2=np2/2
      ntot=ntot/2
      i=1
      do 200 j=1,ntot
      data(j)=data(i)
  200 i=i+2
c     shuffle data by bit reversal, since n=2**k.  as the shuffling
c     can be done by simple interchange, no working array is needed
  210 if(ntwo-np2)290,220,220
  220 np2hf=np2/2
      j=1
      do 280 i2=1,np2,np1
      if(j-i2)230,250,250
  230 i1max=i2+np1-2
      do 240 i1=i2,i1max,2
      do 240 i3=i1,ntot,np2
      j3=j+i3-i2
      tempr=data(i3)
      tempi=data(i3+1)
      data(i3)=data(j3)
      data(i3+1)=data(j3+1)
      data(j3)=tempr
  240 data(j3+1)=tempi
  250 m=np2hf
  260 if(j-m)280,280,270
  270 j=j-m
      m=m/2
      if(m-np1)280,260,260
  280 j=j+m
      go to 370
c     shuffle data by digit reversal for general n
  290 nwork=2*n
      do 360 i1=1,np1,2
      do 360 i3=i1,ntot,np2
      j=i3
      do 350 i=1,nwork,2
      if(icase-3)300,310,300
  300 work(i)=data(j)
      work(i+1)=data(j+1)
      go to 320
  310 work(i)=data(j)
      work(i+1)=0.
  320 ifp2=np2
      if=ifmin
  330 ifp1=ifp2/ifact(if)
      j=j+ifp1
      if(j-i3-ifp2)350,340,340
  340 j=j-ifp2
      ifp2=ifp1
      if=if+1
      if(ifp2-np1)350,350,330
  350 continue
      i2max=i3+np2-np1
      i=1
      do 360 i2=i3,i2max,np1
      data(i2)=work(i)
      data(i2+1)=work(i+1)
  360 i=i+2
c     main loop for factors of two.  perform fourier transforms of
c     length four, with one of length two if needed.  the twiddle factor
c     w=exp(isign*2*pi*sqrt(-1)*m/(4*mmax)).  check for w=isign*sqrt(-1)
c     and repeat for w=w*(1+isign*sqrt(-1))/sqrt(2).
  370 if(ntwo-np1)680,680,380
  380 np1tw=np1+np1
      ipar=ntwo/np1
  390 if(ipar-2)430,410,400
  400 ipar=ipar/4
      go to 390
  410 do 420 i1=1,i1rng,2
      do 420 k1=i1,ntot,np1tw
      k2=k1+np1
      tempr=data(k2)
      tempi=data(k2+1)
      data(k2)=data(k1)-tempr
      data(k2+1)=data(k1+1)-tempi
      data(k1)=data(k1)+tempr
  420 data(k1+1)=data(k1+1)+tempi
  430 mmax=np1
  440 if(mmax-ntwo/2)450,680,680
  450 lmax=max0(np1tw,mmax/2)
      do 670 l=np1,lmax,np1tw
      m=l
      if(mmax-np1)500,500,460
  460 theta=-twopi*float(l)/float(4*mmax)
      if(isign)480,470,470
  470 theta=-theta
  480 wr=cos(theta)
      wi=sin(theta)
  490 w2r=wr*wr-wi*wi
      w2i=2.*wr*wi
      w3r=w2r*wr-w2i*wi
      w3i=w2r*wi+w2i*wr
  500 do 630 i1=1,i1rng,2
      kmin=i1+ipar*m
      if(mmax-np1)510,510,520
  510 kmin=i1
  520 kdif=ipar*mmax
  530 kstep=4*kdif
      if(kstep-ntwo)540,540,630
  540 do 620 k1=kmin,ntot,kstep
      k2=k1+kdif
      k3=k2+kdif
      k4=k3+kdif
      if(mmax-np1)550,550,580
  550 u1r=data(k1)+data(k2)
      u1i=data(k1+1)+data(k2+1)
      u2r=data(k3)+data(k4)
      u2i=data(k3+1)+data(k4+1)
      u3r=data(k1)-data(k2)
      u3i=data(k1+1)-data(k2+1)
      if(isign)560,570,570
  560 u4r=data(k3+1)-data(k4+1)
      u4i=data(k4)-data(k3)
      go to 610
  570 u4r=data(k4+1)-data(k3+1)
      u4i=data(k3)-data(k4)
      go to 610
  580 t2r=w2r*data(k2)-w2i*data(k2+1)
      t2i=w2r*data(k2+1)+w2i*data(k2)
      t3r=wr*data(k3)-wi*data(k3+1)
      t3i=wr*data(k3+1)+wi*data(k3)
      t4r=w3r*data(k4)-w3i*data(k4+1)
      t4i=w3r*data(k4+1)+w3i*data(k4)
      u1r=data(k1)+t2r
      u1i=data(k1+1)+t2i
      u2r=t3r+t4r
      u2i=t3i+t4i
      u3r=data(k1)-t2r
      u3i=data(k1+1)-t2i
      if(isign)590,600,600
  590 u4r=t3i-t4i
      u4i=t4r-t3r
      go to 610
  600 u4r=t4i-t3i
      u4i=t3r-t4r
  610 data(k1)=u1r+u2r
      data(k1+1)=u1i+u2i
      data(k2)=u3r+u4r
      data(k2+1)=u3i+u4i
      data(k3)=u1r-u2r
      data(k3+1)=u1i-u2i
      data(k4)=u3r-u4r
  620 data(k4+1)=u3i-u4i
      kdif=kstep
      kmin=4*(kmin-i1)+i1
      go to 530
  630 continue
      m=m+lmax
      if(m-mmax)640,640,670
  640 if(isign)650,660,660
  650 tempr=wr
      wr=(wr+wi)*rthlf
      wi=(wi-tempr)*rthlf
      go to 490
  660 tempr=wr
      wr=(wr-wi)*rthlf
      wi=(tempr+wi)*rthlf
      go to 490
  670 continue
      ipar=3-ipar
      mmax=mmax+mmax
      go to 440
c     main loop for factors not equal to two.  apply the twiddle factor
c     w=exp(isign*2*pi*sqrt(-1)*(j1-1)*(j2-j1)/(ifp1+ifp2)), then
c     perform a fourier transform of length ifact(if), making use of
c     conjugate symmetries.
  680 if(ntwo-np2)690,900,900
  690 ifp1=ntwo
      if=inon2
      np1hf=np1/2
  700 ifp2=ifact(if)*ifp1
      j1min=np1+1
      if(j1min-ifp1)710,710,760
  710 do 750 j1=j1min,ifp1,np1
      theta=-twopi*float(j1-1)/float(ifp2)
      if(isign)730,720,720
  720 theta=-theta
  730 wstpr=cos(theta)
      wstpi=sin(theta)
      wr=wstpr
      wi=wstpi
      j2min=j1+ifp1
      j2max=j1+ifp2-ifp1
      do 750 j2=j2min,j2max,ifp1
      i1max=j2+i1rng-2
      do 740 i1=j2,i1max,2
      do 740 j3=i1,ntot,ifp2
      tempr=data(j3)
      data(j3)=data(j3)*wr-data(j3+1)*wi
  740 data(j3+1)=tempr*wi+data(j3+1)*wr
      tempr=wr
      wr=wr*wstpr-wi*wstpi
  750 wi=tempr*wstpi+wi*wstpr
  760 theta=-twopi/float(ifact(if))
      if(isign)780,770,770
  770 theta=-theta
  780 wstpr=cos(theta)
      wstpi=sin(theta)
      j2rng=ifp1*(1+ifact(if)/2)
      do 890 i1=1,i1rng,2
      do 890 i3=i1,ntot,np2
      j2max=i3+j2rng-ifp1
      do 880 j2=i3,j2max,ifp1
      j1max=j2+ifp1-np1
      do 850 j1=j2,j1max,np1
      j3max=j1+np2-ifp2
      do 850 j3=j1,j3max,ifp2
      jmin=j3-j2+i3
      jmax=jmin+ifp2-ifp1
      i=1+(j3-i3)/np1hf
      if(j2-i3)790,790,820
  790 sumr=0.
      sumi=0.
      do 810 j=jmin,jmax,ifp1
  800 sumr=sumr+data(j)
  810 sumi=sumi+data(j+1)
      work(i)=sumr
      work(i+1)=sumi
      go to 850
  820 iconj=1+(ifp2-2*j2+i3+j3)/np1hf
      j=jmax
      sumr=data(j)
      sumi=data(j+1)
      oldsr=0.
      oldsi=0.
      j=j-ifp1
  830 tempr=sumr
      tempi=sumi
      sumr=twowr*sumr-oldsr+data(j)
      sumi=twowr*sumi-oldsi+data(j+1)
      oldsr=tempr
      oldsi=tempi
      j=j-ifp1
      if(j-jmin)840,840,830
  840 tempr=wr*sumr-oldsr+data(j)
      tempi=wi*sumi
      work(i)=tempr-tempi
      work(iconj)=tempr+tempi
      tempr=wr*sumi-oldsi+data(j+1)
      tempi=wi*sumr
      work(i+1)=tempr+tempi
      work(iconj+1)=tempr-tempi
  850 continue
      if(j2-i3)860,860,870
  860 wr=wstpr
      wi=wstpi
      go to 880
  870 tempr=wr
      wr=wr*wstpr-wi*wstpi
      wi=tempr*wstpi+wi*wstpr
  880 twowr=wr+wr
      i=1
      i2max=i3+np2-np1
      do 890 i2=i3,i2max,np1
      data(i2)=work(i)
      data(i2+1)=work(i+1)
  890 i=i+2
      if=if+1
      ifp1=ifp2
      if(ifp1-np2)700,900,900
c     complete a real transform in the 1st dimension, n even, by con-
c     jugate symmetries.
  900 go to (1170,1090,1170,910),icase
  910 nhalf=n
      n=n+n
      theta=-twopi/float(n)
      if(isign)930,920,920
  920 theta=-theta
  930 wstpr=cos(theta)
      wstpi=sin(theta)
      wr=wstpr
      wi=wstpi
      imin=3
      jmin=2*nhalf-1
      go to 960
  940 j=jmin
      do 950 i=imin,ntot,np2
      sumr=(data(i)+data(j))/2.
      sumi=(data(i+1)+data(j+1))/2.
      difr=(data(i)-data(j))/2.
      difi=(data(i+1)-data(j+1))/2.
      tempr=wr*sumi+wi*difr
      tempi=wi*sumi-wr*difr
      data(i)=sumr+tempr
      data(i+1)=difi+tempi
      data(j)=sumr-tempr
      data(j+1)=-difi+tempi
  950 j=j+np2
      imin=imin+2
      jmin=jmin-2
      tempr=wr
      wr=wr*wstpr-wi*wstpi
      wi=tempr*wstpi+wi*wstpr
  960 if(imin-jmin)940,970,1000
  970 if(isign)980,1000,1000
  980 do 990 i=imin,ntot,np2
  990 data(i+1)=-data(i+1)
 1000 np2=np2+np2
      ntot=ntot+ntot
      j=ntot+1
      imax=ntot/2+1
 1010 imin=imax-2*nhalf
      i=imin
      go to 1030
 1020 data(j)=data(i)
      data(j+1)=-data(i+1)
 1030 i=i+2
      j=j-2
      if(i-imax)1020,1040,1040
 1040 data(j)=data(imin)-data(imin+1)
      data(j+1)=0.
      if(i-j)1060,1080,1080
 1050 data(j)=data(i)
      data(j+1)=data(i+1)
 1060 i=i-2
      j=j-2
      if(i-imin)1070,1070,1050
 1070 data(j)=data(imin)+data(imin+1)
      data(j+1)=0.
      imax=imin
      go to 1010
 1080 data(1)=data(1)+data(2)
      data(2)=0.
      go to 1170
c     complete a real transform for the 2nd or 3rd dimension by
c     conjugate symmetries.
 1090 if(i1rng-np1)1100,1170,1170
 1100 do 1160 i3=1,ntot,np2
      i2max=i3+np2-np1
      do 1160 i2=i3,i2max,np1
      imin=i2+i1rng
      imax=i2+np1-2
      jmax=2*i3+np1-imin
      if(i2-i3)1120,1120,1110
 1110 jmax=jmax+np2
 1120 if(idim-2)1150,1150,1130
 1130 j=jmax+np0
      do 1140 i=imin,imax,2
      data(i)=data(j)
      data(i+1)=-data(j+1)
 1140 j=j-2
 1150 j=jmax
      do 1160 i=imin,imax,np0
      data(i)=data(j)
      data(i+1)=-data(j+1)
 1160 j=j-np0
c     end of loop on each dimension
 1170 np0=np1
      np1=np2
 1180 nprev=n
 1190 return
      end
c***********************************************************************
      subroutine spspol(itype,x,y,xysq,ar,ai)
c       subroutine 'spspol' operates on input fourier coefficients to
c       obtain the magnetic field reduced to the pole, to make a
c       pseudo-gravity (or pseudo-magnetic) transformation, or convert
c       vertical-component data to total-field data.
c***********************************************************************
      common/basic/c1,el,em,en,bl,bm,bn
c itype   operator
c   1      psdgrv
c   2      redpol
c   8      psdmag
c  10      vertot
c
      xy=sqrt(xysq)
      if(itype-9) 5,50,45
    5 bml=x*bl+y*bm
      eml=x*el+y*em
      ber=bn*en*xy-bml*eml/xy
      bei=(en*bml+bn*eml)
      go to (10,20,50,50,50,50,50,40), itype
   10 c=c1
      go to 30
   20 c=xy
   30 art=ar
      ait=ai
      bes=c/(ber*ber+bei*bei)
      ar=(ber*art+bei*ait)*bes
      ai=(ber*ait-bei*art)*bes
      go to 50
   40 c=1./c1
      art=ar
      ait=ai
      ar=(ber*art-bei*ait)*c
      ai=(bei*art+ber*ait)*c
      go to 50
c new vertot option, code taken from Hildenbrand's vertot.for
   45 ber=en
      bei=(el*x+em*y)/xy
      art=ar
      ait=ai
      ar=(ber*art-bei*ait)
      ai=(bei*art+ber*ait)
c
   50 return
      end
c***********************************************************************
       subroutine scont(c2,xysq,ar,ai)
c  subroutine 'cont' upward or downward continues input fourier
c  coefficients.
c***********************************************************************
      xy=sqrt(xysq)
      e=exp(c2*xy)
      ar=ar*e
      ai=ai*e
      return
      end
c***********************************************************************
       subroutine sfvert(c4,xysq,ar,ai)
c  subroutine 'fvert' operates on input fourier coefficients
c  to obtain the 1st vertical derivative of the input data.
c***********************************************************************
       xsqrt=sqrt(xysq)
       xyp=xsqrt*c4
       ar=ar*xyp
       ai=ai*xyp
       return
       end
c***********************************************************************
       subroutine ssvert(c4,xysq,ar,ai)
c  subroutine 'svert' operates on input fourier coefficients
c  to obtain the 2nd vertical derivative of the input data.
c***********************************************************************
       xyp=xysq*c4
       ar=ar*xyp
       ai=ai*xyp
       return
       end
c***********************************************************************
      subroutine sbpass(a1,a2,f1,f2,f3,f4,btan12,btan34,xysq)
c   subroutine 'sbpass' eliminates wavelengths
c   lying outside the range of w1,w2,w3 & w4.
c***********************************************************************
      rad=sqrt(xysq)
      if(rad.lt.f1) go to 10
      if(rad.gt.f4) go to 10
      go to 20
   10 a1=0.0
      a2=0.0
      go to 40
   20 if(rad.ge.f2.and.rad.le.f3) go to 40
      if(rad.gt.f2) go to 30
      f=(rad-f1)*btan12
      a1=f*a1
      a2=f*a2
      go to 40
   30 f=(f4-rad)*btan34
      a1=f*a1
      a2=f*a2
   40 return
      end
c***********************************************************************
      subroutine strend(x,y,ar,ai)
c  subroutine 'trend' passes or rejects trends striking
c  between angles thet1 and thet2.
c***********************************************************************
      common/tr/prad,p,q,th1p,th2p
      if(x.eq.0.0) go to 50
      if(y.eq.0.0) go to 60
      thet=atan(x/y)*prad
      if(thet.lt.0.0)thet=180.+thet
   10 if(th2p.gt.th1p) go to 40
      if(thet.gt.th2p.and.thet.lt.th1p) go to 30
   20 ar=ar*q
      ai=ai*q
      return
   30 ar=ar*p
      ai=ai*p
      return
   40 if(thet.gt.th1p.and.thet.lt.th2p) go to 30
      go to 20
   50 thet=0.
      if(y.lt.0.0)thet=180.
      go to 10
   60 thet=90.
      go to 10
      end
c***********************************************************************
      subroutine srmean(nx,np,a1)
c  subroutine 'srmean' removes the mean from the input grid
c  employing only boundary values.
c***********************************************************************
      dimension a1(np,3),title(14)
      character*50 fname,coname,fileo
      real mean
      common/main/id(14),pgm(2),nz,yo,xo,dx,dy,iw,kr,ny,nadd,ir
      common/parm1/fname,coname,iopt1,iopt2,idval,icoef,ddx,ddy
      data dval/'ffff7fff'x/
      ko=10
      mean=0.0
      k=0
      if(idval.eq.0) go to 150
c  idval>0: find and sum boundary values.
      call srdbin(kr,a1,ny)
      do 10 j=1,ny
      if(a1(j,1).eq.dval) go to 10
      mean=mean+a1(j,1)
      k=k+1
   10 continue
      call srdbin(kr,a1(1,2),ny)
      do 130 i=3,nx
      call srdbin(kr,a1(1,3),ny)
      iq=-1
      if(a1(1,2).eq.dval) go to 20
      k=k+1
      mean=mean+a1(1,2)
      if(a1(1,1).lt.dval.and.a1(1,3).lt.dval)iq=0
   20 do 90 j=2,ny-1
      if(a1(j,2).eq.dval) go to 60
      if(iq)30,40,50
   30 mean=mean+a1(j,2)
      k=k+1
      iq=0
      if(a1(j,1).eq.dval.or.a1(j,3).eq.dval)iq=-1
      go to 90
   40 iq=1
      if(a1(j,1).lt.dval.and.a1(j,3).lt.dval) go to 90
      iq=-1
      mean=mean+a1(j,2)
      k=k+1
      go to 90
   50 iq=1
      if(a1(j,1).lt.dval.and.a1(j,3).lt.dval) go to 90
      mean=mean+a1(j,2)+a1(j-1,2)
      k=k+1
      iq=-1
      go to 90
   60 if(iq)80,80,70
   70 mean=mean+a1(j-1,2)
      k=k+1
   80 iq=-1
   90 continue
      if(a1(ny,2).eq.dval) go to 100
      k=k+1
      mean=mean+a1(ny,2)
      if(a1(ny,1).lt.dval.and.a1(ny,3).lt.dval) go to 110
  100 if(iq.ne.1) go to 110
      mean=mean+a1(ny-1,2)
      k=k+1
  110 do 120 j=1,ny
      a1(j,1)=a1(j,2)
  120 a1(j,2)=a1(j,3)
  130 continue
      do 140 j=1,ny
      if(a1(j,2).eq.dval) go to 140
      mean=mean+a1(j,2)
      k=k+1
  140 continue
      go to 240
c  idval=0: read grid and check boundary values for flagged values.
  150 i=0
  160 i=i+1
      call srdbin(kr,a1,ny)
      if(i.gt.1.and.i.lt.nx) go to 190
      do 170 j=1,ny
      if(a1(j,1).eq.dval) go to 180
  170 mean=mean+a1(j,1)
      if(i-nx)160,230,160
  180 write(iw,220)i,j,a1(j,1)
      close(11)
      stop
  190 if(a1(1,1).eq.dval.or.a1(ny,1).eq.dval) go to 200
      mean=mean+a1(1,1)+a1(ny,1)
      go to 160
  200 write(iw,210)i,a1(1,1),a1(ny,1)
  210 format(/,' #found flagged value: row = ',i3,2x,'col = 1 or ny',
     12x,' value = ',2e10.4,' may be more?')
  220 format(/,' #found flagged value: row = ',i3,2x,' col = ',i3,
     12x,'value = ',e10.4' may be more?')
      close(11)
      stop
  230 k=(nx+ny-2)*2
c  compute the mean.
  240 c=mean/float(k)
      ip=iopt1
      if(ip.eq.0)ip=iw
      write(ip,250)c,k
  250 format(/,' mean value subtracted = ',e16.8,5x,i5,' values used',/)
      close(11)
c  read input array and write array with the mean subtacted.
      open(11,status='old',form='unformatted',file=fname,readonly)
      read(kr)title,pgm,ny,nx,nz,yop,dyp,xop,dxp
      fileo='fftfil.grd'
      open(10,status='new',form='unformatted',file=fileo)
      write(ko)title,pgm,ny,nx,nz,yo,dy,xo,dx
      do 270 i=1,nx
      call srdbin(kr,a1,ny)
      do 260 j=1,ny
  260 a1(j,1)=a1(j,1)-c
  270 call swrbin(ko,a1,ny)
      close(11)
      close(10)
      return
      end
c***********************************************************************
      subroutine srdbin(no,dat,n)
c       subroutine 'srdbin' reads standard grids (binary).
c***********************************************************************
      dimension dat(n)
      read(no)dum,dat
      return
      end
c***********************************************************************
      subroutine swrbin(no,dat,n)
c       subroutine 'swrbin' writes standard grids (binary).
c***********************************************************************
      dimension dat(n)
      data dum/0.0/
      write(no)dum,dat
      return
      end
c***********************************************************************
       subroutine srdda(no,ipos,dat,n)
c       subroutine 'srrda' reads keyed sequential files.
c***********************************************************************
      dimension dat(n)
      read(no'ipos)dat
      return
      end
c***********************************************************************
      subroutine swrda(no,ipos,dat,n)
c       subroutine 'swrda' writes keyed sequential files.
c***********************************************************************
      dimension dat(n)
      write(no'ipos)dat
      return
      end
