c  gi3.for
c
c  Iterative 3-D inversion of gravity data based on method of
c    Cordell and Henderson, 1968, Geophysics v. 33, p. 596-601.
c
c  Driver for iterative 3-d gravity subroutine 'GRVINV'. Written 
c    by Cordell ca. 1977 and subsequently modified by Cordell,
c    R.N. Godson, et al.  This version by Cordell, Oct 1988.
c
      implicit double precision (d)
      dimension g(50,50),dgcalc(50,50),t(50,50),zs(50,50),
     &da1(52,52),da2(52,52),id(14),pgm(2),bigst(10),ib(10),
     &jb(10),rmsn(10)
      character*50 cmd_file_name
      character*50 ref_surface_array_file_name
      character*50 gravity_data_file_name
      character*50 model_array_file_name
      character*50 output_model_file_name
      character*50 derived_surface_file_name
      character*50 gravity_residual_file_name
      character*50 calculated_gravity_file_name
      character*1 answer
      equivalence (imax,ncol),(jmax,nrow),(iopt3,mode)
      namelist /parms/ w,zs1,rho,nmax,nstar,eps,mode,iopt1,iopt5       
      imax=0
      jmax=0
      rho=0.0
      nstar=0
      nmax=0
      eps=0.0
      mode=0
      iopt2=1
      iopt4=0
      iopt5=0
      iopt6=0
      zs1=0
c  read command and data files
	print*
      write(6,1)
1     format(1x,'Enter command file name:',/,' *',$)
      read(5,2) cmd_file_name
2     format(a50)
       open(10,file=cmd_file_name,status='old')
      read (10,parms,end=99)
      close(10)
      write(6,40)zs1,rho,mode,iopt1
40    format(1x,'Following are the basic input parameters:',/,1x,'zs1 ='
     & ,e12.6,10x,'rho =',e12.6,10x,/,1x,'mode =',i2,10x,
     & 'iopt1 =',i2,/)
      if(nmax.eq.0)nmax=5
      write(6,41) nmax,nstar,eps,iopt5
41    format(1x,'Following are input parameters which may have been
     & assigned by default:',/,1x,'nmax =',i2,10x,'nstar =',i2,
     & 10x,'eps =',e12.6,10x,'iopt5 =',i2,/,1x,'Want to continue ?',
     & 1x,$)
      read(5,42) answer
42    format(a1)
      if(answer.eq.'y'.or.answer.eq.'Y') go to 44
55    write(6,43)
43    format(1x,'Return to command level to revise parameters.')
      go to 999
44    continue
      if(mode.eq.2) go to 56
      if(mode.eq.1.or.mode.eq.0) go to 50
      iferr=6
      go to 90
50    write(6,51)
51    format(/1x,'Enter gravity data file name:',/' *',$)
      read(5,2) gravity_data_file_name
      open(10,file=gravity_data_file_name,status='old',
     & form='unformatted')
      read(10)id,pgm,ncol,nrow,nz,x0,rdx,y0,rdy
	if(ncol.gt.50.or.nrow.gt.50) then
	write(6,52)
52	format(' Error: ncol or nrow > 50.')
	go to 999
	endif
	if(rdx.ne.rdy) then
	print*,'Error: dx must equal dy.'
	go to 999
	endif
	if(nz.ne.1) then
	print*,'Error: nz must equal 1.'
	go to 999
	endif
	w=rdx
      do 61 j=1,nrow
      read(10) rdum,(g(i,j),i=1,ncol)
	do 662 k=1,ncol
	if(g(k,j).ge.1.e+30) then
	print*,'Can''t handle dvals.'
	go to 999
	endif
662	continue
61	continue
      close(10)
56    if(mode.ne.1.and.mode.ne.2) go to 57
53    write(6,54)
54    format(1x,'Enter model array file name:',/' *',$)
      read(5,2) model_array_file_name
       open(10,file=model_array_file_name,status='old',
     & form='unformatted')
      read(10)id,pgm,mcol,mrow,mz,xx0,xdx,yy0,ydy
	if(mode.eq.2)then
	  rdx=xdx
	  rdy=ydy
	  ncol=mcol
	  nrow=mrow
	  nz=mz
	  x0=xx0
	  y0=yy0
	  if(ncol.gt.50.or.nrow.gt.50) then
	    write(6,52)
	    go to 999
	  endif
	  if(rdx.ne.rdy) then
	    print*,'Error: dx must equal dy.'
	    go to 999
	  endif
	  if(nz.ne.1) then
	    print*,'Error: nz must equal 1.'
	    go to 999
	  endif
	  w=rdx
	  go to 661
	endif
	if(mcol.ne.ncol.or.mrow.ne.nrow.or.mz.ne.nz) go to 660
	if(xdx.ne.rdx.or.ydy.ne.rdy) go to 660
	slop=.00001
	if(xx0.gt.x0+slop.or.xx0.lt.x0-slop) go to 660
	if(yy0.gt.y0+slop.or.yy0.lt.y0-slop) go to 660
	go to 661
660	print*,'Grid specifications don''t match'
	go to 999
661   do 62 j=1,nrow
62    read(10) rdum,(t(i,j),i=1,ncol)
      close(10)
57    if(iopt5.eq.0) go to 60
      write(6,58)
58    format(1x,'Enter reference surface array file name:',/' *',$)
      read(5,2) ref_surface_array_file_name
      open(10,file=ref_surface_array_file_name,status='old',
     & form='unformatted')
      read(10)id,pgm,mcol,mrow,mz,xx0,xdx,yy0,ydy
	if(mcol.ne.ncol.or.mrow.ne.nrow.or.mz.ne.nz) go to 664
	if(xdx.ne.rdx.or.ydy.ne.rdy) go to 664
	slop=.00001
	if(xx0.gt.x0+slop.or.xx0.lt.x0-slop) go to 664
	if(yy0.gt.y0+slop.or.yy0.lt.y0-slop) go to 664
	go to 663
664	print*,'Grid specifications don''t match'
	go to 999
663    do 63 j=1,nrow
63    read(10) rdum,(zs(i,j),i=1,ncol)
      close(10)
60    continue
      write(6,64)
64    format(1x,'There may be a long pause;',
     & ' wait for Computation completed')
c  call the GRVINV  subroutine
      call GRVINV(imax,jmax,zs1,rho,w,nmax,nstar,eps,iopt1,iopt2,mode,
     & iopt5,g,dgcalc,t,zs,da1,da2,bigst,ib,jb,rmsn,iferr)
90    if(iferr.eq.0) go to 20
      if(iferr.eq.1) write(6,10)
10    format(' Error: the reference plane or surface is not
     & properly defined')
      if(iferr.eq.3) write(6,12)
12    format(' Error in definition of grid width')
      if(iferr.eq.4) write(6,13)
13    format(' Error: gravity and density contrast',/
     & ' must have the same algebraic sign')
      if(iferr.eq.5) write(6,14)
14    format(' Error in definition of units')
      if(iferr.eq.6) write(6,15)
15    format(' Error: parameter mode must be 0,1,2, or undefined
     & in parms list')
	if(iferr.eq.7) write(6,116)
116	format(' Error:  base or middle of prisms cannot be
     $ referenced to 0 depth.'/'  Check zs1 and iopt1')
      go to 999
20    continue
c  output
30    write(6,31)
31    format(1x,'Computation completed.')
      if(iopt3.eq.2)go to 32
      write(6,21) nmax
21    format(1x,'The program has completed ',i3,' iterations.')
      do 22 i=1,nmax
22    write(6,23)i,bigst(i),ib(i),jb(i),rmsn(i)
23    format(1x,'At iteration ',i3, ' the largest error is ',
     & e12.4,' at point '
     & ,i3,',',i2,' ;'/1x,'    rms error =',e12.4,'.')
32    write(6,100)
100   format(1x,'Want to see some profiles?')
      read(5,42) answer
      if(answer.ne.'y'.and.answer.ne.'Y')go to 120
101   write(6,102)
102   format(1x,'Row or column?')
      read(5,42)answer
      if(answer.eq.'r'.or.answer.eq.'R')go to 103
      if(answer.eq.'c'.or.answer.eq.'C')go to 108
      go to 101
103   write(6,104)
104   format(1x,'Enter row number.')
      read(5,*) jrow
      write(6,140)
140   format(5x,'Gravity   Calc Grav   Thickness')
      do 106 i=1,ncol
106   write(6,107)i,g(i,jrow),dgcalc(i,jrow),t(i,jrow)
107   format(1x,i2,3f10.3)
      go to 111
108   write(6,109)
109   format(1x,'Enter column number.')
      read(5,*)icol
      write(6,140)
      do 110 j=1,nrow
110   write(6,107)j,g(icol,j),dgcalc(icol,j),t(icol,j)
111   write(6,112)
112   format(1x,'More profiles?')
      read(5,42) answer
      if(answer.eq.'y'.or.answer.eq.'Y')go to 101
120   continue
c  output standard files
caution: note that in this section array t is redefined.
      if(iopt3.eq.2)go to 130
121   write(6,123)
123   format(1x,'Enter output model file name (or hit return)')
      read(5,2)output_model_file_name
	if(output_model_file_name.eq.' ') go to 126
      write(6,127)
127   format(1x,'Enter title (record id of standard file).')
      read(5,128)(id(i),i=1,14)
128   format(14a4)
      open(11,file=output_model_file_name,status='new',
     & form='unformatted')
      write(11)id,pgm,ncol,nrow,nz,x0,rdx,y0,rdy
      do 124 j=1,nrow
      y=j
124   write(11)y,(t(i,j),i=1,ncol)
      close(11)
      write(6,125)output_model_file_name
125   format(1x,'Segment ',a50,/' contains derived model',
     & ' in standard fmt.')
126   continue
141   write(6,143)
143   format(1x,'Enter derived surface file name (or hit return).')
      read(5,2)derived_surface_file_name
	if(derived_surface_file_name.eq.' ') go to 150
      write(6,127)
      read(5,128)(id(i),i=1,14)
      open(11,file=derived_surface_file_name,status='new',
     & form='unformatted')
      write(11)id,pgm,ncol,nrow,nz,x0,rdx,y0,rdy
      do 144 j=1,nrow
      do 146 i=1,ncol
146   t(i,j)=sngl(da2(i,j))
      y=j
144   write(11)y,(t(k,j),k=1,ncol)
      close(11)
      write(6,145)derived_surface_file_name
145   format(1x,'Segment ',a50,/' contains derived surface',
     & ' standard file.')
150   continue
130   write(6,132)
132   format(1x,'Enter calculated gravity file name (or hit return).')
      read(5,2)calculated_gravity_file_name
	if(calculated_gravity_file_name.eq.' ') go to 135
      write(6,127)
      read(5,128)(id(i),i=1,14)
      open(11,file=calculated_gravity_file_name,status='new',
     & form='unformatted')
      write(11)id,pgm,ncol,nrow,nz,x0,rdx,y0,rdy
      do 136 j=1,nrow
      do 133 i=1,ncol
133   t(i,j)=sngl(dgcalc(i,j))
      y=j
136   write(11)y,(t(k,j),k=1,ncol)
      close(11)
      write(6,134)calculated_gravity_file_name
134   format(1x,'Segment ',a50,/' contains calculated gravity array in
     & standard file format.')
135   continue
151   write(6,153)
153   format(1x,'Enter gravity residual file name (or hit return).')
      read(5,2)gravity_residual_file_name
	if(gravity_residual_file_name.eq.' ') go to 160
      write(6,127)
      read(5,128)(id(i),i=1,14)
      open(11,file=gravity_residual_file_name,status='new',
     & form='unformatted')
      write(11)id,pgm,ncol,nrow,nz,x0,rdx,y0,rdy
      do 155 j=1,nrow
      do 154 i=1,ncol
154   t(i,j)=g(i,j)-sngl(dgcalc(i,j))
      y=j
155   write(11)y,(t(k,j),k=1,ncol)
      close(11)
      write(6,156)gravity_residual_file_name
156   format(1x,'Segment ',a50,/' contains residual',
     & ' gravity standard file.')
160   continue
      go to 999
99    continue
      write(6,900)
900   format(' Abnormal termination due to problem',
     & ' in command file.')
999   stop
      end
      subroutine GRVINV(imax,jmax,zs1,rho,w,nmax,nstar,eps,iopt1,iopt2,
     & iopt3,iopt5,g,dgcalc,t,zs,da1,da2,bigst,ib,jb,rmsn,iferr)
c  ITERATIVE THREE-DIMENSIONAL SOLUTION OF GRAVITY ANOMALY DATA.
c  LIN CORDELL, U.S. GEOLOGICAL SURVEY, WASHINGTON D.C., NOV. 1968.
c  RECAST AS A SUBROUTINE AUG 1977.
          implicit double precision (d)
	dimension g(50,1),dgcalc(50,1),t(50,1),zs(50,1),
     & da1(52,1),da2(52,1)
          dimension bigst(1),ib(1),jb(1),rmsn(1)
	dpi=3.1415926536d0
	da=w/2.0d0
	total=imax*jmax
	drsq=4.0d0/dpi
c  DEFAULT OPTIONS AND BOUNDS OF INPUT DATA ARRAY(S)
          iferr=0
	gmin=1000000.0
	gmax=-gmin
	do 7 j=1,jmax
	do 6 i=1,imax
	if(iopt3.eq.2) g(i,j)=1.0
	if(iopt5.eq.0) zs(i,j)=zs1
	if(g(i,j).gt.gmax) gmax=g(i,j)
	if(g(i,j).lt.gmin) gmin=g(i,j)
	z=zs(i,j)
	if(z.lt.0.0) iferr=1
	if(iopt3.eq.0) go to 6
	t1=t(i,j)
	if(t1.lt.0.0) iferr=1
	if(iopt1.eq.2.and.(z-t1).lt.0.0) iferr=1
	if(iopt1.eq.3.and.(z-t1/2.0).lt.0.0) iferr = 1
6	continue
7	continue
	if(nmax.le.0.or.nmax.gt.50) nmax=5
      if(iopt3.eq.2) nmax=1
8	if(nstar.le.0.or.nstar.gt.nmax) nstar=nmax+1
c   TEST FOR PARAMETER ERRORS
	if(imax.lt.1.or.imax.gt.50.or.jmax.lt.1.or.jmax.gt.50) iferr=2
	if(w.le.0.0) iferr=3
	if(gmax.gt.0.0.and.gmin.lt.0.0) iferr=4
	if(gmin.lt.0.0.and.rho.ge.0.0) iferr=4
	if(gmin.ge.0.0.and.rho.le.0.0.and.iopt3.ne.2) iferr=4
	eps=abs(eps)
          if(iopt1.ne.1.and.iopt1.ne.2.and.iopt1.ne.3)iferr=1
17	if(iopt2.eq.1) go to 21
	if(iopt2.eq.2) go to 23
	if(iopt2.eq.3) go to 25
          iferr=5
21	c=1.0
	go to 27
23        c=1.60935
	go to 27
25	c=0.304801
27	continue
	dcnst=6.673d0*c*rho*da
	dcnsta=4.0d0*dcnst
          if(iopt3.ne.0.and.iopt3.ne.1.and.iopt3.ne.2) iferr=6
          if(iopt5.ne.0.and.iopt5.ne.1) iferr=1
          if(zs1.lt.0.0) iferr=1
          if(iopt3.eq.2) go to 95
100       if(nstar.eq.1) go to 95
          namax=nstar-1
          if(iferr.ne.0) go to 999
95	continue
c  BEGIN COMPUTATIONS
c  FIRST APPROXIMATION T(I,J),BY BOUGUER SLAB T=G/(2*PI*GAMMA*RHO)
	n=1
	if(iopt3.ne.0) go to 400
	bougr=c*rho*41.9277
	do 35 j=1,jmax
	do 34 i=1,imax
34	t(i,j)=g(i,j)/bougr
35	continue
c  COMPUTATIONS.  NOTE -- (I1,J1)=BODY POINT,(I3,J3)=FIELD POINT,
c   (I2,J2)=FIELD POINT  IN TRANSFORMED FRAME. DA1 = GRAVITY ARRAY,
c   DA2 = STORAGE ARRAY (FOR EXACT FORMULA).
400	continue
c  CLEAR DGCALC ARRAY
	do 403 j3=1,jmax
	do 402 i3=1,imax
402	dgcalc(i3,j3)=0.0
403	continue
	do 415 j1=1,jmax
	l1=j1+1
	j2max1=jmax-j1+1
	if(j2max1.lt.j1) j2max1=j1
	do 414 i1=1,imax
	l2=i1+1
	i2max=imax-i1+1
	if(i2max.lt.i1) i2max=i1
	j2max=j2max1
	if(i2max.ge.j2max) go to 416
	j2max=i2max
	i2max=j2max1
416	continue
c  COMPUTE DG OF T(I1,J1), IN (I2,J2) FRAME
c  DEFINE PRISM TOP AND BASE
	tt=t(i1,j1)
	zz=zs(i1,j1)
	if(iopt1-2) 418,419,420
418	dz1=zz
	dz2=dz1+tt
	go to 421
419	if(zz-tt) 430,431,431
430	t(i1,j1)=zz
	tt=zz
431	dz1=zz-tt
	dz2=zz
	go to 421
420	th=tt/2.0
	if(zz-th) 432,433,433
432	t(i1,j1)=2.0*zz
	th=zz
433	dz1=zz-th
	dz2=zz+th
421	dz1=dz1/da
	dz2=dz2/da
	if(dz2-dz1) 414,414,434
434	dz1sq=dz1*dz1
	dz2sq=dz2*dz2
c  SELECT EXACT OR APPROXIMATE PRISM FORMULAE
	if(nstar-n) 435,435,450
c  GRAVITY OF N-TH MODEL BY EXACT FORMULA
c  AXIAL POINT (1,1)
435	dr1=dsqrt(1.0+dz1sq)
	dr2=dsqrt(1.0+dz2sq)
	db1=1.0+dsqrt(2.0+dz1sq)
	db2=1.0+dsqrt(2.0+dz2sq)
	da2(1,1)=dcnst*(2.0*dlog((db1*dr2)/(db2*dr1))+
     & dz2*(dasin((dz2sq+db2)/(db2*dr2))-dasin(dz2/dr2)))
	if(dz1)438,438,437
437	da2(1,1)=da2(1,1)+dcnst*dz1*(dasin(dz1/dr1)-
     & dasin((dz1sq+db1)/(db1*dr1)))
438	da1(1,1)=da2(1,1)*4.0
c  LINE (I2,1), I2=2,3...I2MAX
	do 401 i2=2,i2max
	dx=2*i2-1
	dxsq=dx*dx
	dp=1.0+dxsq
	dq1=dsqrt(dp+dz1sq)
	dq2=dsqrt(dp+dz2sq)
	dr1=dsqrt(dxsq+dz1sq)
	dr2=dsqrt(dxsq+dz2sq)
	ds1=dsqrt(1.0+dz1sq)
	ds2=dsqrt(1.0+dz2sq)
	db1=dx+dq1
	db2=dx+dq2
	dc1=1.0+dq1
	dc2=1.0+dq2
	da2(i2,1)=dcnst*(dx*dlog((dc1*dr2)/(dc2*dr1))+dlog((db1*ds2)/
     & (db2*ds1))+dz2*(dasin((dc2+dz2sq)/(dc2*ds2))-dasin(dz2/dr2)))
	if(dz1) 440,440,439
439	da2(i2,1)=da2(i2,1)+dcnst*dz1*(dasin(dz1/dr1)-
     & dasin((dc1+dz1sq)/(dc1*ds1)))
440	da1(i2,1)=2.0*(da2(i2,1)-da2(i2-1,1))
	da1(1,i2)=da1(i2,1)
401	continue
c  DIAGONAL (K,K), K=2,3...J2MAX
	j2=1
422	j2=j2+1
          dy=2*j2-1
          dysq=dy*dy
          dp=2.0d0*dysq
          dq1=dsqrt(dp+dz1sq)
          dq2=dsqrt(dp+dz2sq)
          ds1=dsqrt(dysq+dz1sq)
          ds2=dsqrt(dysq+dz2sq)
          dc1=dy+dq1
          dc2=dy+dq2
          da2(j2,j2)=dcnst*(2.0d0*dy*dlog((dc1*ds2)/(dc2*ds1))+dz2
     &    *(dasin((dz2sq+dy*dc2)/(dc2*ds2))-dasin(dz2/ds2)))
          if(dz1) 442,442,441
441       da2(j2,j2)=da2(j2,j2)+dcnst*dz1*(dasin(dz1/ds1)-dasin((
     &    dz1sq+dy*dc1)/(dc1*ds1)))
442	da1(j2,j2)=da2(j2,j2)-2.0*da2(j2,j2-1)+da2(j2-1,j2-1)
c  WEDGE AND RECTANGLE, (I2,J2), J2=2,3...J2MAX; I2=3,4...I2MAX
	i2=j2
417	i2=i2+1
	dx=2*i2-1
	dxsq=dx*dx
	dp=dxsq+dysq
	dq1=dsqrt(dp+dz1sq)
	dq2=dsqrt(dp+dz2sq)
	dr1=dsqrt(dxsq+dz1sq)
	dr2=dsqrt(dxsq+dz2sq)
	db1=dx+dq1
	db2=dx+dq2
	dc1=dy+dq1
	dc2=dy+dq2
      da2(i2,j2)=dcnst*(dx*dlog((dc1*dr2)/(dc2*dr1))+dy*dlog((db1*ds2)/
     &(db2*ds1))+dz2*(dasin((dz2sq+dy*dc2)/
     & (dc2*ds2))-dasin(dz2/dr2)))
	if(dz1) 444,444,443
443	da2(i2,j2)=da2(i2,j2)+dcnst*dz1*(dasin(dz1/dr1)-dasin((dz1sq+
     & dy*dc1)/(dc1*ds1)))
444	da1(i2,j2)=da2(i2,j2)-da2(i2,j2-1)-da2(i2-1,j2)+da2(i2-1,j2-1)
	da1(j2,i2)=da1(i2,j2)
	if(i2-i2max) 417,436,436
436	if(j2-j2max) 422,407,407
c  GRAVITY OF N-TH MODEL BY APPROXIMATE FORMULAE
450	j2=1
	da1(1,1)=(2.0*dpi*dcnst)*(dsqrt(drsq+dz1sq)-dsqrt(drsq+dz2sq)
     & +dz2-dz1)
	do 451 i2=2,i2max
	dp=4*(i2-1)*(i2-1)
	da1(i2, 1)=dcnsta*((1.0/dsqrt(dp+dz1sq))-(1.0/dsqrt(dp+dz2sq)))
451	da1(1,i2)=da1(i2,1)
452	j2=j2+1
	dy=2*(j2-1)
	dp=2.0*dy*dy
	i2=j2
	da1(i2,j2)=dcnsta*((1.0/dsqrt(dp+dz1sq))-(1.0/dsqrt(dp+dz2sq)))
453	i2=i2+1
	dx=2*(i2-1)
	dp=dx*dx+dy*dy
	da1(i2,j2)=dcnsta*((1.0/dsqrt(dp+dz1sq))-(1.0/dsqrt(dp+dz2sq)))
	da1(j2,i2)=da1(i2,j2)
	if(i2-i2max) 453,454,454
454	if(j2-j2max) 452,407,407
c  TRANSFORM G(I2,J2) INTO G(I3,J3), IN FOUR SECTORS
407	do 410 j3=1,j1
	j2=j1-j3+1
	do 408 i3=1,i1
408	dgcalc(i3,j3)=dgcalc(i3,j3)+da1(i1-i3+1,j2)
	do 409 i3=l2,imax
409	dgcalc(i3,j3)=dgcalc(i3,j3)+da1(i3-i1+1,j2)
410	continue
	do 413 j3=l1,jmax
	j2=j3-j1+1
	do 411 i3=1,i1
411	dgcalc(i3,j3)=dgcalc(i3,j3)+da1(i1-i3+1,j2)
	do 412 i3=l2,imax
412	dgcalc(i3,j3)=dgcalc(i3,j3)+da1(i3-i1+1,j2)
413	continue
414	continue
415	continue
c  COMPLETE DGCALC, TEST N FOR OUTPUT
c  COMPUTE LARGEST AND RMS ERROR
	rms=0.0
	big=0.0
	do 42 j=1,jmax
	do 41 i=1,imax
	da1(i,j)=g(i,j)-dgcalc(i,j)
	adiff=dabs(da1(i,j))
	if(adiff.lt.big) go to 40
	big=adiff
	ibig=i
	jbig=j
40	rms=rms+adiff*adiff
41	continue
42	continue
          rmsn(n)=sqrt(rms/total)
          bigst(n)=big
          ib(n)=ibig
          jb(n)=jbig
          if(iopt3.eq.2) go to 50
          if(big.lt.eps) go to 50
          if(n.eq.nmax) go to 50
c  REVISE MODEL AND RECYCLE
	n=n+1
	do 45 j=1,jmax
	do 44 i=1,imax
44	t(i,j)=(t(i,j)*g(i,j))/dgcalc(i,j)
45	continue
	go to 400
c  OUTPUT
50        continue
	do 58 j=1,jmax
	do 57 i=1,imax
	da2(i,j)=zs(i,j)+t(i,j)
	if(iopt1.eq.2) da2(i,j)=zs(i,j)-t(i,j)
	if(iopt1.eq.3) da2(i,j)=zs(i,j)-t(i,j)/2.0d0
57	continue
58	continue
999	return
	end
